/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ANIMATION_H
#define ANIMATION_H

#include "utils.h"
#include "interface_animationObserver.h"
#include "stampSet.h"
#include "resourcePool.h"
#include "interface_cloneable.h"
#include "animationSettings.h"


/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/

class Animation : public Interface_Cloneable{
    public:
        Animation (ResourcePool* resources, AnimationType index, string defFile);
		Animation (AnimationSettings settings);
	
		virtual Interface_Cloneable* clone() const;
    
        virtual ~Animation();

        void setInterruptibleBy (InterruptFlag interruptibleBy){ this->interruptibleBy = interruptibleBy; }
        bool isInterruptibleBy (InterruptFlag interrupter) const{ return (this->interruptibleBy & interrupter) != 0; }
        void setTimeLength (unsigned int timeLength){ this->timeLength = timeLength; }
        unsigned int getTimeLength() const{ return this->timeLength; }
		unsigned int getTimePassed() const{ return this->timePassed; }
        void setRepeating (bool repeating){ this->repeating = repeating; }
        bool getRepeating() const{ return this->repeating; }
        unsigned int getFrameCount() const{ return this->frames.size(); }
		int getCurrentStampIndex() const;
        int getCurrentFrame() const;
		int getAnimationTimeRemaining() const;

        bool play(){
			notifiedObservers = false;
            return true;
        }

        bool reset(){
			notifiedObservers = false;
            this->timePassed = 0;
			//cout << "Animation::reset() reset animation at " << this << endl;
			return true;
        }

		void addAnimationObserver (Interface_AnimationObserver* observer);
		bool hasAnimationObserver (Interface_AnimationObserver* observer);
		void removeAnimationObserver (Interface_AnimationObserver* observer);

		SDL_Surface* getImage();

        // update() returns true if frame changed, false otherwise.
        bool update (unsigned int timePassed);
        SDL_Rect getRect();

		void load (string defFile);

    protected:
        int timeLength, timePassed;
        bool repeating;
        InterruptFlag interruptibleBy;
		AnimationType animationType;
		vector< Interface_AnimationObserver* > animationObservers;
		StampSet* stampSet;
		vector< int > frames;
		ResourcePool* resources;
		bool notifiedObservers;
};

#endif
