/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <iostream>
#include "animationSet.h"
#include "surface.h"

using namespace std;


AnimationSet::AnimationSet (string definitionFile, ResourcePool* resources){
	//cout << "AnimationSet::AnimationSet (string definitionFile, ResourcePool* resources) at " << this << endl;
	this->resources = resources;
    this->load (definitionFile);
    this->currentAnimation = -1;
	this->dirty = true;
	animationDefFile = definitionFile;
}
AnimationSet::AnimationSet (ResourcePool* resources){
	//cout << "AnimationSet::AnimationSet (ResourcePool* resources) at " << this << endl;
    this->resources = resources;
    this->currentAnimation = -1;
	this->dirty = true;
	animationDefFile = "";
}
AnimationSet::AnimationSet (AnimationSetSettings settings){
	//cout << "AnimationSet::AnimationSet (AnimationSetSettings settings) at " << this << endl;
    this->frames = settings.getFrames();
    this->animationObservers = settings.getAnimationObservers();
    this->currentAnimation = settings.getCurrentAnimation();
    this->resources = settings.getResources();
	this->dirty = settings.getDirty();
	animationDefFile = settings.getAnimationDefFile();

	// Make sure we're set up as observer for all the animations (frames) we got.
	vector<int> keys = frames.getKeys();
	for (vector<int>::iterator nextFrame = keys.begin(); nextFrame != keys.end(); nextFrame++){
		if (! frames.get (*nextFrame)->hasAnimationObserver (this)){
			frames.get (*nextFrame)->addAnimationObserver (this);
		}
	}
}

AnimationSet::~AnimationSet(){
    cleanup();
}

Interface_Cloneable* AnimationSet::clone() const{
    //map< int, Animation* > newFrames;
	//AnimationSetMap newFrames;
    // First, clone all the Animations that are kept by this AnimationSet.
    //for (map< int, Animation* >::iterator next = frames.begin();
    //        next != frames.end(); next++){
	//for (vector< vector<Animation*> >::iterator next = nextFrames.begin(); next != nextFrames.end(); next++){
        //newFrames [next->first] = dynamic_cast<Animation*> (next->second->clone());
	//	newFrames.set ((*next) [0], dynamic_cast<Animation*> (*next) [1]);
    //}
	//newFrames.copyFrom (frames);
    // The rest can use a shallow copy.
	AnimationSetMap newFrames;
	const vector<int> keys = frames.getKeys();
	vector<Interface_AnimationObserver*> newObservers;
	for (vector<int>::const_iterator next = keys.begin(); next != keys.end(); next++){
		newFrames.set ( *next, dynamic_cast<Animation*> (frames.get (*next)->clone()) );
	}
    //AnimationSetSettings settings (newFrames, animationObservers, currentAnimation, resources, true, animationDefFile);
    AnimationSetSettings settings (newFrames, newObservers, currentAnimation, resources, true, animationDefFile);

	//Interface_Cloneable* retSet = new AnimationSet (settings);
	AnimationSet* retSet = new AnimationSet (settings);
	//cout << "AnimationSet::clone(): retSet is " << retSet << 
	//	" and, when cast, it is " << dynamic_cast< AnimationSet* >(retSet) << endl;
	//Interface_Cloneable* junk = retSet;
	//cout << "AnimationSet::clone(): junk is " << junk << 
	//	" and, when cast, it is " << dynamic_cast< AnimationSet* >(junk) << endl;
	return retSet;
}

void AnimationSet::cleanup(){
	//for (map< int, Animation* >::iterator next = frames.begin(); 
	//		next != frames.end(); next++){
	//AnimationSetMap allFrames = frames.getAll();
	//for (vector< vector<Animation*> >::iterator next = allFrames.begin(); next != allFrames.end(); next++){
    	//delete (next->second); 
	//	delete ((*next) [1]);
    //}
	vector<int> keys = frames.getKeys();
	for (vector<int>::iterator iter = keys.begin(); iter != keys.end(); iter++){
		if (frames.get (*iter)){ 
			delete frames.get (*iter); frames.erase (*iter);
		}
	}
    frames.clear();
}

bool AnimationSet::play (AnimationType index, InterruptFlag interrupter){
	//cout << "AnimationSet::play() index " << index << endl;
	bool played = false;

	//Animation* derived = frames [index];
	//cout << "AnimationSet::play() trying to play index " << index << endl;
	Animation* derived = frames.get (index);
	if (derived){
		//cout << "AnimationSet::play() frames [index] is good." << endl;
        //if (currentAnimation == -1 || ( currentAnimation != -1 && frames [currentAnimation]->isInterruptibleBy (interrupter) )){
        if (currentAnimation == -1 || ( currentAnimation != -1 && frames.get (currentAnimation)->isInterruptibleBy (interrupter) )){
			//cout << "AnimationSet::play() interruptible check is good." << endl;
	    	if (derived->play()){
				//cout << "AnimationSet::play() is good." << endl;
		        this->currentAnimation = index;
	    	    played = true;
			}
		}
    }
	if (! played){
		cout << "AnimationSet::play() failed! " << endl;
		throw AnimationSetPlayException();
	}

	return played;
}

int AnimationSet::getCurrentAnimationFrame() const{
	if (currentAnimation == -1){
		return -1;
	}
	return frames.get (currentAnimation)->getCurrentFrame();
}

int AnimationSet::getCurrentAnimationFrameCount() const{
	if (currentAnimation == -1){
		return -1;
	}
	return frames.get (currentAnimation)->getFrameCount();
}

int AnimationSet::getCurrentAnimationTimeRemaining() const{
	if (currentAnimation == -1){
		return 0;
	}
	return frames.get (currentAnimation)->getAnimationTimeRemaining();
}

SDL_Surface* AnimationSet::getCurrentSurface(){
	if (currentAnimation == -1){
		return NULL;
	}
	//return frames [currentAnimation]->getImage();
	return frames.get (currentAnimation)->getImage();
}

SDL_Rect AnimationSet::getCurrentRect(){
	//cout << "current animation is " << currentAnimation << endl;
	if (currentAnimation != -1){
		//SDL_Rect retRect = frames [currentAnimation]->getRect();
		SDL_Rect retRect = frames.get (currentAnimation)->getRect();
		//cout << "AnimationSet::getCurrentRect() returning: " << retRect.x << ", " << retRect.y 
		//	<< ", " << retRect.w << ", " << retRect.h << "." << endl;
		return retRect;
	}
	else{
		//cout << "Returning default rect in AnimationSet::getCurrentRect()." << endl;
		//cout << "AnimationSet::getCurrentRect currentAnimation is -1." << endl;
		SDL_Rect retRect;
		retRect.x = 0; retRect.y = 0; retRect.w = 0; retRect.h = 0;
		//cout << "AnimationSet::getCurrentRect() returning: " << retRect.x << ", " << retRect.y 
		//	<< ", " << retRect.w << ", " << retRect.h << "." << endl;
		return retRect;
	}
}

void AnimationSet::load (string definitionFile){
	//cout << "Loading animation set using definitionFile '" << definitionFile << "'." << endl;
	if (frames.size() > 0){
		/*
		for (map< int, Animation* >::iterator next = frames.begin(); next != frames.end(); next++){
			delete next->second; next->second = NULL;
		}
		frames.clear();
		*/
		cleanup();
	}
	// This def file should be a ".set" file with a list of indexes and animation def files (".anm" files).
    ifstream defFile;
    defFile.open (definitionFile.c_str());
    string animationFile;
	AnimationType index;
	defFile >> index;
	defFile >> animationFile;
	while (! defFile.eof()){
		//cout << "AnimationSet's next index: " << index << " and animation file: '" << animationFile << "'." << endl; 

		// Now set up this animation set's next animation.
		//frames [index] = new Animation (resources, index, animationFile);
		//frames [index] = resources->loadAnimation (index, animationFile);
		frames.set (index, resources->loadAnimation (index, animationFile));
		//frames [index]->addAnimationObserver (this);
		frames.get (index)->addAnimationObserver (this);
		defFile >> index;
	    defFile >> animationFile;
	}
    defFile.close();
	//cout << "Done loading animation set." << endl;
}

void AnimationSet::setRepeating (AnimationType index, bool repeating){
	//frames [index]->setRepeating (repeating);
	frames.get (index)->setRepeating (repeating);
}

void AnimationSet::addAnimationObserver (Interface_AnimationObserver* observer){
	if (! hasAnimationObserver (observer)){
		animationObservers.push_back (observer);
	}
}
bool AnimationSet::hasAnimationObserver (Interface_AnimationObserver* observer){
	for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			next != animationObservers.end(); next++){
		if (*next == observer){
			return true;
		}
	}
	return false;
}
void AnimationSet::removeAnimationObserver (Interface_AnimationObserver* observer){
	//cout << "AnimationSet::removeAnimationObserver (" << observer << ")" << endl;
	bool found = false;
	for (vector< Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			!found && next != animationObservers.end(); next++){
		if (*next == observer){
			found = true;
			animationObservers.erase (next);
		}
	}
}

void AnimationSet::animationComplete (Surface* dummy, AnimationType animationType){
	//cout << "AnimationSet::animationComplete (" << dummy << ", " << animationType << ")" << endl;
	//cout << "	(i am " << this << ") with defFile of " << animationDefFile << endl;
	for (vector < Interface_AnimationObserver* >::iterator next = animationObservers.begin();
			next != animationObservers.end(); next++){
		//cout << "AnimationSet::animationComplete (" << dummy << ", " << animationType << ") notifying an observer." << endl;
		(*next)->animationComplete (NULL, animationType);
	}
}

bool AnimationSet::resetAnimation (AnimationType index, InterruptFlag interrupter){
	//cout << "AnimationSet::resetAnimation (" << index << ", " << interrupter << ")" << endl;
	//cout << "frames.size() is " << frames.size() << endl;
	if (index > -1){
		//cout << "AnimationSet::resetAnimation() passed test 1" << endl;
		//if (this->frames [index]->isInterruptibleBy (interrupter)){
		if (this->frames.get (index)->isInterruptibleBy (interrupter)){
			//cout << "AnimationSet::resetAnimation() passed test 2" << endl;
			//this->frames [index]->reset();
			return this->frames.get (index)->reset();
		}
		else{
			//cout << "AnimationSet::resetAnimation() failed test 2" << endl;
			return false;
		}
	}
	else{
		//cout << "AnimationSet::resetAnimation() failed test 1" << endl;
		return false;
	}
}

void AnimationSet::setTimeLength (AnimationType index, unsigned int milliseconds){
	//frames [index]->setTimeLength (milliseconds);
	frames.get (index)->setTimeLength (milliseconds);
}

bool AnimationSet::update (unsigned int timePassed){
	//cout << "AnimationSet::update (" << timePassed << ") called." << endl;
    if (this->currentAnimation != -1){
        //if (this->frames [this->currentAnimation]->update (timePassed)){
		if (this->frames.get (currentAnimation)->update (timePassed)){
            // Current animation is dirty if the frame changed (returned true).
			setDirty (true);
        }
    }

	bool dirty = getDirty();
	setDirty (false);

	return dirty;
}
