/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ANIMATIONSET_H
#define ANIMATIONSET_H

#include <iostream>
#include <string>
#include <fstream>
#include <vector>
#include "interface_animationObserver.h"
#include "utils.h"
#include "animation.h"
#include "interface_cloneable.h"
#include "resourcePool.h"
#include "animationSetSettings.h"
#include "animationSetExceptions.h"
#include "intMap.h"


using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/


class Animation;
class AnimationSet : public Interface_AnimationObserver, public Interface_Cloneable{
    public:
        AnimationSet (string definitionFile, ResourcePool* resources);
        AnimationSet (ResourcePool* resources);
		AnimationSet (AnimationSetSettings settings);
		virtual ~AnimationSet();

		virtual Interface_Cloneable* clone() const;

		bool getDirty(){ return dirty; }
		void setDirty (bool dirtyNow){ dirty = dirtyNow; }
        void load (string definitionFile);
        void setInterruptibleBy (AnimationType index, InterruptFlag interruptibleBy);
        void setTimeLength (AnimationType index, unsigned int timeLength);
        void setRepeating (AnimationType index, bool repeating);
        // Update returns true if the current frame changed.
        bool update (unsigned int timePassed);
        SDL_Surface* getCurrentSurface();
		SDL_Rect getCurrentRect();
		int getCurrentAnimation(){ return currentAnimation; }
		int getCurrentAnimationTimeRemaining() const;
		int getCurrentAnimationFrame() const;
		int getCurrentAnimationFrameCount() const;
        void animationComplete (Surface* dummy, AnimationType animationType);
        bool resetAnimation (AnimationType index, InterruptFlag interrupter = INTERRUPT_SELF);
        bool play (AnimationType index, InterruptFlag interrupter = INTERRUPT_SELF);
		void addAnimationObserver (Interface_AnimationObserver* observer);
		bool hasAnimationObserver (Interface_AnimationObserver* observer);
		void removeAnimationObserver (Interface_AnimationObserver* observer);
    
    protected:
		void cleanup();
        //map< int, Animation* > frames;
		AnimationSetMap frames;
		vector< Interface_AnimationObserver* > animationObservers;
        int currentAnimation;
		ResourcePool* resources;
		bool dirty;
		string animationDefFile;
};

#endif
