#include "audioData.h"
#include <iostream>
#include "utils.h"

using namespace std;


Audio::AudioSample::AudioSample (short leftSample, short rightSample, unsigned int bitSize)
	: leftSample_ (leftSample), rightSample_ (rightSample), bitSize_ (bitSize) { }
Audio::AudioSample::AudioSample() : leftSample_ (0), rightSample_ (0), bitSize_ (16) { }
Audio::AudioSample::~AudioSample() { }

short Audio::AudioSample::getLeftSample() const{
	return leftSample_;
}
short Audio::AudioSample::getRightSample() const{
	return rightSample_;
}
unsigned int Audio::AudioSample::getBitSize() const{
	return bitSize_;
}


Audio::AudioData::AudioData (unsigned int bitSize, unsigned int rate) : bitSize_ (bitSize), rate_ (rate){
	if (bitSize_ != 16) cout << "AudioData does not support bitSize " << bitSize_ << endl;
	rawSamplesReserved_ = 100;
	unsigned int bytes = rawSamplesReserved_ * (bitSize_ / 8) * 2; // Make room for 100 samples of x bit, 2 channels
	raw_ = boost::shared_array<Uint32> (new Uint32 [(bytes / 4) + 2]);
	memset (&(raw_ [0]), 0, bytes);
	samplesCount_ = 0;
}
Audio::AudioData::AudioData (unsigned int bitSize, unsigned int rate, char* dataToCopy, unsigned int sizeInBytes)
		: bitSize_ (bitSize), rate_ (rate){
	//cout << "creating audiodata from raw data, size in bytes: " << sizeInBytes << endl;
	if (bitSize != 16) cout << "AudioData does not support bitSize " << bitSize_ << endl;
	raw_ = boost::shared_array<Uint32> (new Uint32 [(sizeInBytes / 4) + 2]);
	memcpy ((char*) &(raw_ [0]), dataToCopy, sizeInBytes);
	rawSamplesReserved_ = sizeInBytes * 8 / 2 / bitSize_;
	samplesCount_ = rawSamplesReserved_;
	//cout << "raw samples reserved is " << rawSamplesReserved_ << endl;
}
Audio::AudioData::~AudioData(){ }

void Audio::AudioData::copySamplesFrom (Audio::AudioData& from){
	copySamplesFrom (from, 0);
}
void Audio::AudioData::copySamplesFrom (Audio::AudioData& from, unsigned int fromStart){
	copySamplesFrom (from, fromStart, from.getNumberOfSamples());
}
void Audio::AudioData::copySamplesFrom (Audio::AudioData& from, unsigned int fromStart, unsigned int count){
	copySamplesFrom (from, fromStart, count, fromStart);
}
void Audio::AudioData::copySamplesFrom (Audio::AudioData& from, unsigned int fromStart, unsigned int count, unsigned int toStart){
	if (fromStart >= from.getNumberOfSamples()) return;
	if (fromStart + count > from.getNumberOfSamples()) count = from.getNumberOfSamples() - fromStart;
	//if (toStart != 0) cout << "copying " << count << " samples from " << fromStart << " to " << toStart << endl;
	if (count < 1) return;
	//cout << "copy samples from start is " << fromStart << endl;
	if (bitSize_ == 16 && bitSize_ == from.getBitSize()){
		if (count + toStart >= getNumberOfSamples()){
			unsigned int where = (toStart + count) - 1;
			//cout << "toStart is " << toStart << ", count is " << count << endl;
			//cout << "where is " << where << endl;
			setSample (where, Audio::AudioSample (0, 0, bitSize_));
		}
		memcpy ((char*) &(raw_ [toStart]), (char*) &(from.getRaw() [fromStart]), count * 4);
	}
	//if (toStart != 0) cout << "done copying to " << toStart << endl;
}
void Audio::AudioData::setSample (unsigned int where, const AudioSample& sample){
	// If we're setting a sample beyond what we can accomodate, resize raw_ data.
	if (where >= samplesCount_){
		if (rawSamplesReserved_ == 0) rawSamplesReserved_ = 100;
		unsigned int oldSize = samplesCount_ * bitSize_ / 8 * 2;
		//cout << "rawSamplesReserved was " << rawSamplesReserved_ << ", where was " << where << endl;
		while (rawSamplesReserved_ <= where + 1) rawSamplesReserved_ *= 2;
		//cout << "rawSamplesReserved now " << rawSamplesReserved_ << ", where now " << where << endl;
		Uint32* newData = NULL;
		if (bitSize_ == 16){
			newData = new Uint32 [rawSamplesReserved_ + 2];
		}
		else{
			cout << "wtf bitsize " << bitSize_ << " in AudioData::setSample()!" << endl;
		}
		memset ((char*) newData, 0, rawSamplesReserved_ * bitSize_ / 8 * 2);
		memcpy ((char*) newData, (char*) &(raw_ [0]), oldSize);
		raw_ = boost::shared_array<Uint32> (newData);
		samplesCount_ = where + 1;
	}
	//cout << "audio data setting sample " << where << endl;
	if (bitSize_ == 16){
		short* rawPtr = (short*) &(raw_ [where]);
		short left = sample.getLeftSample();
		short right = sample.getRightSample();
		rawPtr [0] = left; // HAX assumes little-endian
		rawPtr [1] = right; // HAX assumes little-endian
	}
	else{
		cout << "wtf bitsize " << bitSize_ << endl;
	}
}
Audio::AudioSample Audio::AudioData::getSample (unsigned int where) const{
	if (where < 0) return AudioSample();
	if (where < samplesCount_ && bitSize_ == 16){
		short* rawPtr = (short*) &(raw_ [where]);
		unsigned short left = rawPtr [0]; // HAX assumes little-endian
		unsigned short right = rawPtr [1]; // HAX assumes little-endian
		return Audio::AudioSample (left, right, bitSize_);
	}
	return AudioSample();
}
size_t Audio::AudioData::getNumberOfSamples() const{
	return samplesCount_;
}
boost::shared_array<Uint32> Audio::AudioData::getRaw() const{
	return raw_;
}
unsigned int Audio::AudioData::getRate() const{
	return rate_;
}
unsigned int Audio::AudioData::getBitSize() const{
	return bitSize_;
}
