#ifndef AUDIO_DATA_H
#define AUDIO_DATA_H

#include <vector>
#include <SDL/SDL.h>
#ifdef WIN32
	#include <memory>
#else
	#include <tr1/memory>
#endif
#include <boost/shared_array.hpp>


namespace Audio{
	class AudioSample{
		public:
			AudioSample (short leftSample, short rightSample, unsigned int bitsize);
			AudioSample();
			~AudioSample();
	
			short getLeftSample() const;
			short getRightSample() const;
			unsigned int getBitSize() const;

		private:
			short leftSample_;
			short rightSample_;
			unsigned int bitSize_;
	};

	class AudioData{
		public:
			AudioData (unsigned int bitSize, unsigned int rate, char* dataToCopy, unsigned int sizeInBytes);
			AudioData (unsigned int bitSize, unsigned int rate);
			AudioData();
			~AudioData();

			void setSample (unsigned int where, const AudioSample& sample);
			AudioSample getSample (unsigned int where) const;
			size_t getNumberOfSamples() const;
			boost::shared_array<Uint32> getRaw() const;
			unsigned int getRate() const;
			unsigned int getBitSize() const;
			void copySamplesFrom (Audio::AudioData& from);
			void copySamplesFrom (Audio::AudioData& from, unsigned int fromStart);
			void copySamplesFrom (Audio::AudioData& from, unsigned int fromStart, unsigned int count);
			void copySamplesFrom (Audio::AudioData& from, unsigned int fromStart, unsigned int count, 
				unsigned int toStart);

		private:
			boost::shared_array<Uint32> raw_;
			unsigned int rawSamplesReserved_;
			unsigned int samplesCount_;
			unsigned int bitSize_;
			unsigned int rate_;
	};
}


#endif
