#include "audioEffectEcho.h"
#include "interface_audioProcessingManager.h"
#include "audioData.h"
#include "utils.h"


#define AUDIO_EFFECT_ECHO_MAX_DECAY .9
#define AUDIO_EFFECT_ECHO_MIN_VOLUME .01

AudioEffectEcho::AudioEffectEcho (Uint32 delay, double decay){
	if (decay < 0) decay = 0;
	if (decay >= AUDIO_EFFECT_ECHO_MAX_DECAY) decay = AUDIO_EFFECT_ECHO_MAX_DECAY;
	decay_ = decay;
	delay_ = delay;
	double volume = 1.0;
	for (echoes_ = 0; volume > AUDIO_EFFECT_ECHO_MIN_VOLUME; echoes_++, volume *= decay_);
}
AudioEffectEcho::~AudioEffectEcho(){}

Uint32 AudioEffectEcho::getTimeLengthOffset (Uint32 currentLength){
	//cout << "length offset is " << echoes_ * delay_ << endl;
	return echoes_ * delay_;
}
void AudioEffectEcho::processAudio (Interface_AudioProcessingManager* manager){
	unsigned int numSamples = manager->getNumberOfSamplesToProcess();
	unsigned int rate = 44100; // HAX how to get the rate?
	unsigned int bitSize = 16;
	double samplesPerMs = rate / 1000.0; 
	int samplesDelay = samplesPerMs * delay_;
	//cout << "samplesDelay is " << samplesDelay << endl;

	int sampleWindowStart = manager->getCurrentSampleIndex();
	Audio::AudioData source (manager->getCopyOfSourceData());
	short* sourceData = (short*) &(source.getRaw() [0]);
	numSamples = std::min (source.getNumberOfSamples() + samplesDelay - sampleWindowStart, numSamples); 

	unsigned int echoedDataSizeInBytes = 2 * sizeof (short) * numSamples; // HAX assumes 16-bit depth
	short* echoedData = (short*) malloc (echoedDataSizeInBytes);

	for (unsigned int y = 0; y < numSamples; y++){
		double volume = decay_;
		unsigned int sourceLoc = (y * 2) + (sampleWindowStart * 2);
		if (y + sampleWindowStart < source.getNumberOfSamples()){
			echoedData [(2 * y)] = sourceData [sourceLoc];
			echoedData [(2 * y) + 1] = sourceData [sourceLoc + 1];
		}
		else{
			echoedData [(2 * y)] = 0;
			echoedData [(2 * y) + 1] = 0;
		}
		for (int x = samplesDelay; volume > AUDIO_EFFECT_ECHO_MIN_VOLUME && y + sampleWindowStart - x >= 0; 
				volume *= decay_, x += samplesDelay){
			if (y + sampleWindowStart - x < source.getNumberOfSamples()){
				short left = sourceData [sourceLoc - (x * 2)] * volume;
				echoedData [(2 * y)] += left;
				short right = sourceData [sourceLoc + 1 - (x * 2)] * volume;
				echoedData [(2 * y) + 1] += right;
			}
		}
	}

	Audio::AudioData echoedAudioData (bitSize, rate, (char*) echoedData, echoedDataSizeInBytes);
	//cout << "echoed data has " << echoedAudioData.getNumberOfSamples() << endl;
	//cout << "source data has " << source.getNumberOfSamples() << endl;
	//cout << "setting. numSamples is " << numSamples << endl;
	manager->setCompositeSamples (echoedAudioData, 0, numSamples, 0);
	//cout << "done setting" << endl;

	free (echoedData); echoedData = NULL;
}
