#ifndef AUDIO_MANAGER_H
#define AUDIO_MANAGER_H

#include <string>
#include <map>
#ifdef WIN32
	#include <memory>
#else
	#include <tr1/memory>
#endif
#include <SDL/SDL_audio.h>
#include <vector>
#include "interface_audioProcessingManager.h"


class SoundInstance;
class SoundInstanceReference;
class Interface_Lockable;
namespace Audio{

	typedef std::vector< std::pair<SoundInstance*, AudioData> > ExternalInstancesList;
	
	class AudioManager{
		friend class AudioManagerTester;
		public:
			AudioManager();
			virtual ~AudioManager();

			std::tr1::shared_ptr<SoundInstanceReference> playSound (
				const std::string& filename, double volumeZeroToOne = 1.0);
			std::tr1::shared_ptr<SoundInstanceReference> loadSound (const std::string& filename);
			void setMainVolume (double volumeZeroToOne);
			double getMainVolume();

			void update (unsigned int timePassed);

			void mixAudio (void *unused, Uint8 *stream, int amountToBuffer);
			
		protected:
		
		private:
			void applyPanning16 (Uint32* start, unsigned int numSamples, double panZeroToOne);

			void refreshInstances (unsigned int timePassed);

			SDL_AudioSpec fmt;
			std::vector<std::tr1::shared_ptr <SoundInstanceReference> > internalInstances_;
			Interface_Lockable* internalInstancesLock_;
			ExternalInstancesList externalInstances_;
			Interface_Lockable* externalInstancesLock_;
			std::map<std::string, int> loadedSounds_;
			Interface_Lockable* loadedSoundsLock_;
			Uint32 bytesPerMs_;
	};
}

namespace Audio{
	extern AudioManager audioManager_;
}

#endif
