#include "audioProcessingManager.h"
#include <iostream>

using namespace std;


Audio::AudioProcessingManager::AudioProcessingManager (unsigned int bitSize, unsigned int rate)
	: Interface_AudioProcessingManager (bitSize, rate),
	sourceData_ (bitSize, rate), compositeData_ (bitSize, rate), sampleWindowStart_ (0), sampleWindowEnd_ (0)
{ }
Audio::AudioProcessingManager::~AudioProcessingManager() { }

void Audio::AudioProcessingManager::startEffectStack (AudioData& sourceData, 
		unsigned int sampleWindowStart, unsigned int sampleWindowEnd){
	sourceData_ = sourceData;
	sampleWindowStart_ = sampleWindowStart;
	sampleWindowEnd_ = sampleWindowEnd;
	sampleWindowEnd_ = std::max (sampleWindowEnd_, sampleWindowStart_);
	unsigned int lastSample = std::min (sampleWindowEnd_, sourceData_.getNumberOfSamples());
	//unsigned int lastSample = sourceData_.getNumberOfSamples();
	compositeData_ = AudioData (sourceData_.getBitSize(), sourceData_.getRate(), 
		(char*) &(sourceData_.getRaw() [0]), lastSample * sourceData_.getBitSize() * 2 / 8);
}
void Audio::AudioProcessingManager::endEffectStack(){ }
unsigned int Audio::AudioProcessingManager::getNumberOfSamplesToProcess(){
	return sampleWindowEnd_ - sampleWindowStart_;
}
Audio::AudioSample Audio::AudioProcessingManager::getSourceSample (int sampleOffset){
	return sourceData_.getSample (sampleWindowStart_ + sampleOffset);
}
Audio::AudioSample Audio::AudioProcessingManager::getCompositeSample (int sampleOffset){
	int where = sampleWindowStart_ + sampleOffset;
	return compositeData_.getSample (where);
}
Audio::AudioSample Audio::AudioProcessingManager::getSourceSampleByTime (int msOffset){
	//cout << "msOffset " << msOffset << endl;
	double samplesPerMs = sourceData_.getRate() / 1000.0;
	//cout << "samplesPerMs " << samplesPerMs << endl;
	int where = (int) (samplesPerMs * msOffset);
	cout << "getting source sample at " << where << ". sourcedata rate is " << sourceData_.getRate() << endl;
	return getSourceSample (where);
}
Audio::AudioSample Audio::AudioProcessingManager::getCompositeSampleByTime (int msOffset){
	unsigned int samplesPerMs = compositeData_.getRate() / 1000.0;
	int where = samplesPerMs * msOffset;
	return getCompositeSample (where);
}
void Audio::AudioProcessingManager::setCompositeSample (int sampleOffset, const Audio::AudioSample& sample){
	int where = sampleWindowStart_ + sampleOffset;
	if (where < 0 || (unsigned int) where >= sampleWindowEnd_) return;
	compositeData_.setSample (where, sample);
}
void Audio::AudioProcessingManager::setCompositeSamples (Audio::AudioData& from, unsigned int start, unsigned int count, unsigned int toStart){
	if (start + count > sampleWindowEnd_) count = sampleWindowEnd_ - start;
	if (start >= sampleWindowEnd_) return;
	compositeData_.copySamplesFrom (from, start, count, sampleWindowStart_ + toStart);
}
unsigned int Audio::AudioProcessingManager::getCurrentSampleIndex(){
	return sampleWindowStart_;
}
Audio::AudioData Audio::AudioProcessingManager::getCompositeAudioData(){
	return compositeData_;
}
Audio::AudioData Audio::AudioProcessingManager::getCopyOfSourceData(){
	Audio::AudioData copy (sourceData_.getBitSize(), sourceData_.getRate());
	copy.copySamplesFrom (sourceData_);
	return copy;
}
Audio::AudioData Audio::AudioProcessingManager::getCopyOfSourceDataInWindow(){
	Audio::AudioData copy (sourceData_.getBitSize(), sourceData_.getRate());
	int windowSize = sampleWindowEnd_ - sampleWindowStart_;
	//cout << "windowSize " << windowSize << endl;
	//cout << "sourcedata has bitsize, rate: " << sourceData_.getBitSize() << ", " << sourceData_.getRate() << endl;
	//cout << "copy has bitsize, rate: " << copy.getBitSize() << ", " << copy.getRate() << endl;
	if (windowSize > 0){
		copy.copySamplesFrom (sourceData_, sampleWindowStart_, windowSize, 0);
	}
	//cout << "copy of source data has " << copy.getNumberOfSamples() << " samples" << endl;
	return copy;
}
