#include "basket.h"
#include "sprite.h"
#include "animationSet.h"


Basket::Basket (Surface* owner, const Rect& perimeter, const std::string& spriteSheet, const std::string& spriteDef)
		: Sprite (owner, spriteSheet){
	loadAnimations (spriteDef);
	setAnimationTimeLength (ANIM_BASKET_STAND_LEFT, 250);
	setAnimationTimeLength (ANIM_BASKET_STAND_RIGHT, 250);
	setAnimationTimeLength (ANIM_BASKET_WALK_LEFT, 250);
	setAnimationTimeLength (ANIM_BASKET_WALK_RIGHT, 250);

	decision = 0;

	timeSinceDecision = 0;

	goingLeft = false;

	setSpeed (BASKET_WALK_SPEED);

	setMovementPerimeter (perimeter);

	rightBound_ = 0; leftBound_ = 0;

	facingFlags = BASKET_LEFT;
	moveFlags = 0;

	playStandingAnim();
}
Basket::~Basket() { }

void Basket::setLeftBound (int leftBound){
	leftBound_ = leftBound;
}
void Basket::setRightBound (int rightBound){
	rightBound_ = rightBound;
}

bool Basket::update (unsigned int timePassed){
	bool result = Sprite::update (timePassed);
	double amountToMoveX = getSpeedInPixelsPerMs() * timePassed;
	if (moveFlags & BASKET_RIGHT) moveXY (amountToMoveX, 0);
	else if (moveFlags & BASKET_LEFT) moveXY (-amountToMoveX, 0);
	if (moveFlags == 0) playStandingAnim();
	moveFlags = 0;

	timeSinceDecision += timePassed;
	if (timeSinceDecision >= 3000){
		timeSinceDecision = 0;
		int choice = rand() % 100;
		decision = goingLeft ? BASKET_LEFT : BASKET_RIGHT;
		if (choice < 20){
			decision = goingLeft ? BASKET_RIGHT : BASKET_LEFT;
		}
		else if (choice < 50) decision = 0;
	}
	if (getX() < leftBound_){ 
		goingLeft = false;
		decision = BASKET_RIGHT;
	}
	if (getX() > rightBound_){
		goingLeft = true;
		decision = BASKET_LEFT;
	}
	if (decision == BASKET_LEFT) moveLeft();
	else if (decision == BASKET_RIGHT) moveRight();
	else standStill();

	return result;
}

void Basket::standStill(){
	moveFlags = 0;
}
void Basket::moveRight(){
	facingFlags = BASKET_RIGHT;
	playWalkingAnim();
	moveFlags = BASKET_RIGHT;
}
void Basket::moveLeft(){
	facingFlags = BASKET_LEFT;
	playWalkingAnim();
	moveFlags = BASKET_LEFT;
}

void Basket::playStandingAnim(){
	if (standingAnimPlaying()) return;
	if (facingFlags & BASKET_LEFT){
		resetAnimation (ANIM_BASKET_STAND_LEFT);
		playAnimation (ANIM_BASKET_STAND_LEFT, true);
	}
	else{
		resetAnimation (ANIM_BASKET_STAND_RIGHT);
		playAnimation (ANIM_BASKET_STAND_RIGHT, true);
	}
}
void Basket::playWalkingAnim(){
	if (walkAnimPlaying()) return;
	if (facingFlags & BASKET_LEFT){
		resetAnimation (ANIM_BASKET_WALK_LEFT);
		playAnimation (ANIM_BASKET_WALK_LEFT, true);
	}
	else{
		resetAnimation (ANIM_BASKET_WALK_RIGHT);
		playAnimation (ANIM_BASKET_WALK_RIGHT, true);
	}
}
bool Basket::animPlaying (int leftVersion, int rightVersion){
	int currentAnim = getAnimations()->getCurrentAnimation();
	int timeLeft = getAnimations()->getCurrentAnimationTimeRemaining();
	if (timeLeft == 0) return false;
	if (facingFlags == BASKET_LEFT && currentAnim != leftVersion) return false;
	if (facingFlags == BASKET_RIGHT && currentAnim != rightVersion) return false;
	return true;
}
bool Basket::walkAnimPlaying(){
	bool playing = animPlaying (ANIM_BASKET_WALK_LEFT, ANIM_BASKET_WALK_RIGHT);
	//cout << "walk playing? " << playing << endl;
	return playing;
}
bool Basket::standingAnimPlaying(){
	//cout << "walk playing? " << playing << endl;
	return animPlaying (ANIM_BASKET_STAND_LEFT, ANIM_BASKET_STAND_RIGHT);
}
