/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "border.h"
#include "imageProcessingSurfaceManager.h"
#include <fstream>

Border::Border (Surface* owner, string definition){
	resources = NULL;
	legs [0] = NULL; legs [1] = NULL; legs [2] = NULL; legs [3] = NULL;
	corners [0] = NULL; corners [1] = NULL; corners [2] = NULL; corners [3] = NULL;
	this->owner = owner;
	if (owner){
		owner->setBorder (this);
	}
	else{
		cout << "My owner is NULL!?" << endl;
	}
	rect = new Rect (0, 0, 0, 0);
	// Load the definition file.
	load (definition);
	vector<Rect> surrounded;
	if (owner) surrounded.push_back (*owner->getRect());
	buildPerimeter (surrounded);
}
Border::~Border(){
	if (legsSet && cornersSet && owner){ 
		owner->reportDyingRect (* (legs [0]->getRect()));
		owner->reportDyingRect (* (legs [1]->getRect()));
		owner->reportDyingRect (* (legs [2]->getRect()));
		owner->reportDyingRect (* (legs [3]->getRect()));
		owner->reportDyingRect (* (corners [0]->getRect()));
		owner->reportDyingRect (* (corners [1]->getRect()));
		owner->reportDyingRect (* (corners [2]->getRect()));
		owner->reportDyingRect (* (corners [3]->getRect()));
	}
	resetLegs(); resetCorners();
	delete rect;
}

Rect* Border::getRect() const{
	return rect;
}

void Border::buildPerimeter (vector<Rect>& surrounded){
    // TODO: this needs to be the actual visible viewport. Otherwise, effects ports don't make 
    //      the border change shape (current behavior!)
    //Rect borderedViewport = getAbsoluteRect();
    vector<Rect>& views = surrounded;
    // If this surface doesn't even have a viewport, just forget about drawing a border.
    if (views.size() < 1){
		Rect* myRect = getRect();
		myRect->setXY (0, 0);
		myRect->setWidth (0); 
		myRect->setHeight (0);
		return;
	}
    double minX = int (views [0].getX());
    double maxX = minX + views [0].getWidth();
    double minY = int (views [0].getY());
    double maxY = minY + views [0].getHeight();

    vector<Rect>::iterator iter;
    vector<Rect>::iterator endIter;
    double iterX, iterX2, iterY, iterY2;
    // We already checked the first viewport, so we can skip it. That's what the '+ 1' is for.
    for (iter = views.begin() + 1, endIter = views.end(); iter != endIter; iter++){
        iterX = int (iter->getX());
        iterX2 = iterX + iter->getWidth();
        iterY = int (iter->getY());
        iterY2 = iterY + iter->getHeight();
        if (iterX < minX) minX = iterX;
        if (iterX2 > maxX) maxX = iterX2;
        if (iterY < minY) minY = iterY;
        if (iterY2 > maxY) maxY = iterY2;
    }
    Rect borderedViewport = Rect (minX, minY, maxX - minX, maxY - minY);

    //cout << "Made a bordered viewport " << borderedViewport.toString() << endl;

    if (borderedViewport.getWidth() < 1 || borderedViewport.getHeight() < 1){ 
		Rect* myRect = getRect();
		myRect->setXY (0, 0);
		myRect->setWidth (0);
		myRect->setHeight (0);
		return;
	}
    Rect* myRect = getRect(); 
	myRect->setXY (borderedViewport.getX(), borderedViewport.getY());
	myRect->setWidth (borderedViewport.getWidth());
	myRect->setHeight (borderedViewport.getHeight());
}

Rect Border::getViewport(){
	Rect borderedViewport = *(getRect());
	
    int reduction = getMaximumFrameThickness() * 2;
	//reduction = 0;
    double width = borderedViewport.getWidth();
    double height = borderedViewport.getHeight();
    double bordX = borderedViewport.getX();
    double bordY = borderedViewport.getY();
    if (width < reduction || height < reduction){ 
		return Rect (0, 0, 0, 0);
	}
    borderedViewport.setWidth (width - reduction);
    borderedViewport.setHeight (height - reduction);
    borderedViewport.setX (bordX + reduction / 2);
    borderedViewport.setY (bordY + reduction / 2);
    //cout << "Bordered viewport is now " << borderedViewport.toString() << endl;

	return borderedViewport;
}

// The file will be a text file containing file names for the individual 
//	corners and legs to be used in the border, one filename per line. 
//	Order will be northwest corner, northeast corner, southeast corner, 
//	southwest corner, 
void Border::load (string definitionFile){
	if (definitionFile == ""){
		string message = "Border::load() got a '' definition file!";
		cout << message << endl;
		throw (message);
	}
	resetCorners(); resetLegs();
	ifstream inputFile;
	inputFile.open (definitionFile.c_str());
	string nwCorner, neCorner, seCorner, swCorner, nLeg, eLeg, sLeg, wLeg;
	inputFile >> nwCorner;
	inputFile >> neCorner;
	inputFile >> seCorner;
	inputFile >> swCorner;
	inputFile >> nLeg;
	inputFile >> eLeg;
	inputFile >> sLeg;
	inputFile >> wLeg;
	inputFile.close();
	ResourcePool* resources = getResourcePool();
	//cout << "Border got resource pool at " << resources << endl;
	Surface* nw = new Surface(); nw->setResourcePool (resources); nw->loadBitmap (nwCorner);
	nw->setColorKeySourcePixelCoords (0, 0);
	Surface* ne = new Surface(); ne->setResourcePool (resources); ne->loadBitmap (neCorner);
	ne->setColorKeySourcePixelCoords (int (ne->getRect()->getWidth() - 1), 0);
	Surface* se = new Surface(); se->setResourcePool (resources); se->loadBitmap (seCorner);
	se->setColorKeySourcePixelCoords (int (se->getRect()->getWidth() - 1), int (se->getRect()->getHeight() - 1));
	Surface* sw = new Surface(); sw->setResourcePool (resources); sw->loadBitmap (swCorner);
	sw->setColorKeySourcePixelCoords (0, int (sw->getRect()->getHeight() - 1));
	BorderLeg* n = new BorderLeg (this, nLeg);
	n->setColorKeySourcePixelCoords (0, 0);
	BorderLeg* e = new BorderLeg (this, eLeg);
	e->setColorKeySourcePixelCoords (int (e->getSourceImageDimensions().getWidth() - 1), 0);
	BorderLeg* s = new BorderLeg (this, sLeg);
	s->setColorKeySourcePixelCoords (0, int (s->getSourceImageDimensions().getHeight() - 1));
	BorderLeg* w = new BorderLeg (this, wLeg);
	w->setColorKeySourcePixelCoords (0, 0);
	setCorners (nw, ne, se, sw);
	setLegs (n, e, s, w);
}

bool Border::update (unsigned int timePassed){
	//cout << "Border updating!" << endl;
	if (legsSet && cornersSet){
		placeElements();
		bool changed = legs [0]->update (timePassed);
		changed |= legs [1]->update (timePassed);
		changed |= legs [2]->update (timePassed);
		changed |= legs [3]->update (timePassed);
		changed |= corners [0]->update (timePassed);
		changed |= corners [1]->update (timePassed);
		changed |= corners [2]->update (timePassed);
		changed |= corners [3]->update (timePassed);
		return changed;
	}
	return false;
}

void Border::getDirtyRects (const Rect& perimeter, vector<Rect>& runningList){
	//vector<Rect> cheat; cheat.push_back (Rect (0, 0, 640, 480)); return cheat;
	//cout << "Border getting dirty rects!" << endl;
	if (legsSet && cornersSet){
		//cout << "Legs set and corners set. Getting dirties." << endl;
		vector<Rect>::iterator iter;
		vector<Rect> nextList;

		legs [0]->getDirtyRects (perimeter, runningList);
		//cout << "Leg0. Now have " << returnVector.size() << " dirties." << endl;
		legs [1]->getDirtyRects (perimeter, runningList);
		//cout << "Leg1. Now have " << returnVector.size() << " dirties." << endl;
		legs [2]->getDirtyRects (perimeter, runningList);
		//cout << "Leg2. Now have " << returnVector.size() << " dirties." << endl;
		legs [3]->getDirtyRects (perimeter, runningList);
		//cout << "Leg3. Now have " << returnVector.size() << " dirties." << endl;
		corners [0]->getDirtyRects (perimeter, runningList);
		//cout << "Corner0. Now have " << returnVector.size() << " dirties." << endl;
		corners [1]->getDirtyRects (perimeter, runningList);
		//cout << "Corner1. Now have " << returnVector.size() << " dirties." << endl;
		corners [2]->getDirtyRects (perimeter, runningList);
		//cout << "Corner2. Now have " << returnVector.size() << " dirties." << endl;
		corners [3]->getDirtyRects (perimeter, runningList);
		//cout << "Corner3. Now have " << returnVector.size() << " dirties." << endl;
	}
	//cout << "Border returns " << returnVector.size() << " dirties in total." << endl;
}

ResourcePool* Border::getResourcePool(){
	if (!resources && owner) resources = owner->getResourcePool(); 
	return resources;
}

void Border::resetLegs(){
	if (legs [0]) delete legs [0];
	if (legs [1]) delete legs [1];
	if (legs [2]) delete legs [2];
	if (legs [3]) delete legs [3];
	legs [0] = legs [1] = legs [2] = legs [3] = NULL;
	legsSet = false;
	minimumFrameThickness = 0;
	maximumFrameThickness = 0;
}
void Border::resetCorners(){	
	if (corners [0]) delete corners [0];
	if (corners [1]) delete corners [1];
	if (corners [2]) delete corners [2];
	if (corners [3]) delete corners [3];
	corners [0] = corners [1] = corners [2] = corners [3] = NULL;
	cornersSet = false;
}

int Border::getFrameThickness(){
	return minimumFrameThickness;
}
int Border::getMaximumFrameThickness(){
	return maximumFrameThickness;
}

void Border::placeElements(){
	if (!legsSet || !cornersSet) return;

	Rect& perimeter = *(getRect());

	int maxThick = getMaximumFrameThickness();
	Rect myViewport = getViewport();
	if (myViewport.getWidth() < maxThick || myViewport.getHeight() < maxThick){
		corners [0]->setVisible (false);
		corners [1]->setVisible (false);
		corners [2]->setVisible (false);
		corners [3]->setVisible (false);
		return;
	}
	else{
		corners [0]->setVisible (true);
		corners [1]->setVisible (true);
		corners [2]->setVisible (true);
		corners [3]->setVisible (true);
	}

	//cout << "Border is placing elements using perimeter " << perimeter.toString() << endl;
	//cout << "Viewport is " << myViewport.toString() << endl;

	double x1 = perimeter.getX();
	double xSpan = perimeter.getWidth();
	double x2 = x1 + xSpan;
	xSpan -= (maxThick << 1);
	//cout << "x1, x2, xSpan: " << x1 << " " << x2 << " " << xSpan << endl;
	double y1 = perimeter.getY();
	double ySpan = perimeter.getHeight();
	double y2 = y1 + ySpan;
	ySpan -= (maxThick << 1);

	if (xSpan < 1 || ySpan < 1) return;

	SDL_Rect csid [4];
	csid [0] = corners [0]->getRect()->toSDL_Rect();
	csid [1] = corners [1]->getRect()->toSDL_Rect();
	csid [2] = corners [2]->getRect()->toSDL_Rect();
	csid [3] = corners [3]->getRect()->toSDL_Rect();

	corners [0]->setXY (x1 - csid [0].w + maxThick, y1 - csid [0].h + maxThick);
	corners [1]->setXY (x2 - maxThick, y1 - csid [1].h + maxThick);
	corners [2]->setXY (x2 - maxThick, y2 - maxThick);
	corners [3]->setXY (x1 - csid [3].w + maxThick, y2 - maxThick);

	SDL_Rect sid [4];
	sid [0] = legs [0]->getSourceImageDimensions().toSDL_Rect();
	sid [1] = legs [1]->getSourceImageDimensions().toSDL_Rect();
	sid [2] = legs [2]->getSourceImageDimensions().toSDL_Rect();
	sid [3] = legs [3]->getSourceImageDimensions().toSDL_Rect();

	//legs [0]->setXY (x1 + maxThick, y1 - sid [0].h + maxThick + 10);
	legs [0]->setXY (x1 + maxThick, y1 - sid [0].h + maxThick);
	legs [0]->setWidth (xSpan);
	legs [0]->setHeight (sid [0].h);
	legs [1]->setXY (x2 - maxThick, y1 + maxThick);
	legs [1]->setWidth (sid [1].w);
	legs [1]->setHeight (ySpan);
	legs [2]->setXY (x1 + maxThick, y2 - maxThick);
	legs [2]->setWidth (xSpan);
	legs [2]->setHeight (sid [2].h);
	legs [3]->setXY (x1 - sid [3].w + maxThick, y1 + maxThick);
	legs [3]->setWidth (sid [3].w);
	legs [3]->setHeight (ySpan);

	//cout << "Elements placed!" << endl;
}

void Border::draw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride){
	//cout << "Border drawing!" << endl;
	if (legsSet && cornersSet){
		corners [0]->draw (manager, screen);
		//cout << "Corner 0 drawing at " << corners [0]->getRect()->toString() << endl;
		corners [1]->draw (manager, screen);
		//cout << "Corner 1 drawing at " << corners [1]->getRect()->toString() << endl;
		corners [2]->draw (manager, screen);
		//cout << "Corner 2 drawing at " << corners [2]->getRect()->toString() << endl;
		corners [3]->draw (manager, screen);
		//cout << "Corner 3 drawing at " << corners [3]->getRect()->toString() << endl;

	/*
		cout << "Telling top leg to draw using these clipboxes: " << endl;
		for (vector<Rect>::iterator iter = clipBoxes.begin(); iter != clipBoxes.end(); iter++){
			cout << (*iter)->toString() << endl;
		}
	*/
		//legs [0]->setName ("shithead");
		//vector<Rect> cheat; cheat.push_back (Rect (0, 0, 640, 480));
		legs [0]->draw (manager, screen);
		//legs [0]->draw (manager, cheat);
		//cout << "Leg 0 drawing at " << legs [0]->getRect()->toString() << endl;
		legs [1]->draw (manager, screen);
		//cout << "Leg 1 drawing at " << legs [1]->getRect()->toString() << endl;
		legs [2]->draw (manager, screen);
		//cout << "Leg 2 drawing at " << legs [2]->getRect()->toString() << endl;
		legs [3]->draw (manager, screen);
		//cout << "Leg 3 drawing at " << legs [3]->getRect()->toString() << endl;
	}
}

void Border::setCorners (Surface* nw, Surface* ne, Surface* se, Surface* sw){
	resetCorners();
	corners [0] = nw; corners [1] = ne; corners [2] = se; corners [3] = sw;
	if (nw && ne && se && sw) cornersSet = true;
}
void Border::setLegs (BorderLeg* n, BorderLeg* e, BorderLeg* s, BorderLeg* w){
	resetLegs();
	legs [0] = n; legs [1] = e; legs [2] = s; legs [3] = w;
	if (n && e && s && w) legsSet = true;
	updateMinimumFrameThickness();
	updateMaximumFrameThickness();
}

void Border::updateMinimumFrameThickness(){
	if (legsSet){
		minimumFrameThickness = legs [0]->getFrameThickness();
		if (legs [1]->getFrameThickness() < minimumFrameThickness) minimumFrameThickness = legs [1]->getFrameThickness();
		if (legs [2]->getFrameThickness() < minimumFrameThickness) minimumFrameThickness = legs [2]->getFrameThickness();
		if (legs [3]->getFrameThickness() < minimumFrameThickness) minimumFrameThickness = legs [3]->getFrameThickness();
	}
}
void Border::updateMaximumFrameThickness(){
	if (legsSet){
		maximumFrameThickness = legs [0]->getFrameThickness();
		if (legs [1]->getFrameThickness() > maximumFrameThickness) maximumFrameThickness = legs [1]->getFrameThickness();
		if (legs [2]->getFrameThickness() > maximumFrameThickness) maximumFrameThickness = legs [2]->getFrameThickness();
		if (legs [3]->getFrameThickness() > maximumFrameThickness) maximumFrameThickness = legs [3]->getFrameThickness();
	}
}
