/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BORDER_H
#define BORDER_H

#include <iostream>
#include "utils.h"
#include "rect.h"
#include "borderLeg.h"
#include <vector>


using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
class ImageProcessingSurfaceManager;
class ResourcePool;
class Surface;
class BorderLeg;
class Border{
    public:
        Border (Surface* owner, string definition);
        virtual ~Border();

		// Yuck! I need all three special methods that surface uses but this is NOT a surface!
		//	Have I missed the boat here or do I indeed need Border to be a separate class?
		virtual void draw (ImageProcessingSurfaceManager* manager, Surface* screen, const vector<Rect*>* dirtyRectsOverride = NULL);
		virtual bool update (unsigned int timePassed);
		virtual void getDirtyRects (const Rect& perimeter, vector<Rect>& runningList);
		ResourcePool* getResourcePool();
		void buildPerimeter (vector<Rect>& surrounded);
		Rect getViewport();
		Rect* getRect() const;

		void load (string definitionFile);

	protected:
		void setCorners (Surface* nw, Surface* ne, Surface* se, Surface* sw);
		void setLegs (BorderLeg* n, BorderLeg* e, BorderLeg* s, BorderLeg* w);
		Surface* getOwner();
		void updateMinimumFrameThickness();
		void updateMaximumFrameThickness();
		void resetCorners();
		void resetLegs();
		void placeElements();

		bool legsSet, cornersSet;
		int minimumFrameThickness;
		int maximumFrameThickness;

		int getFrameThickness();
		int getMaximumFrameThickness();

		Surface* corners [4];
		BorderLeg* legs [4];

		Surface* owner;

		Rect* rect;

		ResourcePool* resources;
};


#endif
