#include "camera.h"
#include "twoDimensionalTransition.h"
#include "interface_cameraEventObserver.h"
#include <vector>
#include <iostream>

#include <algorithm>

using namespace std;


Camera::Camera(){ 
}
Camera::~Camera(){
	for (vector<TwoDimensionalTransition*>::iterator iter = transitions.begin(); iter != transitions.end(); iter++){
		delete *iter;
	}
	transitions.clear();
}

void Camera::clearTransitions(){
	for (vector<TwoDimensionalTransition*>::iterator iter = transitions.begin(); iter != transitions.end(); iter++){
		delete *iter;
	}
	transitions.clear();
}

bool Camera::update (unsigned int timePassed){
	//cout << "camera updating" << endl;
	if (transitions.size() > 0){
		bool updated = transitions [0]->update (timePassed);
		lastX = transitions [0]->getX();
		lastY = transitions [0]->getY();
		cleanupTransitions();
		if (transitions.size() < 1){
			for (vector<Interface_CameraEventObserver*>::iterator iter = observers.begin(); iter != observers.end(); iter++) (*iter)->transitionsComplete (this);
		}
		return updated;
	}
	return false;
}

double Camera::getX() const{
	if (transitions.size() > 0) return transitions [0]->getX();
	return lastX;
}
double Camera::getY() const{
	if (transitions.size() > 0) return transitions [0]->getY();
	return lastY;
}
double Camera::getFinalX() const{
	if (transitions.size() > 0) return transitions [transitions.size() - 1]->getEndX();
	return lastX;
}
double Camera::getFinalY() const{
	if (transitions.size() > 0) return transitions [transitions.size() - 1]->getEndY();
	return lastY;
}

void Camera::cleanupTransitions(){
	if (transitions.size() > 0){
		if (!(transitions [0]->getLive())){
			transitions.erase (transitions.begin());
		}
	}
}

bool Camera::addTransition (TwoDimensionalTransition* transition){
	vector<TwoDimensionalTransition*>::iterator iter = find (transitions.begin(), transitions.end(), transition);
	if (iter != transitions.end()) return false;
	transitions.push_back (transition);
	return true;
}
bool Camera::removeTransition (TwoDimensionalTransition* transition){
	vector<TwoDimensionalTransition*>::iterator iter = find (transitions.begin(), transitions.end(), transition);
	if (iter == transitions.end()) return false;
    transitions.erase (iter);
	return true;
}

bool Camera::addObserver (Interface_CameraEventObserver* observer){
	if (find (observers.begin(), observers.end(), observer) != observers.end()) return false; 
	observers.push_back (observer);
	return true;
}
bool Camera::removeObserver (Interface_CameraEventObserver* observer){
	vector<Interface_CameraEventObserver*>::iterator iter = find (observers.begin(), observers.end(), observer);
	if (iter == observers.end()) return false;
	observers.erase (iter);
	return true;
}
