/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "characterBox.h"
#include "effectShake.h"
#include "border.h"
#include "text.h"
#include <fstream>


CharacterBox::CharacterBox (Surface* parent, const string& definition, const int& x, const int& y, const int& width, const int& height, 
							const string& message) : Wallpaper (parent){
	currentMessage = message;
	timeSpentOnCurrentMessage = 0;
	currentMode = NOMODE;
	definitionFilename = definition;
	ifstream defFile;
	defFile.open (definition.c_str());
	string wallpaperDefFilename; string borderDefFilename;
	string characterAnimationFilename; string textFontFilename;
	bool hasBorder = false;
	defFile >> wallpaperDefFilename;
	defFile >> characterAnimationFilename;
	defFile >> hasBorder;
	if (hasBorder){
		defFile >> borderDefFilename;
		new Border (this, borderDefFilename);
	}
	defFile >> textFontFilename;
	defFile >> msLetterDelay;
	defFile >> idleAnimationIndex;
	defFile >> talkingAnimationIndex;
	defFile.close();

	Wallpaper::init (wallpaperDefFilename, x, y, width, height);

	int margin = 10;

	characterSprite = new Sprite (this, characterAnimationFilename);
	characterSprite->setX (margin);
	characterSprite->setY (margin);
	characterSprite->loadAnimations (characterAnimationFilename);
	characterSprite->addAnimationObserver (this);
	characterSprite->setAnimationTimeLength (getIdleAnimation(), 1000);
	characterSprite->setAnimationTimeLength (getTalkingAnimation(), 1000);
	//cout << "CharacterBox::CharacterBox() new character sprite has me as an observer? " << characterSprite->hasAnimationObserver (this) << endl;
	//cout << "  my characterSprite is at " << characterSprite << endl;

	int characterSpriteWidth = width / 3;

	textControl = NULL;

	textControl = new Text (this, textFontFilename, "");
	textControl->setX (characterSpriteWidth + margin);
	textControl->setY (margin);

	EffectShake* shaker = new EffectShake();
	shaker->update (0);
	addVisualEffect (shaker);

	//cout << "Setting my name." << endl;
	//cout << "I am at " << this << endl;
	//name = "CharacterBox";

	say (message);
}
CharacterBox::~CharacterBox()
{
}

unsigned int CharacterBox::getTotalCurrentMessageLength() const{
	return msLetterDelay * currentMessage.length();
}

void CharacterBox::animationComplete (Surface* caller, AnimationType animation){
	//cout << "CharacterBox::animationComplete (" << caller << ", " << animation << "). characterSprite is " << characterSprite << endl;
	Wallpaper::animationComplete (caller, animation);
	if (caller == characterSprite){
		switch (getCurrentMode()){
			case IDLE:
				//cout << "playing idle 1" << endl;
				if (caller->playAnimationFromStart (getIdleAnimation())){
					//cout << "done playing idle 1" << endl;
				}
				else{
					cout << "could not play idle 1!" << endl;
				}
				break;
			case TALKING:
				//cout << "playing talking 1 now!" << endl;
				if (caller->playAnimationFromStart (getTalkingAnimation())){
					//cout << "done playing talking 1" << endl;
				}
				else{
					cout << "could not play talking 1!" << endl;
				}
				break;
		}
	}
}

bool CharacterBox::update (unsigned int millisecondsPassed){
	//cout << "CharacterBox::update() still an observer of character sprite? " << characterSprite->hasAnimationObserver (this) << endl;
	if (getCurrentMode() == TALKING){
		if (timeSpentOnCurrentMessage < getTotalCurrentMessageLength()){
			timeSpentOnCurrentMessage += millisecondsPassed;
			if (textControl){
				unsigned int currentLetter = min (currentMessage.length(), (timeSpentOnCurrentMessage / msLetterDelay));
				//cout << "CharacterBox::update() current letter is " << currentLetter << endl;
				textControl->setText (currentMessage.substr (0, currentLetter + 1));
			}
		}
		else{
			setCurrentMode ((unsigned int) IDLE);
		}
	}
	bool goodOrBad = Wallpaper::update (millisecondsPassed);
	return goodOrBad;
}

void CharacterBox::setCurrentMode (const unsigned int& newMode){
	if (newMode == getCurrentMode()) return;
	currentMode = newMode;
	switch (currentMode){
		case IDLE:
			//cout << "playing idle 2" << endl;
			if (characterSprite->playAnimationFromStart (getIdleAnimation())){
				//cout << "done playing idle 2" << endl;
			}
			else{
				cout << "could not play idle 2!" << endl;
			}
			break;
		case TALKING:
			//cout << "playing talking 2" << endl;
			if (characterSprite->playAnimationFromStart (getTalkingAnimation())){
				//cout << "done playing talking 2" << endl;
			}
			else{
				cout << "could not play talking 2!" << endl;
			}
			break;
	}
}

void CharacterBox::say (const string& newMessage){
	setCurrentMode ((unsigned int) TALKING);
	timeSpentOnCurrentMessage = 0;
	currentMessage = newMessage;
}

Sprite* CharacterBox::getCharacterSprite(){
	return characterSprite;
}
Text* CharacterBox::getTextControl(){
	return textControl;
}
