/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CHARACTERBOX_H
#define CHARACTERBOX_H

#include "wallpaper.h"
#include "sprite.h"

using namespace std;


/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
class Text;
class CharacterBox : public Wallpaper{
    public:
        CharacterBox (Surface* parent, const string& definition, const int& x, const int& y, const int& width, const int& height, 
					  const string& message = "");
        virtual ~CharacterBox();

		void setIdleAnimation (const AnimationType& index){ idleAnimationIndex = index; }
		AnimationType getIdleAnimation() const{ return idleAnimationIndex; }
		void setTalkingAnimation (const AnimationType& index){ talkingAnimationIndex = index; }
		AnimationType getTalkingAnimation() const{ return talkingAnimationIndex; }

		void say (const string& newMessage);

		Sprite* getCharacterSprite();
		Text* getTextControl();

		bool update (unsigned int millisecondsPassed);

		void animationComplete (Surface* caller, AnimationType animation);

	protected:
		string definitionFilename;
		Sprite* characterSprite;
		Text* textControl;

		AnimationType idleAnimationIndex;
		AnimationType talkingAnimationIndex;

		unsigned int getTotalCurrentMessageLength() const;

		unsigned int getCurrentMode() const{ return currentMode; }
		void setCurrentMode (const unsigned int& newMode);

		unsigned int timeSpentOnCurrentMessage;
		string currentMessage;

		unsigned int msLetterDelay;

		unsigned int currentMode;
		static const unsigned int NOMODE = 0;
		static const unsigned int IDLE = 1 << 0;
		static const unsigned int TALKING = 1 << 1;
};

#endif
