/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,								   *
 *   easlern@easlern-laptop												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "dialogueBox.h"
#include "border.h"
#include "text.h"
#include "sprite.h"
#include "wallpaper.h"
#include <fstream>

using namespace std;


DialogueBox::DialogueBox (Surface* parent, const string& definition, const double& x, const double& y, const int& xMargin, const int& yMargin,
		const string& message, const Rect& source) : Surface (parent){
	const static string me = "[DialogueBox::DialogueBox]";
	//cout << me << endl;

	this->xMargin = xMargin; this->yMargin = yMargin;
	this->source = source;
	definitionFilename = definition;
	currentMessage = message;
	ifstream defFile;
	defFile.open (definition.c_str());
	string wallpaperDefFilename; string borderDefFilename;
	string textFontFilename; string arrowThingAnimationDefs;
	defFile >> wallpaperDefFilename;
	defFile >> borderDefFilename;
	defFile >> textFontFilename;
	defFile >> arrowThingAnimationDefs;
	defFile.close();

	//cout << "setting up arrow thing" << endl;
	arrowThing = new Sprite (this);
	arrowThing->loadAnimations (arrowThingAnimationDefs);
	arrowThing->setAnimationTimeLength (DIALOGUE_BOX_ARROW_LEFT, 30000);
	arrowThing->setAnimationTimeLength (DIALOGUE_BOX_ARROW_RIGHT, 30000);
	arrowThing->playAnimation (DIALOGUE_BOX_ARROW_LEFT, true);

	//cout << "setting up text" << endl;
	textControl = new Text (this, textFontFilename, message);
	textControl->setX (xMargin);
	textControl->setY (yMargin);
	textControl->setZ (10);

	//cout << "setting up background" << endl;
	background = new Wallpaper (this, wallpaperDefFilename, 0, 0, textControl->getRect()->getWidth() + (xMargin * 2), 
								textControl->getRect()->getHeight() + (yMargin * 2), false);
	backgroundBorder = new Border (background, borderDefFilename);

	setXY (x, y - textControl->getRect()->getHeight());
	refreshArrowThingLocation();

	//cout << "Setting my name." << endl;
	//cout << "I am at " << this << endl;
	//name = "DialogueBox";
	//cout << "/" << me << endl;
}
DialogueBox::~DialogueBox()
{
}

void DialogueBox::setX (double x){
	Surface::setX (x);
	refreshArrowThingLocation();
}
void DialogueBox::setY (double y){
	Surface::setY (y - textControl->getRect()->getHeight());
	refreshArrowThingLocation();
}
void DialogueBox::setZ (double z){
	Surface::setZ (z);
	refreshArrowThingLocation();
}
void DialogueBox::refreshArrowThingLocation(){
	getRect()->setWidth (backgroundBorder->getRect()->getWidth());
	getRect()->setHeight (backgroundBorder->getRect()->getHeight() + arrowThing->getHeight());
	arrowThing->setY (background->getY() + backgroundBorder->getRect()->getHeight());
	double sourceLeft = source.getX();
	double sourceRight = sourceLeft + source.getWidth();
	//cout << "sourceLeft, sourceRight, X, backgroundWidth: " << sourceLeft << "," << sourceRight << "," << getX() << "," << background->getWidth() << endl;
	if (sourceRight < getX()){
		//cout << "arrow condition 1" << endl;
		arrowThing->playAnimation (DIALOGUE_BOX_ARROW_LEFT, true);
		arrowThing->setX (0);
	}
	else if (sourceLeft > getX() + background->getWidth()){
		//cout << "arrow condition 2" << endl;
		arrowThing->playAnimation (DIALOGUE_BOX_ARROW_RIGHT, true);
		arrowThing->setX (background->getWidth() - arrowThing->getWidth());
	}
	else if (sourceRight < getX() + background->getWidth() - arrowThing->getWidth()){
		//cout << "arrow condition 3" << endl;
		arrowThing->playAnimation (DIALOGUE_BOX_ARROW_LEFT, true);
		arrowThing->setX (sourceRight - getX());
	}
	else{
		//cout << "arrow condition 4" << endl;
		arrowThing->playAnimation (DIALOGUE_BOX_ARROW_RIGHT, true);
		arrowThing->setX (sourceLeft - getX() - arrowThing->getWidth());
	}
}
