#ifndef EFFECT_H
#define EFFECT_H


#include "rect.h"
#include "interface_effectObserver.h"
#include <vector>
#include "utils.h"

using namespace std;


class ImageProcessingSurfaceManager;
class Interface_EffectObserver;
class Effect{
	public:
		Effect (int timeLength = -1, double startX = 0.0, double startY = 0.0);
		Effect (const Effect& model);
		virtual ~Effect(){}

		virtual Effect* clone() = 0;

		virtual bool update (int timePassed);

		virtual void setActive (const bool& shouldBeActive){ active = shouldBeActive; }
		virtual bool getActive() const{ return active; }

		virtual double getX(){ return getStartX(); }
		virtual double getY(){ return getStartY(); }
		bool getLive(){ return timeLength == -1 || timePassed < timeLength; }

		virtual void processImagePostDraw (ImageProcessingSurfaceManager* manager){};
		virtual void processImage (ImageProcessingSurfaceManager* manager){};

		virtual vector< Rect > getViewports (Rect& absoluteSourceRect);
		virtual vector< Rect > getOcclusionports (Rect& absoluteSourceRect);

		void addObserver (Interface_EffectObserver* observer);
		void removeObserver (Interface_EffectObserver* observer);

		enum Quality{ qualityLow, qualityMedium, qualityHigh };
		void setQuality (const Quality& quality){ quality_ = quality; }
		Quality getQuality() const{ return quality_; }

	protected:
		double getStartX() const{ return startX; }
		double getStartY() const{ return startY; }

		virtual int getTimePassed() const{ return timePassed; }
		virtual int getTimeLength() const{ return timeLength; }

		bool hasObserver (Interface_EffectObserver* observer);
		void notifyObserversOfCompletion();

	private:
		int timePassed;
		int timeLength;
		double startX;
		double startY;

		Quality quality_;
		
		bool active;

		vector< Interface_EffectObserver* > observers;
};


#endif
