#include "effectFadeInOut.h"
#include <iostream>
#include "utils.h"
#include "imageProcessingSurfaceManager.h"

using namespace std;


void EffectFadeInOut::processImage (ImageProcessingSurfaceManager* manager){

	Rect effectRect = manager->getBlittingRect();

	//cout << "blitting rect is " << effectRect.toString() << endl;

	for (int y = int (effectRect.getY()); y < int (effectRect.getY() + effectRect.getHeight()); y++){
		for (int x = int (effectRect.getX()); x < int (effectRect.getX() + effectRect.getWidth()); x++){
			if (manager->compositePixelIsTransparent (x, y)){ 
				//cout << "transparent";
				continue;
			}

			Uint16 targetPixel = manager->getTargetPixel (x, y);
			Uint16 sourcePixel = manager->getCompositePixel (x, y);

			double sourceWeight = 1.0;
			int fadeInSeconds = int (getFadeInTime() * 1000);
			int displaySeconds = int (getDisplayTime() * 1000);
			int fadeOutSeconds = int (getFadeOutTime() * 1000);

			if (getTimePassed() > (fadeInSeconds + displaySeconds)){
				sourceWeight = 1 - ( (getTimePassed() - fadeInSeconds - displaySeconds) * 1.0 / fadeOutSeconds );
			}
			if (getTimePassed() < fadeInSeconds) sourceWeight = getTimePassed() * 1.0 / fadeInSeconds;

			double sourceWeightRecip = 1.0 - sourceWeight;

			// Linear interpolation between faded and solid pixel weights
			double pixelWeight = min (1.0, (sourceWeight * solidWeight_) + (sourceWeightRecip * fadedWeight_));

			//cout << "sourceWeight is " << sourceWeight << endl;
			//cout << "time passed = " << getTimePassed() << " fade in time = " << getFadeInTime() << " display time = " 
			//	 << getDisplayTime() << endl;
			manager->setCompositePixel (x, y, Utils::linearPixelBlendRGB (targetPixel, sourcePixel, pixelWeight));
		}
	}

	//cout << endl;

}
