#ifndef EFFECTFADEINOUT_H
#define EFFECTFADEINOUT_H


#include "effect.h"

class ImageProcessingSurfaceManager;
class EffectFadeInOut : public Effect{

	public:
		EffectFadeInOut (double fadeIn = 2, double display = 4, double fadeOut = 2, double fadedWeight = 0.0, double solidWeight = 1.0)
			: Effect (int ((fadeIn + display + fadeOut) * 1000))
		{
			fadeInTime = fadeIn;
			fadeOutTime = fadeOut;
			displayTime = display;
			fadedWeight_ = max (0.0, min (1.0, fadedWeight));
			solidWeight_ = max (0.0, min (1.0, solidWeight));
		}
		EffectFadeInOut (const EffectFadeInOut& model) : Effect (int ((model.getFadeInTime() + model.getDisplayTime() + model.getFadeOutTime()) * 1000))
		{
			fadeInTime = model.getFadeInTime();
			fadeOutTime = model.getFadeOutTime();
			displayTime = model.getDisplayTime();
			fadedWeight_ = model.fadedWeight_;
			solidWeight_ = model.solidWeight_;
			update (model.getTimePassed());
		}

		virtual ~EffectFadeInOut(){}

		virtual EffectFadeInOut* clone(){ EffectFadeInOut* cloned = new EffectFadeInOut (*this); return cloned; }

		virtual void processImage (ImageProcessingSurfaceManager* manager);

		const double& getFadeInTime() const{ return fadeInTime; }
		double getFadeInTimeRemaining() const{ 
			return max (0.0, (fadeInTime * 1000) - getTimePassed());
		}
		const double& getDisplayTime() const{ return displayTime; }
		double getDisplayTimeRemaining() const{ 
			if (getFadeInTimeRemaining() != 0) return displayTime;
			return max (0.0, (displayTime * 1000) - (getTimePassed() - (fadeInTime * 1000)));
		}
		const double& getFadeOutTime() const{ return fadeOutTime; }
		double getFadeOutTimeRemaining() const{ 
			if (getDisplayTimeRemaining() != 0) return fadeOutTime;
			return max (0.0, (fadeOutTime * 1000) - (getTimePassed() - ((fadeInTime + displayTime) * 1000)));
		}

	private:
		double fadeInTime;
		double displayTime;
		double fadeOutTime;
		double fadedWeight_;
		double solidWeight_;
};


#endif
