#include "effectSoftShadowOnChildren.h"
#include <iostream>
#include <math.h>
#include "utils.h"

using namespace std;


void EffectSoftShadowOnChildren::getDirtyRects (Rect& absoluteSourceRect, vector<Rect>& runningList){
	for (vector<Rect>::iterator iter = dirtied.begin(); iter != dirtied.end(); iter++){
		runningList.push_back (*iter);
	}
}

bool EffectSoftShadowOnChildren::update (int timePassed){
	return Effect::update (timePassed);
}

void EffectSoftShadowOnChildren::processPreCompositeImage (const SDL_Surface* screenSurface, const Rect& screenRect,
														   SDL_Surface* processorSurface, const Rect& processorRect,
														   const vector<Rect*>& screenDirties){
	if (mySurfaceWithoutChildren && 
		(
			mySurfaceWithoutChildren->w != processorSurface->w
			|| mySurfaceWithoutChildren->h != processorSurface->h
		)
	){
		SDL_FreeSurface (mySurfaceWithoutChildren);
		mySurfaceWithoutChildren = NULL;
	}

	if (!mySurfaceWithoutChildren) mySurfaceWithoutChildren = Utils::createSDL_SurfaceLike (processorSurface);

	// Copy the preComposite image over to our mySurfaceWithoutChildren.
	SDL_FillRect (mySurfaceWithoutChildren, NULL, mySurfaceWithoutChildren->format->colorkey);
	SDL_BlitSurface (processorSurface, NULL, mySurfaceWithoutChildren, NULL);
}

void EffectSoftShadowOnChildren::processPostCompositeImage (const SDL_Surface* screenSurface, const Rect& screenRect, 
												  			SDL_Surface* processorSurface, const Rect& processorRect,
												  			const vector<Rect*>& screenDirties){

	/*
	cout << "EffectSoftShadow::processPostCompositeImage has screenRect " << screenRect.toString() 
		 << " processor rect " << processorRect.toString() << endl;
	cout << "pixel 1 in top-left corner of processorRect is " 
		 << Utils::getPixel (processorSurface, int (processorRect.getX()), int (processorRect.getY())) << endl;
	*/

	if (! screenRect.intersects (processorRect)) return;

	dirtied.clear();

	int offsetX = 5;
	int offsetY = 5;

	Rect effectRect = processorRect;
	effectRect.setWidth (effectRect.getWidth() + offsetX);
	effectRect.setHeight (effectRect.getHeight() + offsetY);
	effectRect.trimTo (screenRect);
	if (effectRect.getWidth() < 1 || effectRect.getHeight() < 1) return;

	int x1 = int (effectRect.getX()); int x2 = x1 + int (effectRect.getWidth());
	int y1 = int (effectRect.getY()); int y2 = y1 + int (effectRect.getHeight());

	Uint32 processorColorKey = Utils::getColorKey (processorSurface);

	Uint16 screenPixel;

	bool* shadowed = (bool*) calloc (processorSurface->w * processorSurface->h, sizeof (bool));
	memset (shadowed, false, sizeof (bool) * processorSurface->w * processorSurface->h);

	for (int y = y1; y < y2 - offsetY; y++){
		for (int x = x1; x < x2 - offsetX; x++){

			// Ignore source pixels that don't belong to children.
			if (Utils::getPixel (processorSurface, x, y) == Utils::getPixel (mySurfaceWithoutChildren, x, y)) continue;
			// Ignore destination pixels that belong to children.
			if (Utils::getPixel (processorSurface, x + offsetX, y + offsetY) != Utils::getPixel (mySurfaceWithoutChildren, x + offsetX, y + offsetY)) continue;
			// Ignore source pixels that are transparent.
			if (Utils::getPixel (processorSurface, x, y) == processorColorKey) continue;
			// Ignore source pixels we've already shadowed.
			if (shadowed [y * processorSurface->w + x]) continue;

			screenPixel = Utils::getPixel (processorSurface, x + offsetX, y + offsetY);

			Utils::setPixel (processorSurface, x + offsetX, y + offsetY, Utils::linearPixelBlendRGB (screenPixel, 0x00, 0.5));
			shadowed [(y + offsetY) * processorSurface->w + x + offsetX] = true;
	
			/*
			for (int z = 0; z < 20; z++){
				Utils::setPixel (processorSurface, x1 + z, int (y), 0xff00);
			}
			*/
			
			//cout << "effectSoftShadowOnChildren: x,y: " << x << "," << y << endl;

			/*
			for (int z = 0; z < 20; z++){ 
				Utils::setPixel (processorSurface, int (effectRect.getX()) + z, int (effectRect.getY() + 100), 0xff00);
				Utils::setPixel (processorSurface, int (effectRect.getX()) + z, int (effectRect.getY() + 200), 0xff00);
				Utils::setPixel (processorSurface, int (effectRect.getX()) + z, int (effectRect.getY() + 300), 0xff00);
				Utils::setPixel (processorSurface, int (effectRect.getX()) + z, int (effectRect.getY() + 400), 0xff00);
			}
			*/
		}
	}

	free (shadowed); shadowed = NULL;

	//SDL_BlitSurface (mySurfaceWithoutChildren, NULL, processorSurface, NULL);

	dirtied.push_back (effectRect);

}
