#include "event.h"

#include "interface_eventController.h"
#include "interface_eventTrigger.h"
#include "interface_eventResult.h"

using namespace std;


Event::~Event(){
	for (vector<Interface_EventTrigger*>::iterator iter = triggers.begin(); iter != triggers.end(); iter++){
		delete *iter;
	}
	for (vector<Interface_EventResult*>::iterator iter = results.begin(); iter != results.end(); iter++){
		delete *iter;
	}
}
Event::Event (const Event& model){
	*this = model;
}
Event& Event::operator= (const Event& model){
	if (this != &model){
		triggers.clear();
		results.clear();

		vector<Interface_EventTrigger*> modelTriggers = model.getTriggers();
		vector<Interface_EventResult*> modelResults = model.getResults();
		for (vector<Interface_EventTrigger*>::const_iterator iter = modelTriggers.begin(); iter != modelTriggers.end(); iter++){
			addTrigger ((*iter)->clone());
		}
		for (vector<Interface_EventResult*>::const_iterator iter = modelResults.begin(); iter != modelResults.end(); iter++){
			addResult ((*iter)->clone());
		}
	}
	return *this;
}
Event* Event::clone() const{
	return new Event (*this);
}

bool Event::operator== (const Event& model) const{

	vector<Interface_EventTrigger*> triggers = getTriggers();
	vector<Interface_EventTrigger*> modelTriggers = model.getTriggers();
	if (triggers.size() != modelTriggers.size()) return false;
	for (vector<Interface_EventTrigger*>::const_iterator iter = triggers.begin(); iter != triggers.end(); iter++){
		if (! model.hasTrigger (*iter)) return false;
	}
	
	vector<Interface_EventResult*> results = getResults();
	vector<Interface_EventResult*> modelResults = model.getResults();
	if (results.size() != modelResults.size()) return false;
	for (vector<Interface_EventResult*>::const_iterator iter = results.begin(); iter != results.end(); iter++){
		if (! model.hasResult (*iter)) return false;
	}

	return true;
}

bool Event::check (Interface_EventController* controller){
	for (vector<Interface_EventTrigger*>::iterator iter = triggers.begin(); iter != triggers.end(); iter++){
		if ((*iter)->check (controller)){ 
			for (vector<Interface_EventResult*>::iterator iter = results.begin(); iter != results.end(); iter++){
				(*iter)->trigger (controller);
			}
			return true;
		}
	}

	return false;
}

vector<Interface_EventTrigger*> Event::getTriggers() const{
	return triggers;
}
vector<Interface_EventResult*> Event::getResults() const{
	return results;
}

bool Event::addTrigger (Interface_EventTrigger* trigger){
	if (!hasTrigger (trigger)){ 
		triggers.push_back (trigger);
		return true;
	}
	return false;
}
bool Event::addResult (Interface_EventResult* result){
	if (!hasResult (result)){
		results.push_back (result);
		return true;
	}
	return false;
}

bool Event::hasTrigger (const Interface_EventTrigger* trigger) const{
	for (vector<Interface_EventTrigger*>::const_iterator iter = triggers.begin(); iter != triggers.end(); iter++){
		if (**iter == *trigger) return true;
	}
	return false;
}
bool Event::hasResult (const Interface_EventResult* result) const{
	for (vector<Interface_EventResult*>::const_iterator iter = results.begin(); iter != results.end(); iter++){
		if (**iter == *result) return true;
	}
	return false;
}
