#include "eventLoader.h"
#include "interface_eventController.h"
#include <fstream>
#include <iostream>
#include "event.h"
#include "eventTrigger.h"
#include "eventTriggerRequirementEqual.h"
#include "eventTriggerRequirementNotEqual.h"
#include "eventTriggerRequirementGreaterThan.h"
#include "eventTriggerRequirementGreaterThanOrEqualTo.h"
#include "eventTriggerRequirementLessThan.h"
#include "eventTriggerRequirementLessThanOrEqualTo.h"
#include "eventResultSet.h"
#include "eventVariable.h"
#include "eventValue.h"

using namespace std;


void EventLoader::load (const string& filename, Interface_EventController* controller){

	stack<string> tokens;

	ifstream file;
	try{
		file.open (filename.c_str());
	
			while (! file.eof()){
				string nextToken;
				file >> nextToken;
				//cout << "grabbed from file: " << nextToken << endl;
				if (nextToken.size() > 0 && nextToken [0] == '\"'){
					if (nextToken [nextToken.size() - 1] == '\"'){
						nextToken = nextToken.substr (1, nextToken.size() - 2);
					}
					else{
						nextToken = nextToken.substr (1);
						string follower;
						file >> follower;
						while (follower [follower.size() - 1] != '\"'){
							//cout << "grabbed from file: " << follower << endl;
							nextToken += " " + follower;
							file >> follower;
						}
						follower = follower.substr (0, follower.size() - 1);
						nextToken += " " + follower;
					}
				}
				//cout << "next token: " << nextToken << endl;
				tokens.push (nextToken);
			}
		
		file.close();
	}
	catch (...){
		cout << "Unspecified error loading event definition file!" << endl;
		file.close();
		return;
	}

	try{
		stack<string> tokensReversed;
		for (; tokens.size() > 0; tokens.pop()){
			tokensReversed.push (tokens.top());
		}

		parse (EVENT_LOADER_EVENTS, tokensReversed, controller);
		controller->refresh();
	}
	catch (...){
		cout << "Unspecified error parsing event definition file!" << endl;
		return;
	}
}

void EventLoader::parse (const int& mode, stack<string>& input, Interface_EventController* controller){

	Interface_EventTriggerRequirement* requirement = NULL;
	Interface_EventResult* result = NULL;

	switch (mode){
		case EVENT_LOADER_EVENTS:
			while (input.size() > 0){
				while (input.size() > 0 && input.top() != "("){
					input.pop();
				}
				if (input.size() < 1) return;
				currentEvent = new Event();
				currentTrigger = new EventTrigger();
				currentEvent->addTrigger (currentTrigger);
				controller->addEvent (currentEvent);
				parse (EVENT_LOADER_REQUIREMENTS, input, controller);
			}

			break;
		case EVENT_LOADER_REQUIREMENTS:
//cout << "nextToken " << input.top() << endl;
			if (input.top() == "("){
				input.pop();
				for (bool goAgain = true; goAgain == true; ){
//cout << "nextToken " << input.top() << endl;
					string variable = input.top(); input.pop();
//cout << "nextToken " << input.top() << endl;
					string req = input.top(); input.pop();
//cout << "nextToken " << input.top() << endl;
					string value = input.top(); input.pop();
					if (req == "==") requirement = new EventTriggerRequirementEqual (EventVariable (variable, controller), EventValue (value, controller));
					else if (req == "!=") requirement = new EventTriggerRequirementNotEqual (EventVariable (variable, controller), EventValue (value, controller));
					else if (req == ">") requirement = new EventTriggerRequirementGreaterThan (EventVariable (variable, controller), EventValue (value, controller));
					else if (req == "<") requirement = new EventTriggerRequirementLessThan (EventVariable (variable, controller), EventValue (value, controller));
					else if (req == ">=" || req == "=>") requirement = new EventTriggerRequirementGreaterThanOrEqualTo (EventVariable (variable, controller), EventValue (value, controller));
					else if (req == "<=" || req == "=<") requirement = new EventTriggerRequirementLessThanOrEqualTo (EventVariable (variable, controller), EventValue (value, controller));
					currentTrigger->addRequirement (requirement);
//cout << "nextToken " << input.top() << endl;
					if (input.top() != ",") goAgain = false;
					input.pop();
				}
			}
			else{ 
				cout << "Expected requirements token while parsing events file, got " << input.top() << endl;
//cout << "nextToken " << input.top() << endl;
				input.pop();
			}
			parse (EVENT_LOADER_RESULTS, input, controller);

			break;
		case EVENT_LOADER_RESULTS:
			if (input.top() == "{"){
				input.pop();
				for (bool goAgain = true; goAgain == true; ){
					string variable = input.top(); input.pop();
					string act = input.top(); input.pop();
					string value = input.top(); input.pop();
					if (act == "=") result = new EventResultSet (variable, value);
					currentEvent->addResult (result);
					if (input.top() != ",") goAgain = false;
					input.pop();
				}
			}
			else{ 
				cout << "Expected results token while parsing events file, got " << input.top() << endl;
				input.pop();
			}

			break;
		default:
			cout << "Unknown mode in EventLoader::parse()." << endl;
			break;
	}

}

