#include "eventTrigger.h"

#include "interface_eventController.h"
#include "interface_eventTriggerRequirement.h"

using namespace std;


EventTrigger::~EventTrigger(){
	for (vector<Interface_EventTriggerRequirement*>::iterator iter = requirements.begin(); iter != requirements.end(); iter++){
		delete *iter;
	}
}
EventTrigger::EventTrigger (const EventTrigger& model){
	*this = model;
}
EventTrigger::EventTrigger(){}
EventTrigger& EventTrigger::operator= (const EventTrigger& model){
	if (this != &model){
		requirements.clear();

		const vector<Interface_EventTriggerRequirement*> modelReqs = model.getRequirements();
		for (vector<Interface_EventTriggerRequirement*>::const_iterator iter = modelReqs.begin(); iter != modelReqs.end(); iter++){
			addRequirement ((*iter)->clone());
		}
	}

	return *this;
}

bool EventTrigger::addRequirement (Interface_EventTriggerRequirement* requirement){
	if (hasRequirement (requirement)) return false;
	requirements.push_back (requirement);
	return true;
}
bool EventTrigger::hasRequirement (const Interface_EventTriggerRequirement* requirement) const{
	for (vector<Interface_EventTriggerRequirement*>::const_iterator iter = requirements.begin(); iter != requirements.end(); iter++){
		if (**iter == *requirement) return true;
	}
	return false;
}
vector<Interface_EventTriggerRequirement*> EventTrigger::getRequirements() const{
	return requirements;
}

bool EventTrigger::check (const Interface_EventController* controller){
	for (vector<Interface_EventTriggerRequirement*>::iterator iter = requirements.begin(); iter != requirements.end(); iter++){
		if (!(*iter)->satisfied (controller)) return false;
	}

	return true;
}

EventTrigger* EventTrigger::clone() const{
	return new EventTrigger (*this);
}

bool EventTrigger::operator== (const EventTrigger& model) const{
	vector<Interface_EventTriggerRequirement*> modelRequirements = model.getRequirements();

	if (requirements.size() != modelRequirements.size()) return false;
	for (vector<Interface_EventTriggerRequirement*>::const_iterator iter = requirements.begin(); iter != requirements.end(); iter++){
		if (! model.hasRequirement (*iter)) return false;
	}

	return true;
}
