#include "surface.h"
#include "text.h"
#include "effectFadeInOut.h"
#include "interface_surfaceEventObserver.h"
#include "interface_effectObserver.h"
#include <string>


class FadingText : Interface_SurfaceEventObserver, Interface_EffectObserver{
	public:
		FadingText (Surface* owner, const std::string& fontFile, const std::string& message, 
					double x, double y, int fadeInTime = 0, int displayTime = 1, int fadeOutTime = 0){
			text_ = new Text (owner, fontFile, message);
			text_->setZ (10000);
			text_->setScreenX (x);
			text_->setScreenY (y);
			text_->addSurfaceEventObserver (this);
			fadeEffect_ = new EffectFadeInOut (fadeInTime, displayTime, fadeOutTime);
			fadeEffect_->addObserver (this);
			text_->addVisualEffect (fadeEffect_);
		}
		~FadingText(){
			if (fadeEffect_) fadeEffect_->removeObserver (this);
			if (text_){ 
				text_->removeSurfaceEventObserver (this);
				text_->setLive (false);
			}
		}

		void surfaceDestroying (Surface* caller){ text_ = NULL; fadeEffect_ = NULL; }
		void perspectiveTransitionsComplete (Surface* caller){ }
		void effectComplete (Effect* effect){ if (effect == fadeEffect_) text_->setLive (false); }
		
	private:
		FadingText& operator= (const FadingText& other){ return *this; }
		FadingText(){ }

		Text* text_;
		EffectFadeInOut* fadeEffect_;
};
