#include "game.h"
#include "surface.h"
#include "screen.h"
#include "beach.h"
#include "palmTree.h"
#include "player.h"
#include "basket.h"
#include "eventController.h"
#include "eventLoader.h"
#include "soundInstanceReference.h"
#include "coconut.h"
#include "utils.h"

#include <iostream>
#include <vector>
#include <sstream>
using namespace std;


Game::Game(){
	running = true;

	srand ((unsigned int) time (NULL));

	timeSinceCoconut = 0;

	basketed = 0;

	titleScreen = NULL;

	playerLabel1 = NULL;
	playerLabel2 = NULL;
	playerLabel3 = NULL;

	beachBackground = new Beach (dynamic_cast<Surface*> (&getScreenRef()));

	comboCounter1 = NULL;
	comboCounter2 = NULL;
	comboCounter3 = NULL;

	Rect perimeter (50, -200, 470, 680);
	player1 = new Player (beachBackground, perimeter); playerLabel1 = new Surface (beachBackground, "label.1.bmp");
	player2 = new Player (beachBackground, perimeter); playerLabel2 = new Surface (beachBackground, "label.2.bmp");
	player3 = new Player (beachBackground, perimeter); playerLabel3 = new Surface (beachBackground, "label.3.bmp");

	events = new EventController();
	EventLoader loader;
	loader.load ("game.events", events);

	Surface* tally = new Surface (beachBackground, "basketTally.bmp");
	int tallyX = atoi (events->getVariable ("tally_x").c_str());
	int tallyY = atoi (events->getVariable ("tally_y").c_str());
	tally->setXY (tallyX, tallyY);
	tally->setZ (1.0);

	basket = new Basket (beachBackground, perimeter);

	while (running){
		update();
		updatePlayers();
		updateTrees();
		updateBasket();
		updateCoconuts();
		updateTitles();
		draw();
	}
	cout << "Game started." << endl;
}
Game::~Game(){
	delete events; events = NULL;
	cout << "Game stopped." << endl;
}

void Game::createTrees(){
	for (unsigned int x = 0; x < trees.size(); x++){
		trees [x].first->setLive (false);
	}
	trees.clear();

	int numTrees = atoi (events->getVariable ("trees_count").c_str());
	for (int x = 0; x < numTrees; x++){
		std::ostringstream var;
		var << "tree_size_" << x;
		PalmTree* newPalm = NULL;
		if (events->getVariable (var.str()) == "small"){
			newPalm = new PalmTree (beachBackground, PALM_SMALL);
			newPalm->setZ (1.0);
		}
		else{
			newPalm = new PalmTree (beachBackground, PALM_TALL);
			newPalm->setZ (0.5);
		}
		std::ostringstream var2;
		var2 << "tree_x_" << x;
		int startX = atoi (events->getVariable (var2.str()).c_str());
		std::ostringstream var3;
		var3 << "tree_y_" << x;
		//cout << "lookup is " << var3.str() << endl;
		int startY = atoi (events->getVariable (var3.str()).c_str());
		newPalm->setXY (startX, startY);
		trees.push_back (std::pair <PalmTree*, Coconut*> (newPalm, NULL));
	}
}

void Game::update(){
	GameBase::update();
}
void Game::update (unsigned int timePassed){
	GameBase::update (timePassed);
	timeSinceCoconut += timePassed;
}

void Game::updateTitles(){
	if (events->getVariable ("intro_status") == "opening"){
		events->setVariable ("intro_status", "opened");
		titleScreen = new Surface (beachBackground, events->getVariable ("intro_bitmap"), false);
		titleScreen->setZ (10.0);
	}
	if (events->getVariable ("intro_status") == "closing"){
		events->setVariable ("intro_status", "closed");
		titleScreen->setLive (false);
		titleScreen = NULL;
	}
}

std::vector<int> Game::getCoconutCounts(){
	std::vector<int> ret;
	ret.push_back (-1);
	ret.push_back (-1);
	ret.push_back (-1);
	if (player1) ret [0] = 0;
	if (player2) ret [1] = 0;
	if (player3) ret [2] = 0;
	for (unsigned int x = 0; x < trees.size(); x++){
		if (trees [x].second){
			int id = trees [x].second->getCollector();
			ret [id]++;
		}
	}
	return ret;
}
int Game::assignCoconut (int playerToIgnore){
	int leastCount = 0;
	int leastOwner = 3;
	std::vector<int> counts = getCoconutCounts();
	if (playerToIgnore < (int) counts.size()) counts [playerToIgnore] = -1;
	int randStart = rand() % counts.size();
	for (unsigned int x = 0; x < counts.size(); x++){
		int pick = (randStart + x) % counts.size();
		if (counts [pick] == -1) continue;
		if (leastOwner == 3){
			leastOwner = pick;
			leastCount = counts [pick];
		}
		else if (counts [pick] <= leastCount){
			leastOwner = pick;
			leastCount = counts [pick];
		}
	}
	//cout << "counts: " << counts [0] << ", " << counts [1] << ", " << counts [2] << endl;
	return leastOwner;
}

void Game::updateComboCounter (int player, int value){
	//cout << "updating combo for player " << player << ", value " << value << endl;
	if (player == 0){
		if (comboCounter1){
			comboCounter1->setLive (false);
			comboCounter1 = NULL;
		}
		if (value == 1) comboCounter1 = new Surface (beachBackground, "combo1.red.bmp");
		if (value == 2) comboCounter1 = new Surface (beachBackground, "combo2.red.bmp");
		if (value == 3) comboCounter1 = new Surface (beachBackground, "combo3.red.bmp");
		int x = atoi (events->getVariable ("combo1_x").c_str());
		int y = atoi (events->getVariable ("combo1_y").c_str());
		if (comboCounter1){ 
			comboCounter1->setXY (x, y);
			comboCounter1->setZ (3.0);
		}
	}
	else if (player == 1){
		if (comboCounter2){
			comboCounter2->setLive (false);
			comboCounter2 = NULL;
		}
		if (value == 1) comboCounter2 = new Surface (beachBackground, "combo1.green.bmp");
		if (value == 2) comboCounter2 = new Surface (beachBackground, "combo2.green.bmp");
		if (value == 3) comboCounter2 = new Surface (beachBackground, "combo3.green.bmp");
		int x = atoi (events->getVariable ("combo2_x").c_str());
		int y = atoi (events->getVariable ("combo2_y").c_str());
		if (comboCounter2){ 
			comboCounter2->setXY (x, y);
			comboCounter2->setZ (3.0);
		}
	}
	else if (player == 2){
		if (comboCounter3){
			comboCounter3->setLive (false);
			comboCounter3 = NULL;
		}
		if (value == 1) comboCounter3 = new Surface (beachBackground, "combo1.blue.bmp");
		if (value == 2) comboCounter3 = new Surface (beachBackground, "combo2.blue.bmp");
		if (value == 3) comboCounter3 = new Surface (beachBackground, "combo3.blue.bmp");
		int x = atoi (events->getVariable ("combo3_x").c_str());
		int y = atoi (events->getVariable ("combo3_y").c_str());
		if (comboCounter3){ 
			comboCounter3->setXY (x, y);
			comboCounter3->setZ (3.0);
		}
	}
}

void Game::assignAllCoconutsTo (int player){
	for (unsigned int x = 0; x < trees.size(); x++){
		std::string kind;
		if (player == 0) kind = COCONUT_LARGE_RED;
		if (player == 1) kind = COCONUT_LARGE_GREEN;
		if (player == 2) kind = COCONUT_LARGE_BLUE;
		Coconut* oldNut = trees [x].second;
		if (oldNut != NULL){
			Coconut* newNut = new Coconut (beachBackground, kind);
			newNut->setPointValue (oldNut->getPointValue());
			newNut->setZ (oldNut->getZ());
			newNut->setXY (oldNut->getX(), oldNut->getY());
			coconuts.push_back (newNut);
			trees [x].second = newNut;
			killCoconut (oldNut);
		}
	}
}

void Game::updateCoconuts(){
	// Spawn new coconuts
	int spawnTime = atoi (events->getVariable ("coconut_spawn_time").c_str());
	if (spawnTime < timeSinceCoconut){
		timeSinceCoconut = 0;
		if (trees.size() > 0){
			bool placed = false;
			int randStart = rand() % trees.size();
			for (unsigned int x = 0; x < trees.size() && !placed; x++){
				int pick = (x + randStart) % trees.size();
				if (!trees [pick].second){
					int owner = assignCoconut (trees [pick].first->getLastOwner());
					placed = true;
					int stemX = trees [pick].first->getStemX();
					int stemY = trees [pick].first->getStemY();
					std::string bitmap = COCONUT_LARGE_BROWN;
					if (owner == 0) bitmap = COCONUT_LARGE_RED;
					if (owner == 1) bitmap = COCONUT_LARGE_GREEN;
					if (owner == 2) bitmap = COCONUT_LARGE_BLUE;
					Coconut* nut = new Coconut (beachBackground, bitmap);
					trees [pick].first->setLastOwner (owner);
					nut->setZ (1.5);
					stemX -= nut->getRect()->getWidth() / 2;
					nut->setXY (stemX + (rand() % 10 - 5), stemY + (rand() % 10 - 5));
					trees [pick].second = nut;
				}
			}
		}
	}
	// Look for landing coconuts
	{
		int groundY = atoi (events->getVariable ("ground_y").c_str());
		for (unsigned int x = 0; x < coconuts.size(); x++){
			if (coconuts [x]->getY() + coconuts [x]->getRect()->getHeight() > groundY
					&& !coconuts [x]->isGrounded()){
				//cout << "coconut at " << coconuts [x]->getY() << " landing" << endl;
				Player* player = NULL;
				Coconut* nut = coconuts [x];
				if (nut->getCollector() == 0) player = player1;
				if (nut->getCollector() == 1) player = player2;
				if (nut->getCollector() == 2) player = player3;
				getAudioManagerRef().playSound ("sounds/hurt1.wav");
				player->setCombo (0);
				updateComboCounter (nut->getCollector(), player->getCombo());
				coconuts [x]->land();
			}
		}
	}
	// Look for scoring coconuts
	{
		for (unsigned int x = 0; x < coconuts.size(); x++){
			if (!coconuts [x]->isGrounded() && coconuts [x]->collidesWith (basket)){
				Player* player = NULL;
				if (coconuts [x]->getCollector() == 0) player = player1;
				if (coconuts [x]->getCollector() == 1) player = player2;
				if (coconuts [x]->getCollector() == 2) player = player3;
				if (player){
					Coconut* oldNut = coconuts [x];
					if (player->getCombo() == 0) getAudioManagerRef().playSound ("sounds/hit1.wav");
					if (player->getCombo() == 1) getAudioManagerRef().playSound ("sounds/hit2.wav");
					//if (player->getCombo() == 2) getAudioManagerRef().playSound ("sounds/hit3.wav");
					if (player->getCombo() > 1){
						player->setCombo (0);
						assignAllCoconutsTo (coconuts [x]->getCollector());
						getAudioManagerRef().playSound ("sounds/combo.wav");
					}
					else{
						player->setCombo (player->getCombo() + 1);
					}
					updateComboCounter (coconuts [x]->getCollector(), player->getCombo());
					//int points = player->getCombo() + oldNut->getPointValue();
					int points = 1; //oldNut->getPointValue();
					int yLoc = atoi (events->getVariable ("scored_coconuts_row_y").c_str());
					int width = atoi (events->getVariable ("scored_coconut_width").c_str());
					int xStart = atoi (events->getVariable ("scored_coconuts_start_x").c_str());
					int speed = atoi (events->getVariable ("scored_coconut_speed").c_str());
					for (int p = 0; p < points; p++){
						basketed++;
						Coconut* nut = new Coconut (beachBackground, oldNut->getBitmap());
						nut->setSpeed (speed);
						nut->setXY (oldNut->getX(), oldNut->getY());
						nut->setZ (1.5);
						nut->headTo (xStart + width * scoredCoconuts.size(), yLoc);
						//nut->headTo (xStart + width * scoredCoconuts.size(), yLoc);
						//cout << "heading to " << xStart + width * scoredCoconuts.size() << ", " << yLoc << endl;
						scoredCoconuts.push_back (nut);
					}
					player->setScore (player->getScore() + points);
					oldNut->land();
					oldNut->setXY (-100, -100); // HAX
					//oldNut->setLive (false);
					coconuts.erase (coconuts.begin() + x); x--;
				}
			}
		}
	}
}

void Game::updateBasket(){
	int leftBound = atoi (events->getVariable ("basket_left_bound").c_str());
	int rightBound = atoi (events->getVariable ("basket_right_bound").c_str());
	basket->setLeftBound (leftBound);
	basket->setRightBound (rightBound);
}

void Game::updateTrees(){
}

void Game::updatePlayers(){
	if (events->getVariable ("game_state") == "intro" && events->getVariable ("intro_music") == ""){
		events->setVariable ("intro_music", "started");
		music = getAudioManagerRef().loadSound ("sounds/islandTitle.wav");
		int volume = atoi (events->getVariable ("music_volume_0_to_100").c_str());
		music->setVolume (volume / 100.0);
		music->play (true);
	}
	else if (events->getVariable ("game_state") == "starting"){
		events->setVariable ("game_state", "started");

		music->stop();

		int playerJumpSpeed = atoi (events->getVariable ("player_jump_speed").c_str());
		if (player1){
			int p1_x = atoi (events->getVariable ("player1_start_x").c_str());
			int p1_y = atoi (events->getVariable ("player1_start_y").c_str());
			//cout << "p1 start: " << p1_x << ", " << p1_y << endl;
			player1->setXY (p1_x, p1_y);
			player1->setZ (2.0);
			player1->setJumpSpeed (playerJumpSpeed);
		}	
		if (player2){
			int p2_x = atoi (events->getVariable ("player2_start_x").c_str());
			int p2_y = atoi (events->getVariable ("player2_start_y").c_str());
			player2->setXY (p2_x, p2_y);
			player2->setZ (2.0);
			player2->setJumpSpeed (playerJumpSpeed);
		}
		if (player3){
			int p3_x = atoi (events->getVariable ("player3_start_x").c_str());
			int p3_y = atoi (events->getVariable ("player3_start_y").c_str());
			player3->setXY (p3_x, p3_y);
			player3->setZ (2.0);
			player3->setJumpSpeed (playerJumpSpeed);
		}

		int basket_x = atoi (events->getVariable ("basket_start_x").c_str());
		int basket_y = atoi (events->getVariable ("basket_start_y").c_str());
		basket->setZ (3.0);
		basket->setXY (basket_x, basket_y);
	
		createTrees();

		music = getAudioManagerRef().loadSound ("sounds/islandLevel.wav");
		int volume = atoi (events->getVariable ("music_volume_0_to_100").c_str());
		music->setVolume (volume / 100.0);
		music->play (true);
	}
	else if (events->getVariable ("game_state") == "started" && basketed > 11){
		music->stop();
		music = getAudioManagerRef().loadSound ("sounds/islandWin.wav");
		int volume = atoi (events->getVariable ("music_volume_0_to_100").c_str());
		music->setVolume (volume / 100.0);
		music->play (false);
		setPause (true);
		events->setVariable ("game_state", "waiting_for_music");
	}
	else if (events->getVariable ("game_state") == "waiting_for_music" && music->getState() != SoundInstanceReference::statePlaying){
		bool win1 = player1->getScore() >= player2->getScore() && player1->getScore() >= player3->getScore();
		bool win2 = player2->getScore() >= player3->getScore() && player2->getScore() >= player1->getScore();
		bool win3 = player3->getScore() >= player1->getScore() && player3->getScore() >= player2->getScore();
		int count = 0; if (win1) count++; if (win2) count++; if (win3) count++;
		std::vector<int> locations;
		if (count != 2){ 
			locations.push_back (300);
		}
		if (count >= 2){
			locations.push_back (200);
			locations.push_back (400);
		}
		if (win1){
			Surface* winner = new Surface (beachBackground, "label.1.bmp");
			winner->setXY (locations [0], 200);
			winner->setZ (20.0);
			locations.erase (locations.begin());
			win1 = false;
		}
		if (win2){
			Surface* winner = new Surface (beachBackground, "label.2.bmp");
			winner->setXY (locations [0], 200);
			winner->setZ (20.0);
			locations.erase (locations.begin());
			win2 = false;
		}
		if (win3){
			Surface* winner = new Surface (beachBackground, "label.3.bmp");
			winner->setXY (locations [0], 200);
			winner->setZ (20.0);
			locations.erase (locations.begin());
			win3 = false;
		}
		setPause (false);
		events->setVariable ("game_state", "outro");
	}
	else if (events->getVariable ("game_state") == "ending"){
		running = false;
	}

	std::vector<Player*> players;
	int labelDist = 20;
	if (player1){ 
		players.push_back (player1);
		playerLabel1->setXY (player1->getCenterX() - (player1->getRect()->getWidth() - playerLabel1->getRect()->getWidth()) / 2, 
			player1->getY() - labelDist);
		playerLabel1->setZ (2.0);
	}
	if (player2){ 
		players.push_back (player2);
		playerLabel2->setXY (player2->getCenterX() - (player2->getRect()->getWidth() - playerLabel2->getRect()->getWidth()) / 2, 
			player2->getY() - labelDist);
		playerLabel2->setZ (2.0);
	}
	if (player3){
		players.push_back (player3);
		playerLabel3->setXY (player3->getCenterX() - (player2->getRect()->getWidth() - playerLabel2->getRect()->getWidth()) / 2, 
			player3->getY() - labelDist);
		playerLabel3->setZ (2.0);
	}
	for (unsigned int x = 0; x < players.size(); x++){
		int platformHeightFudge = atoi (events->getVariable ("platform_height_fudge").c_str());
		int platformWidthFudge = atoi (events->getVariable ("platform_width_fudge").c_str());
		int groundY = atoi (events->getVariable ("ground_y").c_str());
		//int platformCenterXFudge = atoi (events->getVariable ("platform_center_x_fudge").c_str());
		//cout << "platformHeightFudge is " << platformHeightFudge << endl;
		//cout << "platformWidthFudge is " << platformWidthFudge << endl;
		//cout << "platformCenterXFudge is " << platformCenterXFudge << endl;
		if (players [x]->getY() + players [x]->getRect()->getHeight() >= groundY){
			//cout << "grounding player " << x << endl;
			players [x]->setGrounded (true);
			players [x]->setY (groundY - players [x]->getRect()->getHeight());
		}
		else if (!players [x]->isJumping()){
			bool onATree = false;
			for (unsigned int t = 0; t < trees.size(); t++){
				if (players [x]->collidesWith (trees [t].first)){
					double centerDist = fabs (players [x]->getCenterX() - trees [t].first->getStemX());
					//double centerDist = fabs (players [x]->getCenterX() - (trees [t].first->getCenterX() + platformCenterXFudge));
					double footDist = fabs (players [x]->getY() + players [x]->getHeight() - trees [t].first->getY());
					//cout << "centerDist: " << centerDist << endl;
					//cout << "footDist: " << footDist << endl;
					if (footDist < platformHeightFudge && centerDist < platformWidthFudge){
						//cout << "on a tree" << endl;
						onATree = true;
					}
				}
			}
			if (players [x]->fallingFast() && onATree){
				double closestDist = 0;
				int closestIndex = -1;
				for (unsigned int z = 0; z < trees.size(); z++){
					Point treeCenter (trees [z].first->getStemX(), trees [z].first->getStemY());
					Point playerCenter (players [x]->getCenterX(), 
						players [x]->getY() + players [x]->getRect()->getHeight());
					double centerDist = Utils::get2dDistance (treeCenter, playerCenter);
					if (closestIndex == -1 || closestDist > centerDist){
						closestDist = centerDist;
						closestIndex = z;
					}
				}
				if (closestIndex > -1){
					trees [closestIndex].first->shake();
					getAudioManagerRef().playSound ("sounds/islandShake.wav");
					Coconut* oldNut = trees [closestIndex].second;
					if (oldNut != NULL){
						std::string kind = oldNut->getBitmap();
						Coconut* newNut = new Coconut (beachBackground, kind);
						newNut->setPointValue (1);
						if (trees [closestIndex].first->getSize() == PALM_TALL) newNut->setPointValue (2);
						newNut->setZ (2.5);
						newNut->setXY (oldNut->getX(), oldNut->getY());
						coconuts.push_back (newNut);
						newNut->fall();
						trees [closestIndex].second = NULL;
						killCoconut (oldNut);
					}
				}
			}
			players [x]->setGrounded (onATree);
		}
	}
}

void Game::killCoconut (Coconut* oldNut){
	oldNut->setXY (-100, -100); // No time to fix the issue!
	for (unsigned int y = 0; y < coconuts.size(); y++){
		if (coconuts [y] == oldNut){ 
			coconuts.erase (coconuts.begin() + y);
			y--;
		}
	}
	//oldNut->setLive (false);
}

void Game::inputHandlerUpdate (const KeyStatesSet& keyStates){
	if (keyStates.keyJustPressed (SDLK_ESCAPE)) running = false;
	if (getPause()) return;
	if (events->getVariable ("game_state") == "started"){
		handleGameKeys (keyStates);
	}
	else if (keyStates.keyJustPressed (SDLK_RETURN) || keyStates.keyJustPressed (SDLK_SPACE)){
		if (titleScreen) events->setVariable ("intro_status", "closing");
	}
}

void Game::handleGameKeys (const KeyStatesSet& keyStates){
	if (player1){
		if (keyStates.keyDown (SDLK_RIGHT)) player1->moveRight();
		else if (keyStates.keyDown (SDLK_LEFT)) player1->moveLeft();
		if (keyStates.keyDown (SDLK_UP)){ 
			if (player1->isGrounded()) getAudioManagerRef().playSound ("sounds/islandJump1.wav");
			player1->jump();
		}
		if (!keyStates.keyDown (SDLK_UP) && player1->isJumping()) player1->coast();
	}
	if (player2){
		if (keyStates.keyDown (SDLK_e)) player2->moveRight();
		else if (keyStates.keyDown (SDLK_a)) player2->moveLeft();
		if (keyStates.keyDown (SDLK_COMMA) && player2->isGrounded()){ 
			if (player2->isGrounded()) getAudioManagerRef().playSound ("sounds/islandJump1.wav");
			player2->jump();
		}
		if (!keyStates.keyDown (SDLK_COMMA) && player2->isJumping()) player2->coast();
	}
	if (player3){
		if (keyStates.keyDown (SDLK_l)) player3->moveRight();
		else if (keyStates.keyDown (SDLK_j)) player3->moveLeft();
		if (keyStates.keyDown (SDLK_i) && player3->isGrounded()){ 
			if (player3->isGrounded()) getAudioManagerRef().playSound ("sounds/islandJump1.wav");
			player3->jump();
		}
		if (!keyStates.keyDown (SDLK_i) && player3->isJumping()) player3->coast();
	}
}

int main (int numArgs, char** args){
	Game testGame;

	return 0;
}
