/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,                                  *
 *   easlern@easlern-laptop                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "gameBase.h"
#include "resourcePool.h"
#include "inputHandler.h"
#include "screen.h"

using namespace std;


GameBase::GameBase(){
	if (SDL_Init (SDL_INIT_VIDEO)< 0){ 
    	SDL_Quit();
		throw ("Could not initialize SDL: " + string (SDL_GetError()));
	}

	resources_ = tr1::shared_ptr<ResourcePool> (new ResourcePool());
	screen_ = tr1::shared_ptr<Screen> (new Screen());
	resources_->getInputHandler()->setFocus (this);
	screen_->setResourcePool (&(*resources_));

	lastUpdate_ = SDL_GetTicks();
	
	paused_ = false;
}
GameBase::~GameBase(){}

void GameBase::update(){
	if (paused_){ 
		update (0);
	}
	else{
		Uint32 ticks = SDL_GetTicks();
		unsigned int elapsed = ticks - lastUpdate_;
		update (elapsed);
	}
}
void GameBase::update (Uint32 elapsed){
	getInput();
	screen_->update (elapsed);
	lastUpdate_ = SDL_GetTicks();
}
void GameBase::draw(){
	screen_->draw();
}
Screen& GameBase::getScreenRef(){
	return *screen_;
}
ResourcePool& GameBase::getResourcesRef(){
	return *resources_;
}
Audio::AudioManager& GameBase::getAudioManagerRef(){
	return Audio::audioManager_;
}
bool GameBase::getPause(){ return paused_; }
void GameBase::setPause (bool pause){ paused_ = pause; }

void GameBase::getInput(){
	resources_->getInputHandler()->handleInput();
}
