#ifndef GEOMETRY_H
#define GEOMETRY_H


#include <string>
#include <vector>
#include "utilTypes.h"


class Vector2{
	public:
		Vector2 (const double& x, const double& y);
		~Vector2();

		Vector2 getNormalized();
		double getLength();
		Vector2 getInverted() const;
		double dotProduct (Vector2& otherVector);
		bool equals (const Vector2& otherVector) const;
		Vector2 getRotated (const double& angle) const;
		std::string toString() const;

		double getX() const;
		double getY() const;

	private:
		double x_;
		double y_;
		Optional<double> length_;
};

class Point{
	public:
		Point (const double& x, const double& y);
		Point();
		~Point();

		bool operator< (const Point& rhs) const;
		bool operator== (const Point& rhs) const;

		double getX() const;
		double getY() const;
		
		Vector2 vectorToOtherPoint (const Point& otherPoint) const;
		bool equals (const Point& otherPoint) const;
		std::string toString() const;

	private:
		double x_;
		double y_;
};

class Line{
	public:
		Line (const Point& point1, const Point& point2);
		Line();
		~Line();

		bool equals (const Line& otherLine) const;
		bool intersectsLine (const Line& otherLine) const;
		double getLength() const;
		double distanceToPoint (const Point& point, const double& tolerance = 0.000001) const;
		Optional<Point> getLineIntersectionPoint (const Line& otherLine) const;
		std::string toString() const;
	
		Point getPoint1() const;
		Point getPoint2() const;

	private:
		Point point1_;
		Point point2_;
};

#endif
