#ifndef IMAGEPROCESSINGSURFACEMANAGER_H
#define IMAGEPROCESSINGSURFACEMANAGER_H

#include "utils.h"
using namespace std;


struct SDL_Surface;
class Rect;
class Screen;
class ImageProcessingSurfaceManager{

	public:
		ImageProcessingSurfaceManager (SDL_Surface* originalSurface, const Rect& screenRect);
		~ImageProcessingSurfaceManager();

		void setCompositePixel (const unsigned int& x, const unsigned int& y, const Uint16& newPixel);
		void blitToComposite (SDL_Surface* sourceSurface, Rect sourceRect, Rect targetRect);
		Uint16 getCompositePixel (const unsigned int& x, const unsigned int& y) const;
		Uint16 getTargetPixel (const unsigned int& x, const unsigned int& y) const;

		SDL_Surface* getTargetSurface() const{ return targetSurface; }

		// The provided rect needs to be in absolute coords!
		void startEffectStack (SDL_Surface* blitSurface, SDL_Rect blitSurfaceRect, SDL_Rect blitSurfaceClipRect, 
			const vector<Rect*>* dirtyScreenRects  = NULL);
		void endEffectStack (const vector<Rect>& viewports, const double& offsetX = 0, const double& offsetY = 0);

		Rect getCompositeRect() const{ return compositeRect; }

		Rect getBlittingRect() const{ return Rect (blittingSurfaceRect.x, blittingSurfaceRect.y, blittingSurfaceRect.w, blittingSurfaceRect.h); }

		Rect getScreenRect() const{ return screenRect; }

		bool compositePixelIsTransparent (unsigned int x, unsigned int y) const;
		bool blittingPixelIsTransparent (unsigned int x, unsigned int y) const;
		Uint16 getBlittingPixel (const unsigned int& x, const unsigned int& y) const;
	
		void reset();

		vector<Rect> getDirtyEffectRects() const{ return dirtyEffectRects; }
		vector<Rect*> getDirtyScreenRects() const{ return dirtyScreenRects; }

		SDL_Surface* getBlittingSurface(){ return blittingSurface; }
		SDL_Surface* getEffectSurface(){ return effectSurface; }
	
	protected:
		void construct (SDL_Surface* originalSurface, const Rect& screenRect);
		void destruct();

		Uint16 getEffectPixel (const unsigned int& x, const unsigned int& y) const;

	private:
		ImageProcessingSurfaceManager (const ImageProcessingSurfaceManager& model);
		ImageProcessingSurfaceManager& operator= (const ImageProcessingSurfaceManager& model);

		SDL_Surface* effectSurface;
		SDL_Surface* targetSurface;
		SDL_Surface* blittingSurface;
		// Blitting surface rect is the rect describing exactly what area's going to be drawn. x,y are the absolute visual x and y 
		//	of the blitting surface, with w,h that are the width and height of the surface if the surface has no animations, and 
		//	or the width and height of the current frame if the surface does have an animation object.
		SDL_Rect blittingSurfaceRect; 
		// Blitting surface clip rect is the animation rect of the surface. x,y are the location of the current frame in the blitting 
		//	SDL_Surface and w,h are unused. If the blitting surface has no animations, blitting surface clip rect x and y are both 0.
		SDL_Rect blittingSurfaceClipRect;

		void growCompositeRect (const int& x, const int& y);
		void growCompositeRect (const Rect& include);

		Rect compositeRect;
		Rect screenRect;

		vector<Rect> dirtyEffectRects;
		vector<Rect*> dirtyScreenRects;
};


#endif
