#include "inputHandler.h"
#include <iostream>
#include <algorithm>
#include "interface_inputHandlerObserver.h"
#include "keyStatesSet.h"


InputHandlerObserverState::InputHandlerObserverState (Interface_InputHandlerObserver* observer) : observer_ (observer){
	for (int x = 0; x < SDLK_LAST; x++) ignoreKeys_.push_back (false);
}
void InputHandlerObserverState::ignoreKey (int SDLKey){
	ignoreKeys_ [SDLKey] = true;
}
void InputHandlerObserverState::unignoreKey (int SDLKey){
	ignoreKeys_ [SDLKey] = false;
}
KeyStatesSet& InputHandlerObserverState::getKeyStatesSet(){
	return keyStates_;
}
void InputHandlerObserverState::push (const KeyStates& newKeys){
	//cout << "observer state pushing" << endl;
	KeyStates keys (newKeys);
	for (int x = 0; x < SDLK_LAST; x++){
		if (ignoreKeys_ [x]){ 
			//cout << "ignoring key" << endl;
			keys.setKeyState (x, SDL_RELEASED);
		}
	}
	keyStates_.push (keys);
	observer_->inputHandlerUpdate (keyStates_);
}
Interface_InputHandlerObserver* InputHandlerObserverState::getObserver(){
	return observer_;
}
bool InputHandlerObserverState::isIgnoredKey (int SDLKey) const{
	return ignoreKeys_ [SDLKey];
}

void InputHandler::setFocus (Interface_InputHandlerObserver* observer){
	if (observer){
		observers_.push_back (InputHandlerObserverState (observer));
		observers_ [observers_.size() - 1].push (keyStates_.getCurrentStates());
		observers_ [observers_.size() - 1].push (keyStates_.getCurrentStates());
	}
}
// oy
void InputHandler::returnFocus (Interface_InputHandlerObserver* observer){
	if (observer){
		int x = 0;
		for (x = 0; x < (int) observers_.size(); x++){
			if (observers_ [x].getObserver() == observer){
				for (int key = 0; key < SDLK_LAST; key++){
					if (!observers_ [x].isIgnoredKey (key) &&
						observers_ [x].getKeyStatesSet().keyDown (key)
					){
						int next = x - 1;
						for ( ; next >= 0 && observers_ [next].isIgnoredKey (key); next--);
						if (next >= 0){ 
							observers_ [next].getKeyStatesSet().
								getCurrentStates().setKeyState (key, SDL_PRESSED);
							observers_ [next].getKeyStatesSet().
								getPreviousStates().setKeyState (key, SDL_PRESSED);
						}
					}
				}
				observers_.erase (observers_.begin() + x);
				return;
			}
		}
	}
}
void InputHandler::ignoreAllKeys (Interface_InputHandlerObserver* observer){
	for (int x = 0; x < SDLK_LAST; x++) ignoreKey (observer, x);
}
void InputHandler::ignoreKey (Interface_InputHandlerObserver* observer, int SDLKey){
	for (unsigned int x = 0; x < observers_.size(); x++){
		if (observer == observers_ [x].getObserver()){ 
			observers_ [x].ignoreKey (SDLKey);
			return;
		}
	}
}
void InputHandler::unignoreKey (Interface_InputHandlerObserver* observer, int SDLKey){
	for (unsigned int x = 0; x < observers_.size(); x++){
		if (observer == observers_ [x].getObserver()){ 
			observers_ [x].unignoreKey (SDLKey);
			return;
		}
	}
}
void InputHandler::handleInput(){
	handleInput (SDL_GetKeyState (NULL));
}
void InputHandler::handleInput (const Uint8* keyStates){
	KeyStates keys (keyStates);
	keyStates_.push (keys);
	//cout << "InputHandler has " << observers_.size() << " observers_." << endl;

	//cout << "InputHandler has " << observers_.size() << " observers_." << endl;
	// For each observer, create a KeyStatesSet containing current key states for any keys
	//	ignored by the previous observer.
	//cout << "Pushing keystates for " << observers_.size() << " observers" << endl;
	for (int x = observers_.size() - 1; x >= 0; x--){
		//cout << "space set 1? " << keys.getKeyDown (SDLK_SPACE) << endl;
		observers_ [x].push (keys);
		//cout << "space set 2? " << keys.getKeyDown (SDLK_SPACE) << endl;
		//cout << "space is " << SDLK_SPACE << ", first is " << SDLK_FIRST << endl;
		for (int k = 0; k < SDLK_LAST; k++){
			if (! observers_ [x].isIgnoredKey (k)){
				//if (k == SDLK_SPACE) cout << "clearing space" << endl;
				keys.setKeyState (k, SDL_RELEASED);
			}
		}
		//cout << "space set 3? " << keys.getKeyDown (SDLK_SPACE) << endl;
		//cout << "Pushed to an observer" << endl;
	}
	SDL_PumpEvents();
}
