#ifndef INTMAP_H
#define INTMAP_H


/*
	This thing is meant to be used as a map from ints to resources in a graphics app. It's good for a small number of items that 
		are mapped once and accessed many times. I'm using it as a replacement for std::map, which is very general but for my 
		purposes not very fast.
*/


//#define INTMAPTOPOINTERHASHED_TESTING


#include <iostream>
#include <vector>
#include <stdlib.h>

#ifdef INTMAPTOPOINTERHASHED_TESTING
#include <map>
#endif

using namespace std;


typedef struct{
	int loc;
	void* item;
} IntMapHashedHit;

template <typename T>
#define INTMAPTOPOINTERHASHED_MASK 0xfff
#define INTMAPTOPOINTERHASHED_SLOTS INTMAPTOPOINTERHASHED_MASK
class IntMapToPointerHashed{
	public:
		IntMapToPointerHashed(){
			items = (IntMapHashedHit*) malloc (sizeof (IntMapHashedHit) * INTMAPTOPOINTERHASHED_SLOTS);
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				items [x].loc = -1;
			}
			collisionCount = 0;
			itemCount = 0;
		}
	 	IntMapToPointerHashed (const IntMapToPointerHashed<T>& source){
			if (this == &source) return;
			items = (IntMapHashedHit*) malloc (sizeof (IntMapHashedHit) * INTMAPTOPOINTERHASHED_SLOTS);
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				items [x].loc = -1;
			}
			collisionCount = 0;
			itemCount = 0;
			vector<int> keys = source.getKeys();
			for (vector<int>::iterator next = keys.begin(); next != keys.end(); next++){
				set (*next, source.get (*next));
			}
		}
		IntMapToPointerHashed& operator= (const IntMapToPointerHashed<T>& source){
			if (this == &source) return *this;
			if (items) free (items); 
			items = (IntMapHashedHit*) malloc (sizeof (IntMapHashedHit) * INTMAPTOPOINTERHASHED_SLOTS);
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				items [x].loc = -1;
			}
			collisionCount = 0;
			itemCount = 0;
			vector<int> keys = source.getKeys();
			for (vector<int>::iterator next = keys.begin(); next != keys.end(); next++){
				set (*next, source.get (*next));
			}
			return *this;
		}
		virtual ~IntMapToPointerHashed(){
			free (items); items = NULL;
		}

		void copyFrom (IntMapToPointerHashed<T>& other){
			vector<IntMapHashedHit> allRecs = other.getAll();
			for (unsigned int x = 0; x < allRecs.size(); x++){
				set (allRecs [x].loc, (T) allRecs [x].item);
			}
		}

		void erase (int loc){
			int base = loc;
			int count = 0;
			while (items [loc & INTMAPTOPOINTERHASHED_MASK].loc != base && count < INTMAPTOPOINTERHASHED_SLOTS){
				count++;
				loc = (base + count) % INTMAPTOPOINTERHASHED_SLOTS;
				collisionCount++;
			}
			if (! (count < INTMAPTOPOINTERHASHED_SLOTS)){
				cout << "Can't find " << base << ". Not erasing it." << endl;
			}
			else{
				items [loc & INTMAPTOPOINTERHASHED_MASK].loc = -1;
				items [loc & INTMAPTOPOINTERHASHED_MASK].item = NULL;
				itemCount--;
			}
		}
		void set (int loc, T item){
			int base = loc;
			int count = 0;
			while (items [loc & INTMAPTOPOINTERHASHED_MASK].loc != -1 && items [loc & INTMAPTOPOINTERHASHED_MASK].loc != base 
					&& count < INTMAPTOPOINTERHASHED_SLOTS){
				count++;
				loc = (base + count) % INTMAPTOPOINTERHASHED_SLOTS;
				collisionCount++;
			}
			if (! (count < INTMAPTOPOINTERHASHED_SLOTS)){
				cout << "Holy crap- IntMap is full?! You can't add more than " << INTMAPTOPOINTERHASHED_SLOTS 
					<< " things. That's kind of dumb, isn't it? Stop judging me!" << endl;
				cout << "Didn't add " << base << endl;
			}
			else{
				if (items [loc & INTMAPTOPOINTERHASHED_MASK].loc != base){
					itemCount++;
				}
				items [loc & INTMAPTOPOINTERHASHED_MASK].loc = base; items [loc & INTMAPTOPOINTERHASHED_MASK].item = (void*) item;
			}
		}
		T get (int loc) const{
			int count = 0;
			int base = loc;
			while (items [loc & INTMAPTOPOINTERHASHED_MASK].loc != base && count < INTMAPTOPOINTERHASHED_SLOTS){
				count++;
				loc = (base + count) % INTMAPTOPOINTERHASHED_SLOTS;
			}
			if (! (count < INTMAPTOPOINTERHASHED_SLOTS)){
				cout << "Can't find " << base << endl;
				return NULL;
			}
			else{
				return (T) items [loc & INTMAPTOPOINTERHASHED_MASK].item;
			}
		}
		vector<int> getKeys() const{
			vector<int> keys;
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				if (items [x].loc != -1) keys.push_back (items [x].loc);
			}
			//cout << "getKeys returning " << keys.size() << " keys." << endl;
			return keys;
		}
		vector<IntMapHashedHit> getAll(){
			vector<IntMapHashedHit> all;
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				if (items [x].loc != -1){	
					all.push_back (items [x]);
				}
			}
			return all;
		}
		void clear(){
			for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
				items [x].loc = -1;
			}
			itemCount = 0;
		}
		int size(){
			return itemCount;
		}

		int getCollisionCount(){ return collisionCount; }
			
	protected:
		IntMapHashedHit* items;
		int collisionCount;
		int itemCount;
};


#ifdef INTMAPTOPOINTERHASHED_TESTING
bool inVector (vector<int> vect, int item){
	for (vector<int>::iterator iter = vect.begin(); iter != vect.end(); iter++){
		if (*iter == item) return true;
	}
	return false;
}
int main (char** what, int ever){

	string huh = "one hundred";
	string huh2 = "eleven hundred";
	string huh3 = "one hundred one";
	string* ok = &huh;
	string* ok2 = &huh2;
	string* ok3 = &huh3;
	
	IntMapToPointerHashed<string*> foo;
	cout << "Foo constructed." << endl;

	foo.set (100, ok);
	cout << "Foo set." << endl;

	if (foo.get (100) != NULL) cout << *(foo.get (100)) << " returned." << endl;

	foo.set (101, ok3);
	cout << "Foo set." << endl;

	if (foo.get (101) != NULL) cout << *(foo.get (101)) << " returned." << endl;

	foo.set (1100, ok2);
	cout << "Foo set again." << endl;
	
	if (foo.get (1100) != NULL) cout << *(foo.get (1100)) << " returned." << endl;

	cout << "0xff is " << 0xff << endl;
	cout << "0xfff is " << 0xfff << endl;

	string* strings [INTMAPTOPOINTERHASHED_SLOTS];
	for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
		int valueToSet = x * 100;
		char* num = (char*) malloc (sizeof (char) * 100); num [0] = '\0';
		sprintf (num, "%d", valueToSet);
		char* final = (char*) malloc (sizeof (char) * 100); final [0] = '\0';
		int currentChar = 0;
		if (valueToSet >= 100000){
			final [currentChar] = num [currentChar++];
		}
		if (valueToSet >= 10000){
			final [currentChar] = num [currentChar++];
		}
		if (valueToSet >= 1000){
			final [currentChar] = num [currentChar++];
		}
		if (valueToSet >= 100){
			final [currentChar] = num [currentChar++];
		}
		if (valueToSet >= 10){
			final [currentChar] = num [currentChar++];
		}
		final [currentChar] = num [currentChar++];

		strings [x] = new string (final);

		free (num); free (final);
	}

	#ifdef INTMAP_H_COMPARE
	string* junk = NULL;

	foo.clear();
	cout << "Running my map." << endl;
	cout << "SETTING." << endl;
	for (int y = 0; y < 100000; y++){
		for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x += 10){
			foo.set (x, strings [x]);
		}
	}
	cout << "GETTING." << endl;
	for (int y = 0; y < 100000; y++){
		for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x += 10){
			junk = foo.get (x);
		}
	}
	cout << "Done running my map." << endl;

	map<int, string*> whatever;
	cout << "Running STL map." << endl;
	cout << "SETTING." << endl;
	for (int y = 0; y < 100000; y++){
		for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x += 10){
			whatever [x] = strings [x];
		}
	}
	cout << "GETTING." << endl;
	for (int y = 0; y < 100000; y++){
		for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x += 10){
			junk = whatever [x];
		}
	}
	cout << "Done running STL map." << endl;

	cout << "My map collided " << foo.getCollisionCount() << " times." << endl;

	/*
	for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
		cout << *(strings [x]) << endl;
	}
	*/
	#endif	

	for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
		foo.set (x, strings [x]);
	}
	for (int x = 0; x < INTMAPTOPOINTERHASHED_SLOTS; x++){
		if (!foo.get (x) || *strings [x] != *(foo.get (x))){
			cout << "Crap!" << endl;
		}
		if (foo.get (x)){
			cout << *(foo.get (x)) << endl;
		}
	}

	cout << "Collided " << foo.getCollisionCount() << " times." << endl;

	foo.clear();
	foo.set (100, ok); foo.set (1100, ok2); foo.set (101, ok3);
	if ( foo.getKeys().size() == 3 && inVector (foo.getKeys(), 100) && inVector (foo.getKeys(), 1100) && inVector (foo.getKeys(), 101) ){
		cout << "Passed ";
	}
	else cout << "Failed ";
	cout << "getKeys test 1" << endl;

	IntMapToPointerHashed<string*> foo2 = foo;
	if ( foo2.getKeys().size() == 3 && inVector (foo2.getKeys(), 100) && inVector (foo2.getKeys(), 1100) && inVector (foo2.getKeys(), 101) ){
		cout << "Passed ";
	}
	else cout << "Failed ";
	cout << "getKeys test 2" << endl;

	return 0;
}
#endif


#endif
