#ifndef INTERFACE_LOCKABLE_H
#define INTERFACE_LOCKABLE_H


#include <SDL/SDL.h>
#ifndef WIN32
	#include <unistd.h>
#endif

class Interface_Lockable{
	public:
		Interface_Lockable() : locked_ (false){
			lockedMutex_ = SDL_CreateMutex();
			mutex_ = SDL_CreateMutex();
			SDL_UnlockMutex (mutex_);
		}
		~Interface_Lockable(){
			SDL_DestroyMutex (lockedMutex_); lockedMutex_ = NULL;
			SDL_DestroyMutex (mutex_); mutex_ = NULL; 
		}

		void lock(){
			#ifdef WIN32
				while (!tryLock()); // What is the Windows equivalent to usleep()?
			#else
				while (!tryLock()) usleep (10);
			#endif
		}
		bool tryLock(){
			if (!mutex_) throw ("Tried to lock uninitialized mutex!");
			bool lockedIt = false;

			SDL_LockMutex (lockedMutex_);
				if (!locked_){
					SDL_LockMutex (mutex_);
					lockedIt = true;
					locked_ = true;
				}
			SDL_UnlockMutex (lockedMutex_);

			return lockedIt;
		}
		void unlock(){
			if (!mutex_) throw ("Tried to unlock uninitialized mutex!");
			SDL_LockMutex (lockedMutex_);
				SDL_UnlockMutex (mutex_);
				locked_ = false;
			SDL_UnlockMutex (lockedMutex_);
		}

	private:
		bool locked_; SDL_mutex* lockedMutex_;
		SDL_mutex* mutex_;
};


#endif
