#include <vector>
#include <boost/shared_array.hpp>
#include <SDL/SDL.h>


#ifndef KEY_STATES_SET_H
#define KEY_STATES_SET_H


class KeyStates{
	public:
		KeyStates(){
			keyStates_ = boost::shared_array<Uint8> (new Uint8 [SDLK_LAST + 1]);
			memset (&(keyStates_ [0]), 0, sizeof (Uint8) * (SDLK_LAST + 1));
		}
		KeyStates (const KeyStates& model){
			*this = model;
		}
		const KeyStates& operator= (const KeyStates& rhs){
			if (this == &rhs) return rhs;
			keyStates_ = boost::shared_array<Uint8> (new Uint8 [SDLK_LAST + 1]);
			memset (&(keyStates_ [0]), 0, sizeof (Uint8) * (SDLK_LAST + 1));
			for (int x = 0; x < SDLK_LAST; x++){
				if (rhs.getKeyDown (x)) keyStates_ [x] = SDL_PRESSED;
			}
			return *this;
		}
		KeyStates (boost::shared_array<Uint8> states) : keyStates_ (states) { }
		KeyStates (const Uint8* states){
			keyStates_ = boost::shared_array<Uint8> (new Uint8 [SDLK_LAST + 1]);
			memcpy (&(keyStates_ [0]), states, sizeof (Uint8) * (SDLK_LAST + 1));
		}
		void setKeyState (int SDLKey, Uint8 SDLKeyState){
			keyStates_ [SDLKey] = SDLKeyState;
		}
		bool getKeyDown (int SDLKey) const{
			return keyStates_ [SDLKey] == SDL_PRESSED;
		}

	private:
		boost::shared_array<Uint8> keyStates_;
};

class KeyStatesSet{
	public:
		KeyStatesSet() { }
		KeyStatesSet (const KeyStates& previousStates, const KeyStates& currentStates)
			: previousStates_ (previousStates), currentStates_ (currentStates) { }

		bool keyDown (int SDLKey) const{
			return currentStates_.getKeyDown (SDLKey);
		}
		bool keyJustPressed (int SDLKey) const{
			return (!previousStates_.getKeyDown (SDLKey)) && currentStates_.getKeyDown (SDLKey);
		}
		void push (const KeyStates& newStates){
			previousStates_ = currentStates_;
			currentStates_ = newStates;
		}
		KeyStates& getPreviousStates(){
			return previousStates_;
		}
		KeyStates& getCurrentStates(){
			return currentStates_;
		}

	private:
		KeyStates previousStates_;
		KeyStates currentStates_;
};


#endif
