#ifndef microtilefield_h
#define microtilefield_h

#include "rect.h"
#include <vector>
#include <exception>
#include <string>
#include <iostream>
#include <math.h>
#include "rectQuadTree.h"

using namespace std;

#define TILE_ENTRY_TYPE uchar
// sizeof (TILE_ENTRY_TYPE)
#define BITS_PER_TILE_ENTRY 8
// 2 ^ BITS_PER_TILE_ENTRY
#define TILE_ENTRY_STATES 256


typedef unsigned char uchar;

typedef struct{
	TILE_ENTRY_TYPE mask;
	short int start;
	short int width;
} LookupRec;

class MicroTileField {
	public:
		MicroTileField (unsigned int width, unsigned int height, unsigned int widthInTiles, unsigned int heightInTiles);
		~MicroTileField();
		
		void dirty (const vector<Rect>&);
		void getConsolidatedRects (const Rect&, vector<Rect*>&);

		void reset();

		TILE_ENTRY_TYPE runningHorizontalMask;

		#ifdef TEST_MICROTILEFIELD
			string test();
		#endif

	protected:
		unsigned int getHorizontalPixelsPerBox(){ return width / widthInTiles; }
		unsigned int getVerticalPixelsPerBox(){ return height / heightInTiles; }
		Rect getBoxesIntersecting (const Rect&);

		#ifdef TEST_MICROTILEFIELD
			string testLookups();
			string testDirty();
			string testCompile();
			string testGet();
			string check (string, string, string);
			string check (string, int, int);
			string check (const char*, vector<Rect*>&, vector<Rect*>&);
			string check (string, vector<Rect*>&, vector<Rect*>&);
			string tileEntryToString (TILE_ENTRY_TYPE);
			string markFieldToString (TILE_ENTRY_TYPE** markField);
		#endif
		#ifdef MICROTILEFIELD_MEMCHECK
			void* myMalloc (size_t);
			void myFree (void*);
			void checkMem();
			void iAllocated (void*);
			bool iDeallocated (void*);
			
			vector<void*> allocatedMemory;
			vector<void*> doubleFrees;
		#endif

	private:
		void initMarkField();
		void initLookup();
		void initCompilation();
		void initDirtyMaskLookup();
		void resetMarkField();
		void resetCompilation();
		void killMarkField();
		void killLookup();
		void killCompilation();
		void killDirtyMaskLookup();

		void placeInCompilation (Rect*);
		
		Rect myPerimeter;

		TILE_ENTRY_TYPE** markField;
		RectQuadTree* compilation;
		// This lookup only works with an even number of BITS_PER_TILE_ENTRY (odd shouldn't be possible anyway)
		LookupRec lookup [TILE_ENTRY_STATES][BITS_PER_TILE_ENTRY / 2 + 1];

		// dirtyMaskLookup [setting bits right to left or left to right][bit offset] 
		// Example (in case of unsigned char TILE_ENTRY_TYPE):
		// {{ 11111111 01111111 00111111 00011111 00001111 00000111 00000011 00000001 00000000 },
		//  { 00000000 10000000 11000000 11100000 11110000 11111000 11111100 11111110 11111111 }}
		TILE_ENTRY_TYPE dirtyMaskLookup [2][BITS_PER_TILE_ENTRY + 1];

		void compile();
		bool hasCompiled;

		unsigned int width;
		unsigned int height;
		unsigned int widthInTiles;
		unsigned int heightInTiles;
};

#endif
