#ifndef NOISEMAKER_H
#define NOISEMAKER_H

#include "utils.h"

#define NOISE_CACHE_SIZE 4096


class NoiseMakerCacheEntry{
	public:
		NoiseMakerCacheEntry(){
			input = -1;
			output = 0;
			lowerBound = -1;
			upperBound = -1;
		}
		~NoiseMakerCacheEntry(){}

		int input;
		double output;
		int lowerBound;
		int upperBound;
};

class NoiseMaker{
	public:
		NoiseMaker(){}
		~NoiseMaker(){}

    	double perlinOneDimension (int x, int lower = 0, int upper = 1000000000){
			int slot = x % NOISE_CACHE_SIZE;
			if (noiseCache [slot].input == x && noiseCache [slot].lowerBound == lower && noiseCache [slot].upperBound == upper) return noiseCache [slot].output;

        	double total = 0.0;
	        int lowerBound = lower; int upperBound = upper;
    	    for (double persistence = 1.0; persistence > 1.0 / 32.0; persistence /= 2.0){
        	    //cout << "lowerbound, upperbound: " << lowerBound << "," << upperBound << endl;
            	double factor = (x - lowerBound * 1.0) / (upperBound - lowerBound * 1.0);
	            double inverseFactor = 1.0 - factor;
    	        //TODO: not quite right. can i get away with it though?
        	    //cout << "checking pseudo rand for " << lowerBound << " and " << upperBound << endl;
            	//cout << "weight is " << inverseFactor << endl;
	            total += persistence * ( (inverseFactor * Utils::fastPseudoRand (lowerBound)) + (factor * Utils::fastPseudoRand (upperBound)) );
    	        if (x < (lowerBound + upperBound) / 2){
        	        upperBound = (lowerBound + upperBound) / 2;
            	}
	            else{
    	            lowerBound = (lowerBound + upperBound) / 2;
        	    }
	        }
    	    //cout << "returning total " << total << endl;
			noiseCache [slot].input = x;
			noiseCache [slot].output = (total + 2.0) / 4.0;
			noiseCache [slot].lowerBound = lower;
			noiseCache [slot].upperBound = upper;

        	return noiseCache [slot].output;
	    }
	protected:

	private:
		NoiseMakerCacheEntry noiseCache [NOISE_CACHE_SIZE];
};


#endif
