#include "player.h"
#include "sprite.h"
#include "animationSet.h"


Player::Player (Surface* owner, const Rect& perimeter, const std::string& spriteSheet, const std::string& spriteDef)
		: Sprite (owner, spriteSheet){
	loadAnimations (spriteDef);
	setAnimationTimeLength (ANIM_PLAYER_STAND_LEFT, 500);
	setAnimationTimeLength (ANIM_PLAYER_STAND_RIGHT, 500);
	setAnimationTimeLength (ANIM_PLAYER_WALK_LEFT, 250);
	setAnimationTimeLength (ANIM_PLAYER_WALK_RIGHT, 250);
	setAnimationTimeLength (ANIM_PLAYER_JUMP_LEFT, 500);
	setAnimationTimeLength (ANIM_PLAYER_JUMP_RIGHT, 500);

	coasting = false;
	jumpSpeed = 0;

	setSpeed (PLAYER_WALK_SPEED);

	setMovementPerimeter (perimeter);

	facingFlags = 0;
	moveFlags = 0;

	yVelocity = 0;
	grounded = true;

	score = 0;
	combo = 0;

	playStandingAnim();
}
Player::~Player() { }

bool Player::fallingFast(){
	return isFalling() && yVelocity < -350;
}
bool Player::isFalling(){
	return !isGrounded() && yVelocity < 0;
}
bool Player::isJumping(){
	return !isGrounded() && yVelocity > 0;
}
void Player::setGrounded (bool ground){
	if (!grounded && ground){
		yVelocity = 0;
		playStandingAnim();
	}
	grounded = ground;
}
bool Player::isGrounded(){
	return grounded;
}

bool Player::update (unsigned int timePassed){
	//if (jumpAnimPlaying()) cout << "jump anim playing " << endl;
	/*
	if (jumpAnimPlaying() && !coasting){
		//cout << "adding jump" << endl;
		yVelocity += timePassed * (jumpSpeed / 1000.0);
	}
	*/

	bool result = Sprite::update (timePassed);
	double amountToMoveX = getSpeedInPixelsPerMs() * timePassed * (isGrounded() ? 1.0 : PLAYER_JUMP_X_SPEED_MULT);
	double amountToMoveY = -yVelocity * (timePassed / 1000.0);
	if (moveFlags & PLAYER_RIGHT) moveXY (amountToMoveX, 0);
	else if (moveFlags & PLAYER_LEFT) moveXY (-amountToMoveX, 0);
	moveXY (0, amountToMoveY);
	if (moveFlags == 0 && isGrounded()) playStandingAnim();
	moveFlags = 0;
	if (!isGrounded() && (coasting || !jumpAnimPlaying())) yVelocity -= PLAYER_GRAVITY * (timePassed / 1000.0);
	return result;
}

void Player::setJumpSpeed (double speed){
	jumpSpeed = speed;
}
void Player::jump(){
	if (isGrounded()){
		coasting = false;
		playJumpingAnim();
		setGrounded (false);
		//yVelocity += PLAYER_SHORT_JUMP_SPEED;
		yVelocity += jumpSpeed;
	}
}
void Player::coast(){
	coasting = true;
}
void Player::moveRight(){
	facingFlags = PLAYER_RIGHT;
	playWalkingAnim();
	moveFlags = PLAYER_RIGHT;
}
void Player::moveLeft(){
	facingFlags = PLAYER_LEFT;
	playWalkingAnim();
	moveFlags = PLAYER_LEFT;
}

void Player::playStandingAnim(){
	if (standingAnimPlaying()) return;
	if (facingFlags & PLAYER_LEFT){
		resetAnimation (ANIM_PLAYER_STAND_LEFT);
		playAnimation (ANIM_PLAYER_STAND_LEFT, true);
	}
	else{
		resetAnimation (ANIM_PLAYER_STAND_RIGHT);
		playAnimation (ANIM_PLAYER_STAND_RIGHT, true);
	}
}
void Player::playWalkingAnim(){
	if (!isGrounded() || walkAnimPlaying()) return;
	if (facingFlags & PLAYER_LEFT){
		resetAnimation (ANIM_PLAYER_WALK_LEFT);
		playAnimation (ANIM_PLAYER_WALK_LEFT, true);
	}
	else{
		resetAnimation (ANIM_PLAYER_WALK_RIGHT);
		playAnimation (ANIM_PLAYER_WALK_RIGHT, true);
	}
}
void Player::playJumpingAnim(){
	if (jumpAnimPlaying()) return;
	if (facingFlags & PLAYER_LEFT){
		resetAnimation (ANIM_PLAYER_JUMP_LEFT);
		playAnimation (ANIM_PLAYER_JUMP_LEFT, false);
	}
	else{
		resetAnimation (ANIM_PLAYER_JUMP_RIGHT);
		playAnimation (ANIM_PLAYER_JUMP_RIGHT, false);
	}
}
bool Player::animPlaying (int leftVersion, int rightVersion){
	int currentAnim = getAnimations()->getCurrentAnimation();
	int timeLeft = getAnimations()->getCurrentAnimationTimeRemaining();
	if (timeLeft == 0) return false;
	if (currentAnim != leftVersion && currentAnim != rightVersion) return false;
	return true;
}
bool Player::walkAnimPlaying(){
	bool playing = animPlaying (ANIM_PLAYER_WALK_LEFT, ANIM_PLAYER_WALK_RIGHT);
	//cout << "walk playing? " << playing << endl;
	return playing;
}
bool Player::jumpAnimPlaying(){
	return animPlaying (ANIM_PLAYER_JUMP_LEFT, ANIM_PLAYER_JUMP_RIGHT);
}
bool Player::standingAnimPlaying(){
	//cout << "walk playing? " << playing << endl;
	return animPlaying (ANIM_PLAYER_STAND_LEFT, ANIM_PLAYER_STAND_RIGHT);
}
