#ifndef PLAYER_H
#define PLAYER_H

#include <string>
#include "sprite.h"


#define PLAYER_LEFT 1
#define PLAYER_RIGHT 2

#define ANIM_PLAYER_STAND_LEFT 0
#define ANIM_PLAYER_STAND_RIGHT 1
#define ANIM_PLAYER_WALK_LEFT 2
#define ANIM_PLAYER_WALK_RIGHT 3
#define ANIM_PLAYER_JUMP_LEFT 4
#define ANIM_PLAYER_JUMP_RIGHT 5

#define PLAYER_WALK_SPEED 75
#define PLAYER_SHORT_JUMP_SPEED 50
#define PLAYER_JUMP_X_SPEED_MULT 1.0

#define PLAYER_GRAVITY 700 // lol I have no idea; it's completely arbitrary


class Player : public Sprite{
	public:
		Player (Surface* owner, const Rect& perimeter, const std::string& spriteSheet = "monkey.bmp", 
			const std::string& spriteDef = "monkey.set");
		virtual ~Player();

		bool update (unsigned int timePassed);

		void setGrounded (bool ground);
		bool isGrounded();
		void jump();
		void coast();
		void moveRight();
		void moveLeft();
		bool isFalling();
		bool isJumping();
		bool fallingFast();
		void setJumpSpeed (double);

		int getCombo(){ return combo; }
		void setCombo (int num){ combo = num; }
		int getScore(){ return score; }
		void setScore (int num){ score = num; }
	private:
		void playStandingAnim();
		void playJumpingAnim();
		void playWalkingAnim();
		bool animPlaying (int, int);
		bool walkAnimPlaying();
		bool jumpAnimPlaying();
		bool standingAnimPlaying();

		unsigned int moveFlags;
		unsigned int facingFlags;

		double yVelocity;
	
		bool grounded;

		double jumpSpeed;
		bool coasting;

		int combo;
		int score;
};



#endif
