/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QUADTREE_H
#define QUADTREE_H


#include <vector>
#include <iostream>
#include "rect.h"
#include "geometry.h"

using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
template <typename T>
class QuadTreeItem{
	public:
		QuadTreeItem (const T& item, const double& x, const double& y){
			this->item = item;
			this->x = x;
			this->y = y;
		}
		~QuadTreeItem(){}
		double getX() const{
			return x;
		}
		double getY() const{
			return y;
		}
		T getItem() const{
			return item;
		}

	private:
		T item;
		double x;
		double y;
};

template <typename T>
class QuadTreeNode;

template <typename T>
class QuadTreeNode{
	public:
		QuadTreeNode (const QuadTreeItem<T>&);
		virtual ~QuadTreeNode();

		void insert (const QuadTreeItem<T>&);
		void getItems (const Rect&, vector<T>& = vector<T>()) const;

	private:
		QuadTreeNode* nodes [4];
		QuadTreeItem<T>* item;
};

template <typename T>
class QuadTree{
    public:
        QuadTree();
        virtual ~QuadTree();

		void insert (const T&, const double&, const double&);
		vector<T> getItems (const Rect&) const;
		void clear();

	private:
		QuadTreeNode<T>* startNode;
};




// For some reason I don't quite understand, template defs need to be included with decs.  :P

// Do I really need to preface all my function definitions with the stupid template crap? How 
//	can I get it to compile without that junk?


template <typename T>
QuadTreeNode<T>::QuadTreeNode (const QuadTreeItem<T>& item){
	for (int x = 0; x < 4; x++){
		nodes [x] = NULL;
	}
	this->item = new QuadTreeItem<T> (item);
}
template <typename T>
QuadTreeNode<T>::~QuadTreeNode(){
	for (int x = 0; x < 4; x++){
		if (nodes [x]) delete nodes [x];
	}
	if (item) delete item;
}
template <typename T>
void QuadTreeNode<T>::insert (const QuadTreeItem<T>& newItem){
	int nextNodeIndex = 3;
	if (newItem.getX() <= item->getX() && newItem.getY() <= item->getY())
		nextNodeIndex = 0;
	else if (newItem.getX() > item->getX() && newItem.getY() <= item->getY())
		nextNodeIndex = 1;
	else if (newItem.getX() <= item->getX() && newItem.getY() > item->getY())
		nextNodeIndex = 2;
	if (!nodes [nextNodeIndex])
		nodes [nextNodeIndex] = new QuadTreeNode<T> (newItem);
	else
		nodes [nextNodeIndex]->insert (newItem);
}
template <typename T>
void QuadTreeNode<T>::getItems (const Rect& perimeter, vector<T>& runningList) const{
	//cout << "QuadTreeNode::getItems()" << endl;
	if (nodes [0])
		if (perimeter.getX() <= item->getX() && perimeter.getY() <= item->getY())
			nodes [0]->getItems (perimeter, runningList);
	if (nodes [1]) 
		if (perimeter.getX() + perimeter.getWidth() > item->getX() && perimeter.getY() <= item->getY())
			nodes [1]->getItems (perimeter, runningList);
	if (nodes [2]) 
		if (perimeter.getX() <= item->getX() && perimeter.getY() + perimeter.getHeight() > item->getY())
			nodes [2]->getItems (perimeter, runningList);
	if (nodes [3]) 
		if (perimeter.getX() + perimeter.getWidth() > item->getX() && perimeter.getY() + perimeter.getHeight() > item->getY())
			nodes [3]->getItems (perimeter, runningList);
	if (perimeter.containsPoint (Point (item->getX(), item->getY())))
		runningList.push_back (item->getItem());
}

template <typename T>
QuadTree<T>::QuadTree(){
	startNode = NULL;
}
template <typename T>
QuadTree<T>::~QuadTree(){
	clear();
}
template <typename T>
void QuadTree<T>::insert (const T& item, const double& x, const double& y){
	if (!startNode)	startNode = new QuadTreeNode<T> (QuadTreeItem<T> (item, x, y));
	else startNode->insert (QuadTreeItem<T> (item, x, y));
}
template <typename T>
vector<T> QuadTree<T>::getItems (const Rect& perimeter) const{
	vector<T> items;
	if (!startNode) return items;
	startNode->getItems (perimeter, items);
	return items;
}
template <typename T>
void QuadTree<T>::clear(){
	if (startNode){ 
		delete startNode; 
		startNode = NULL;
	}
}



#endif
