/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "rect.h"

#include <iostream>
using namespace std;

Rect::Rect() 
	: x (0), y (0), width (0), height (0){
}
Rect::Rect (const double& pX, const double& pY) 
	: x (pX), y (pY), width (0), height (0){
}
Rect::Rect (const double& pX, const double& pY, const double& pWidth, const double& pHeight) 
	: x (pX), y (pY), width (pWidth), height (pHeight){
}
Rect::Rect (const SDL_Rect& rect) 
	: x (rect.x), y (rect.y), width (rect.w), height (rect.h){
}
Rect::Rect (const Rect& model){
	//printf ("Copy ctor called.\n");
	copyFrom (model);
}
Rect& Rect::operator= (const Rect& model) throw(){
	copyFrom (model);
	return *this;
}
bool Rect::operator== (const Rect& model) throw(){
	return model.getWidth() == getWidth() && model.getHeight() == getHeight() && model.getX() == getX() && model.getY() == getY();
}
bool Rect::operator< (const Rect& model) throw(){
	return toString() < model.toString();
}
Rect::~Rect()
{
    //printf ("Rect at %p destroying.\n", this);
}

string Rect::toString() const{
	char strX [100], strY [100], strW [100], strH [100];
	sprintf (strX, "%f", getX()); sprintf (strY, "%f", getY()); 
	sprintf (strW, "%f", getWidth()); sprintf (strH, "%f", getHeight());
	string out = "Rect: (" + string (strX) + ", " + string (strY) + ") w,h: " + string (strW) + ", " + string (strH);
	return out;
}

Point Rect::getUpperLeftPoint() const{
	return Point (getX(), getY());
}
Point Rect::getUpperRightPoint() const{
	return Point (getX() + getWidth(), getY());
}
Point Rect::getLowerLeftPoint() const{
	return Point (getX(), getY() + getHeight());
}
Point Rect::getLowerRightPoint() const{
	return Point (getX() + getWidth(), getY() + getHeight());
}
Point Rect::getCenter() const{
	return Point (getX() + (getWidth() / 2), getY() + (getHeight() / 2));
}

bool Rect::containsPoint (const Point& point) const{
    return ( (point.getX() >= getX()) && (point.getX() <= getX() + getWidth()) && (point.getY() >= getY()) && (point.getY() <= getY() + getHeight()) );
}
bool Rect::containsRect (const Rect& rect) const{
	return !(rect.getX() < getX() || rect.getY() < getY() || rect.getX() + rect.getWidth() > getX() + getWidth() 
			|| rect.getY() + rect.getHeight() > getY() + getHeight());
}

bool Rect::intersectsLine (const Line& line) const{
	Point p1 = line.getPoint1();
	Point p2 = line.getPoint2();
	if (containsPoint (p1) || containsPoint (p2)) return true;
	double myX1 = getX();
	double myX2 = myX1 + getWidth();
	double myY1 = getY();
	double myY2 = myY1 + getHeight();
	Line top (getUpperLeftPoint(), getUpperRightPoint());
	Line right (getUpperRightPoint(), getLowerRightPoint());
	Line bottom (getLowerLeftPoint(), getLowerRightPoint());
	Line left (getUpperLeftPoint(), getLowerLeftPoint());
	
	//return top.intersectsLine (line) || bottom.intersectsLine (line) || left.intersectsLine (line) || right.intersectsLine (line);
	
	if (p1.getX() < myX1){
		if (p2.getX() < myX1) return false;
		if (line.intersectsLine (left)) return true;
	}
	else if (p1.getX() > myX2){ 
		if (p2.getX() > myX2) return false;
		if (line.intersectsLine (right)) return true;
	}
	if (p1.getY() < myY1){
		if (p2.getY() < myY1) return false;
		if (line.intersectsLine (top)) return true;
	}
	else if (p1.getY() > myY2){
		if (p2.getY() > myY2) return false;
		if (line.intersectsLine (bottom)) return true;
	}
	return false;
	
}
bool Rect::intersects (const Rect& other) const{
	if (other.getX() >= x + width || other.getY() >= y + height || other.getX() + other.getWidth() <= x || other.getY() + other.getHeight() <= y) return false;
	return true;
	
/*
	//int ax1, ax2, ay1, ay2;
	double ax1, ax2, ay1, ay2;
	//int bx1, bx2, by1, by2;
	double bx1, bx2, by1, by2;
	
	ax1 = getX(); ax2 = ax1 + getWidth();
	ay1 = getY(); ay2 = ay1 + getHeight();
	bx1 = other.getX(); bx2 = bx1 + other.getWidth();
	by1 = other.getY(); by2 = by1 + other.getHeight();

	// Ugly, but I'm hoping it's faster than other possible solutions. 
	//   Could be better still, I'm sure!

	if (
		(ax1 >= bx1 && ax1 <= bx2 && ay1 >= by1 && ay1 <= by2) ||
		(ax2 >= bx1 && ax2 <= bx2 && ay1 >= by1 && ay1 <= by2) || 
		(ax1 >= bx1 && ax1 <= bx2 && ay2 >= by1 && ay2 <= by2) ||
		(ax2 >= bx1 && ax2 <= bx2 && ay2 >= by1 && ay2 <= by2) ||
		(bx1 >= ax1 && bx1 <= ax2 && by1 >= ay1 && by1 <= ay2) ||
		(bx2 >= ax1 && bx2 <= ax2 && by1 >= ay1 && by1 <= ay2) || 
		(bx1 >= ax1 && bx1 <= ax2 && by2 >= ay1 && by2 <= ay2) ||
		(bx2 >= ax1 && bx2 <= ax2 && by2 >= ay1 && by2 <= ay2) 
	){
		return true;
	}

	if (ay1 < by1){
		return (ay1 < by1 && ay2 > by2 && ax1 > bx1 && ax2 < bx2);
	}
	else{
		return (by1 < ay1 && by2 > ay2 && bx1 > ax1 && bx2 < ax2);
	}
*/
}

void Rect::copyFrom (const Rect& model){
	//cout << "Copying from " << model.toString() << endl;
	x = model.getX();
	y = model.getY();
	width = model.getWidth();
	height = model.getHeight();
}

bool Rect::equals (const Rect& compare) const{
    return x == compare.getX() && y == compare.getY()
        && width == compare.getWidth() && height == compare.getHeight();
}

vector< Rect > Rect::cutOut (const Rect& other){
    vector< Rect > returnVector;

	if (! intersects (other)){ 
		returnVector.push_back (*this); 
		return returnVector; 
	}

    //int myX1 = getX(); int myX2 = getX() + getWidth();
	double myX1 = getX(); double myX2 = getX() + getWidth();
    //int myY1 = getY(); int myY2 = getY() + getHeight();
    double myY1 = getY(); double myY2 = getY() + getHeight();
    //int otherX1 = other.getX(); int otherX2 = other.getX() + other.getWidth();
    double otherX1 = other.getX(); double otherX2 = other.getX() + other.getWidth();
    //int otherY1 = other.getY(); int otherY2 = other.getY() + other.getHeight();
    double otherY1 = other.getY(); double otherY2 = other.getY() + other.getHeight();

    // Possible number of points of one rect in another (num different cases)
    // This rect : other rect
    // 0 : 0
    // 0 : 2
    // 0 : 4
    // 1 : 1
    // 2 : 0
    // 2 : 2
    // 2 : 4
    // 4 : 0
    // 4 : 2
    // 4 : 4

    //int myPoints = other.containsPoint (myX1) + other.containsPoint (myX2) + other.containsPoint (myY1) + other.containsPoint (myY2);
    //int otherPoints = containsPoint (otherX1) + containsPoint (otherX2) + containsPoint (otherY1) + containsPoint (otherY2);

    if (myY1 < otherY1){
        Rect newRect (myX1, myY1, myX2 - myX1, otherY1 - myY1);
        //newRect.trimTo (*this);
        if (newRect.getWidth() > 0 && newRect.getHeight() > 0){
            returnVector.push_back (newRect);
        }
    }
    if (myX1 < otherX1){
        Rect newRect (myX1, max (myY1, otherY1), otherX1 - myX1, min (myY2, otherY2) - max (myY1, otherY1));
        //newRect.trimTo (*this);
        if (newRect.getWidth() > 0 && newRect.getHeight() > 0){
            returnVector.push_back (newRect);
        }
    }
    if (myX2 > otherX2){
        Rect newRect (otherX2, max (myY1, otherY1), myX2 - otherX2, min (myY2, otherY2) - max (myY1, otherY1));
        //newRect.trimTo (*this);
        if (newRect.getWidth() > 0 && newRect.getHeight() > 0){
            returnVector.push_back (newRect);
        }
    }
    if (myY2 > otherY2){
        Rect newRect (myX1, otherY2, myX2 - myX1, myY2 - otherY2);
        //newRect.trimTo (*this);
        if (newRect.getWidth() > 0 && newRect.getHeight() > 0){
            returnVector.push_back (newRect);
        }
    }
    return returnVector;
}

void Rect::stretchToInclude (const Rect& other){
	if (getX() > other.getX()){ 
		setWidth (getWidth() + (getX() - other.getX()));
		setX (other.getX());
	}
	if (getY() > other.getY()){
		setHeight (getHeight() + (getY() - other.getY()));
		setY (other.getY());
	}
	if (getX() + getWidth() < other.getX() + other.getWidth()){
		setWidth (getWidth() + ((other.getX() + other.getWidth()) - (getX() + getWidth())));
	}
	if (getY() + getHeight() < other.getY() + other.getHeight()){
		setHeight (getHeight() + ((other.getY() + other.getHeight()) - (getY() + getHeight())));
	}
}
void Rect::trimTo (const Rect& model){
	if (! intersects (model)){
		setWidth (0); setHeight (0);
	} // Commented out- costs too much.

	// Still not optimal- why is this so slow?

	//SDL_Rect modelRect = model.getRect();
	//Rect modelRect = model;
	//SDL_Rect oldRect = rect;

    //int modelX1 = modelRect.x;
	//double modelX1 = modelRect.getX();
	double modelX1 = model.getX();
    //int modelY1 = modelRect.y;
	//double modelY1 = modelRect.getY();
	double modelY1 = model.getY();
    //int modelX2 = modelX1 + modelRect.w;
	//double modelX2 = modelX1 + modelRect.getWidth();
	double modelX2 = modelX1 + model.getWidth();
    //int modelY2 = modelY1 + modelRect.h;
	//double modelY2 = modelY1 + modelRect.getHeight();
	double modelY2 = modelY1 + model.getHeight();

	double startX = getX();
	double startY = getY();
	double endX = startX + getWidth();
	double endY = startY + getHeight();

    /*
    cout << toString() << endl;
    cout << "start x,y: " << startX << "," << startY << endl;
    cout << "end x,y: " << endX << "," << endY << endl;
    cout << "actual: " << getRect().x << "," << getRect().y << "," << getRect().w << "," << getRect().h << endl;
    */

    if (startX < modelX1) startX = modelX1;
    else if (startX > modelX2){
		setWidth (0); setHeight (0); return;
	}
    if (startY < modelY1) startY = modelY1;
    else if (startY > modelY2){
		setWidth (0); setHeight (0); return;
	}
    if (endX > modelX2) endX = modelX2;
	else if (endX < modelX1){
		setWidth (0); setHeight (0); return;
	}
    if (endY > modelY2) endY = modelY2;
	else if (endY < modelY1){
		setWidth (0); setHeight (0); return;
	}

	setX (startX); setY (startY);
	setWidth (endX - startX); setHeight (endY - startY);
}
