/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RECT_H
#define RECT_H

#define MAX(a,b) a>b?a:b

#include <SDL/SDL.h>
#include <string>
#include <vector>
#include "geometry.h"

using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
class Rect{
	public:
		Rect();
		Rect (const double&, const double&);
		Rect (const double&, const double&, const double&, const double&);
		Rect (const SDL_Rect& rect);
		Rect (const Rect& model);
		Rect& operator= (const Rect& model) throw();
		bool operator== (const Rect& model) throw();
		bool operator< (const Rect& model) throw();
		~Rect();
        
		bool intersects (const Rect& other) const;
		bool intersectsLine (const Line& line) const;
		void setXY (const double& x, const double& y){ this->x = x; this->y = y; }
		void setX (const double& x){ this->x = x; }
		void setY (const double& y){ this->y = y; }
		Point getUpperLeftPoint() const;
		Point getLowerLeftPoint() const;
		Point getUpperRightPoint() const;
		Point getLowerRightPoint() const;
		Point getCenter() const;
		double getX() const{ return x; }
		double getY() const{ return y; }
		double getCenterX() const{ return x + width / 2; }
		double getCenterY() const{ return y + height / 2; }
		double getWidth() const{ return width; }
		double getHeight() const{ return height; }
		void setWidth (const double& width){ this->width = width; }
		void setHeight (const double& height){ this->height = height; }
		SDL_Rect toSDL_Rect() const{ 
			SDL_Rect retRect;
			retRect.x = int (x);
			retRect.y = int (y);
			retRect.w = int (width);
			retRect.h = int (height);
			return retRect;
		}
		void copyFrom (const Rect& model);
		void trimTo (const Rect& model);
		void stretchToInclude (const Rect& other);
		string toString() const;
		vector< Rect > cutOut (const Rect& other);
		bool containsPoint (const Point& point) const;
		bool containsRect (const Rect& rect) const;

		bool equals (const Rect& compare) const;

	protected:
		double x; double y;
		double width; double height;
	
	private:
};


#endif
