/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


// I did this without using recursion because I was bored and not thinking about how stupid 
//	an idea it is. I do love C though.  :)
// This is a trie struct kind of like a quadtree but meant to hold 2d-rectangles on a 2d-surface 
//	covering mutually exclusive areas. trying to insert a rect that intersects a rect already in 
//	the struct fails. i'm not even really sure this thing is efficient yet; i haven't really got 
//	into an analysis of it (i'm not sure i'd be any good at doing that anyway, to be honest.  :P )
// Got several days into debugging before I gave up and started using recursion. Memory management
//	was a nightmare. Now it's passing with flying colors!


#ifndef RECTQUADTREE_H
#define RECTQUADTREE_H


#include <vector>
#include <iostream>
#include "rect.h"
using namespace std;


/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
struct Node{
	struct Node* nodes [4];
	double x;
	double y;
	Rect* item;
	bool searched;
};

class RectQuadTree{
	public:
		RectQuadTree(){
			startNode = NULL;
			blankNode = (Node*) malloc (sizeof (Node));{
				blankNode->nodes [0] = NULL; blankNode->nodes [1] = NULL; blankNode->nodes [2] = NULL; blankNode->nodes [3] = NULL;
				blankNode->x = 0; blankNode->y = 0;
				blankNode->item = NULL;
				blankNode->searched = false;
			}
			#ifdef MEMCHECK
				cout << "Creating Node at " << blankNode << endl;
			#endif
		}
		~RectQuadTree(){
			if (startNode) deleteNode (startNode);
			#ifdef MEMCHECK
				cout << "Deleting Node at " << blankNode << endl;
			#endif
			free (blankNode);
		}
		
		void getItems (const Rect* perimeter, vector<Rect*>& runningList){
			if (!startNode) return;
			getItems (startNode, perimeter, runningList);
			resetSearched (startNode);
		}
		bool insert (Rect* item){
			bool pass = false;
			if (!startNode){
				startNode = newNode (item); 
				return true;
			}
			else{
				vector<Rect*> gets;
				getItems (item, gets); 	
				if (gets.size() > 0){ 
					cout << "Can't insert " << item->toString() << " because it intersects an existing rect!" << endl;
					pass = false;
				}
				else{
					Node* toPlace = newNode (item);
					pass = insertItem (startNode, toPlace);
				}
			}
			resetSearched (startNode);
			return pass;
		}
		void clear(){
			if (startNode) deleteNode (startNode); startNode = NULL;
		}

	private:
		Node* startNode;
		Node* blankNode;

		void resetSearched (Node* start){
			if (start->searched){
				start->searched = false;
				if (start->nodes [0]) resetSearched (start->nodes [0]);
				if (start->nodes [1]) resetSearched (start->nodes [1]);
				if (start->nodes [2]) resetSearched (start->nodes [2]);
				if (start->nodes [3]) resetSearched (start->nodes [3]);
			}
		}
		Node* newNode (Rect* item){
			Node* newOne = (Node*) malloc (sizeof (Node));
			#ifdef MEMCHECK
				cout << "Creating Node at " << newOne << endl;
			#endif
			memcpy (newOne, blankNode, sizeof (Node));{
				newOne->item = item;
				newOne->x = item->getCenterX();
				newOne->y = item->getCenterY();
			}
			return newOne;
		}
		void killNode (Node* toKill){
			#ifdef MEMCHECK
				cout << "Deleting Node at " << toKill << endl;
			#endif
			free (toKill);
		}		
		void getItems (Node* start, const Rect* perimeter, vector<Rect*>& runningList) const{
			if (start->searched) return;
			start->searched = true;

			Node* markedNodes [3]; markedNodes [0] = NULL; markedNodes [1] = NULL; markedNodes [2] = NULL;
			if (perimeter->getX() <= start->x && perimeter->getY() <= start->y){
				if (start->nodes [0]){
					getItems (start->nodes [0], perimeter, runningList);
					markedNodes [0] = start->nodes [0];
				}
			}
			if (perimeter->getX() + perimeter->getWidth() > start->x && perimeter->getY() <= start->y){
				if (start->nodes [1] 
						&& markedNodes [0] != start->nodes [1]){
					getItems (start->nodes [1], perimeter, runningList);
					markedNodes [1] = start->nodes [1];
				}
			}
			if (perimeter->getX() <= start->x && perimeter->getY() + perimeter->getHeight() > start->y){
				if (start->nodes [2] 
						&& markedNodes [0] != start->nodes [2]
						&& markedNodes [1] != start->nodes [2]){
					getItems (start->nodes [2], perimeter, runningList);
					markedNodes [2] = start->nodes [2];
				}
			}
			if (perimeter->getX() + perimeter->getWidth() > start->x && perimeter->getY() + perimeter->getHeight() > start->y){
				if (start->nodes [3]
						&& markedNodes [1] != start->nodes [3]
						&& markedNodes [2] != start->nodes [3]){
					getItems (start->nodes [3], perimeter, runningList);
				}
			}
			if (start->item->intersects (*perimeter)) runningList.push_back (start->item);
		}
		bool insertItem (Node* start, Node* toPlace){
			if (start == toPlace || start->searched) return true; 
			start->searched = true;
			// TODO: this is inefficient- should be a way to prevent a single node from being called more than once in an insert chain!
			//	Could fix with a flag that gets set during insert and reset after all are done? Not too brilliant but 'twill do. 
			//cout << "Inserting " << toPlace->item->toString() << " in node " << start << " with rect " << start->item->toString() << endl;
			Node* markedNodes [3]; markedNodes [0] = NULL; markedNodes [1] = NULL; markedNodes [2] = NULL;
			if (toPlace->item->getX() <= start->x && toPlace->item->getY() <= start->y){
				if (start->nodes [0]){
					insertItem (start->nodes [0], toPlace);
					markedNodes [0] = start->nodes [0];
				}
				else{
					start->nodes [0] = toPlace;
				}
			}
			if (toPlace->item->getX() + toPlace->item->getWidth() > start->x && toPlace->item->getY() <= start->y){
				if (start->nodes [1]){
					if (markedNodes [0] != start->nodes [1]){
						insertItem (start->nodes [1], toPlace);
						markedNodes [1] = start->nodes [1];
					}
				}
				else{
					start->nodes [1] = toPlace;
				}
			}
			if (toPlace->item->getX() <= start->x && toPlace->item->getY() + toPlace->item->getHeight() > start->y){
				if (start->nodes [2]){
					if (markedNodes [0] != start->nodes [2] && markedNodes [1] != start->nodes [2]){
						insertItem (start->nodes [2], toPlace);
						markedNodes [2] = start->nodes [2];
					}
				}
				else{
					start->nodes [2] = toPlace;
				}
			}
			if (toPlace->item->getX() + toPlace->item->getWidth() > start->x && 
					toPlace->item->getY() + toPlace->item->getHeight() > start->y){
				if (start->nodes [3]){
					if (markedNodes [1] != start->nodes [3] && markedNodes [2] != start->nodes [3]){
						insertItem (start->nodes [3], toPlace);
					}
				}
				else{
					start->nodes [3] = toPlace; 
				}
			}
			return true;
		}
		void deleteNode (Node* kill, const bool& parent = true, vector<Node*>* toKill = NULL){
			if (parent) toKill = new vector<Node*>();

			if (kill->searched) return;
			kill->searched = true;
			if (kill->nodes [0]){
				deleteNode (kill->nodes [0], false, toKill);
			}
			if (kill->nodes [1]){
				deleteNode (kill->nodes [1], false, toKill);
			}
			if (kill->nodes [2]){
				deleteNode (kill->nodes [2], false, toKill);
			}
			if (kill->nodes [3]){
				deleteNode (kill->nodes [3], false, toKill);
			}
			toKill->push_back (kill);
	
			if (parent){
				for (vector<Node*>::iterator iter = toKill->begin(); iter != toKill->end(); iter++){
					killNode (*iter);
				}
			}
		}
};


#endif
