#include "resourcePool.h"
#include "audioManager.h"
#include <iostream>

using namespace std;


ResourcePool::ResourcePool(){
	inputHandler = new InputHandler();
	//cout << "Resource pool's new input handler is at " << inputHandler << endl;
	lockMutex = SDL_CreateMutex();
	cpuAbilities = Utils::getCpuAbilities();
}
ResourcePool::~ResourcePool(){
	for (map< string, SDL_Surface* >::iterator next = bitmaps.begin();
			next != bitmaps.end(); next++){
		SDL_FreeSurface (next->second);
	}

	//cout << "Resource pool is killing its inputHandler." << endl;
	delete inputHandler; inputHandler = NULL;
	SDL_DestroyMutex (lockMutex); lockMutex = NULL;
}

void ResourcePool::lock(){
	SDL_LockMutex (lockMutex);
}
void ResourcePool::unlock(){
	SDL_UnlockMutex (lockMutex);
}

InputHandler* ResourcePool::getInputHandler(){
	//cout << "Returning inputHandler at " << inputHandler << endl;
	return inputHandler;
}
Audio::AudioManager* ResourcePool::getAudioManager(){
	return &Audio::audioManager_;
}

AnimationSet* ResourcePool::loadAnimationSet (string definitionFile){
	if (definitionFile == ""){
		cout << "ResourcePool::loadAnimationSet() got a blank definitionFile." << endl;
		return NULL;
	}
	bool found = false;
	//cout << "I've collected " << animationSets.size() << " animation sets." << endl;
	for (map < string, AnimationSet* >::iterator next = animationSets.begin();
			!found && next != animationSets.end(); next++){
		//cout << "Comparing " << next->first << " and " << definitionFile << endl;
		if (next->first == definitionFile){
			found = true;
		    //cout << "Found animation set in cache." << endl;
		}
	}
	if (!found){
	    //printf ("Loading animation set %s.\n", definitionFile.c_str());
		AnimationSet* model = new AnimationSet (definitionFile, this);
		animationSets [definitionFile] = model;
	    //cout << "Done loading animation set." << endl;
	}
	//cout << "In ResourcePool::loadAnimationSet(), animationSets [definitionFile] is " 
	//	<< animationSets [definitionFile] << endl;
	Interface_Cloneable* cloned = animationSets [definitionFile]->clone();
	//cout << "In ResourcePool::loadAnimationSet(), cloned Cloneable is " << cloned << endl;
	AnimationSet* retSet = dynamic_cast< AnimationSet* >(cloned);
	//cout << "In ResourcePool::loadAnimationSet(), cast animationSet is " << retSet << endl;
	return retSet;
}

Animation* ResourcePool::loadAnimation (int index, string definitionFile){
	if (definitionFile == ""){
		return NULL;
	}
	bool found = false;
	//cout << "I've collected " << animations.size() << " animations." << endl;
	for (map < string, Animation* >::iterator next = animations.begin();
			!found && next != animations.end(); next++){
		//cout << "Comparing " << next->first << " and " << definitionFile << endl;
		if (next->first == definitionFile){
			found = true;
		}
	}
	if (!found){
	    //printf ("Loading animation %s.\n", definitionFile.c_str());
		Animation* model = new Animation (this, index, definitionFile);
		animations [definitionFile] = model;
	    //cout << "Done loading animation." << endl;
	}
	return dynamic_cast<Animation*> (animations [definitionFile]->clone());
}

StampSet* ResourcePool::loadStampSet (string definitionFile){
	if (definitionFile == ""){
		return NULL;
	}
	bool found = false;
	//cout << "I've collected " << stampSets.size() << " stamp sets." << endl;
	for (map < string, StampSet* >::iterator next = stampSets.begin();
			!found && next != stampSets.end(); next++){
		//cout << "Comparing " << next->first << " and " << definitionFile << endl;
		if (next->first == definitionFile){
			found = true;
		}
	}
	if (!found){
	    //printf ("Loading stamp set %s.\n", definitionFile.c_str());
		StampSet* model = new StampSet (this, definitionFile);
		stampSets [definitionFile] = model;
	    //cout << "Done loading stamp set." << endl;
	}
	return dynamic_cast<StampSet*> (stampSets [definitionFile]->clone());
}

/*
Stamp* ResourcePool::loadStamp (string definitionFile){
	if (definitionFile == ""){
		return NULL;
	}
	bool found = false;
	//cout << "I've collected " << stamps.size() << " stamps." << endl;
	for (map < string, Stamp* >::iterator next = stamps.begin();
			!found && next != stamps.end(); next++){
		//cout << "Comparing " << next->first << " and " << definitionFile << endl;
		if (next->first == definitionFile){
			found = true;
		}
	}
	Stamp* stamp;
	if (!found){
	    //printf ("Loading stamp %s.\n", definitionFile.c_str());
		Stamp* model = new Stamp (definitionFile);
		stamps [definitionFile] = model;
	    //cout << "Done loading stamp." << endl;
	}
	return dynamic_cast<Stamp*> (stamps [definitionFile]->clone());
}
*/

SDL_Surface* ResourcePool::loadBitmap (string bmpFile){
	if (bmpFile == ""){
		return NULL;
	}
	bool found = false;
	//cout << "I've collected " << bitmaps.size() << " bitmaps." << endl;
	for (map < string, SDL_Surface* >::iterator next = bitmaps.begin();
			!found && next != bitmaps.end(); next++){
		//cout << "Comparing " << next->first << " and " << bmpFile << endl;
		if (next->first == bmpFile){
			found = true;
		}
	}
	SDL_Surface* surface = NULL;
	if (!found){
	    //printf ("Loading bitmap %s.\n", bmpFile.c_str());
        SDL_Surface* tempSurface = SDL_LoadBMP (bmpFile.c_str());
        if (tempSurface) surface = SDL_DisplayFormat (tempSurface);
        if (tempSurface) SDL_FreeSurface (tempSurface);
		bitmaps [bmpFile] = surface;
		if (surface && surface->format && surface->format->BytesPerPixel){
			//cout << "Loaded " << bmpFile << " as " << surface->format->BytesPerPixel * 8 << "-bit surface" << endl;
		}
		else{
			//cout << "Loaded " << bmpFile << endl;
		}
	    //cout << "Done loading bitmap." << endl;
	}
	/*
	if (SDL_GetError() != ""){
		cout << "SDL error! " << SDL_GetError() << endl;
	}
	*/
	return bitmaps [bmpFile];
}
