#ifndef RESOURCE_POOL_H
#define RESOURCE_POOL_H

#include <SDL/SDL.h>
#include <string>
#include <map>
#include <vector>
#include "stamp.h"
#include "stampSet.h"
#include "animation.h"
#include "animationSet.h"
#include "inputHandler.h"

using namespace std;


class Stamp;
class StampSet;
class Animation;
class AnimationSet;
namespace Audio{
	class AudioManager;
}

class ResourcePool{
	public:
		ResourcePool();
		~ResourcePool();
	
		SDL_Surface* loadBitmap (string bmpFile);
		AnimationSet* loadAnimationSet (string definitionFile);
		Animation* loadAnimation (int index, string definitionFile);
		StampSet* loadStampSet (string definitionFile);

		InputHandler* getInputHandler();
		Audio::AudioManager* getAudioManager();

		unsigned int getCpuAbilities() const{ return cpuAbilities; }

	protected:
		void lock();
		void unlock();

	private:
		map< string, SDL_Surface* > bitmaps;
		map< string, AnimationSet* > animationSets;
		map< string, Animation* > animations;
		map< string, StampSet* > stampSets;
		map< string, Stamp* > stamps;

		InputHandler* inputHandler;

		SDL_mutex* lockMutex;

		unsigned int cpuAbilities;
};

#endif
