/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "screen.h"
#include "microTileField.h"
#include "imageProcessingSurfaceManager.h"
#include "transition.h"
#include <algorithm>


Screen::Screen() : Surface (NULL)
{
    // Horizontal res, vertical res, bpp (0 automatically sets best), mode
    int horizontal = 640; int vertical = 480;
	//this->surface = SDL_SetVideoMode(horizontal, vertical, 32, SDL_SWSURFACE); // Currently limited to 32-bit mode (pixel ops expect Int32.)
	setSurface (SDL_SetVideoMode(horizontal, vertical, 16, SDL_SWSURFACE)); // Currently limited to 16-bit mode (pixel ops expect Int32.)
	/*
	cout << "Video info: HW? " << SDL_GetVideoInfo()->hw_available << " WM? " << SDL_GetVideoInfo()->wm_available << endl;
	cout << "Video mode: " << SDL_GetVideoInfo()->vfmt->BytesPerPixel * 8 << "-bit" << endl;
	cout << "Blits: " << SDL_GetVideoInfo()->blit_hw << " " << SDL_GetVideoInfo()->blit_hw_CC << " " << SDL_GetVideoInfo()->blit_hw_A;
	cout << " " << SDL_GetVideoInfo()->blit_sw << " " << SDL_GetVideoInfo()->blit_sw_CC << " " << SDL_GetVideoInfo()->blit_sw_A;
	cout << " fill: " << SDL_GetVideoInfo()->blit_fill << " video mem: " << SDL_GetVideoInfo()->video_mem << endl;
	*/
    getRect()->setHeight (vertical);
    getRect()->setWidth (horizontal);
    //printf ("Screen constructor done.\n");
	//microTiles = new MicroTileField (horizontal, vertical, 64, 48);
	//microTiles = new MicroTileField (horizontal, vertical, 32, 24);
	microTiles = new MicroTileField (horizontal, vertical, 1, 1);
	//microTiles = new MicroTileField (horizontal, vertical, 20, 15);
	//microTiles = new MicroTileField (horizontal, vertical, 10, 5);
	imageProcessingSurfaceManager = new ImageProcessingSurfaceManager (getSurface(), *getScreenRect());
}

Screen::~Screen()
{
	delete microTiles;
	delete imageProcessingSurfaceManager;
	//printf ("Screen destroying.\n");
}

bool Screen::update (unsigned int timePassed){
	return Surface::update (timePassed);
}

const Rect* Screen::getScreenRect(){
	return getRect();
}
void Screen::getRectsToDraw (const Rect& intersecting, vector<Rect*>& runningList){
	microTiles->getConsolidatedRects (intersecting, runningList);
	/*
	cout << "Screen is saying you need to draw rects ";
	for (vector<Rect*>::iterator iter = runningList.begin(); iter != runningList.end(); iter++){
		cout << (*iter)->toString() << ", ";
	}
	cout << endl << endl;
	*/
}
void Screen::forcedAddDirtyRects (const vector<Rect>& newRects){
	microTiles->dirty (newRects);
}
void Screen::draw(){
	//cout << "Screen drawing." << endl;
	if (getSurface()){
		vector<Rect> allDirtyRects;
		// First, grab all the dirty rectangles from children.
		//cout << "Screen has " << children.size() << " children." << endl;
		Rect dirtyPerimeter = *getRect();
		vector<Surface*> children = getChildren();
		for (vector<Surface*>::iterator kid = children.begin(); kid != children.end(); kid++){
			vector<Rect> thisKidsRects; (*kid)->getDirtyRects (dirtyPerimeter, thisKidsRects);
			for (vector<Rect>::iterator nextRect = thisKidsRects.begin(); nextRect != thisKidsRects.end(); nextRect++){
				if (dirtyPerimeter.intersects (*nextRect)) allDirtyRects.push_back (*nextRect);
			}
		}
		vector<Rect> effectRects = imageProcessingSurfaceManager->getDirtyEffectRects();
		for (vector<Rect>::iterator nextDirtyEffectRect = effectRects.begin(); nextDirtyEffectRect != effectRects.end(); nextDirtyEffectRect++){
			allDirtyRects.push_back (*nextDirtyEffectRect);
		}
		/*
		if (allDirtyRects.size() > 0) cout << "Screen got " << allDirtyRects.size() << " dirty rects." << endl;
		for (vector<Rect>::iterator iter = allDirtyRects.begin(); iter != allDirtyRects.end(); iter++){
			cout << (*iter).toString() << ", ";
		}
		cout << endl << endl;
		*/
		// Next, consolidate the rects so they occupy exclusive areas.
		microTiles->reset(); // Microtiles now reset as they are returned in dirty rects.
		imageProcessingSurfaceManager->reset();
		//this->microTiles = MicroTileField (640, 480, 20, 15);
		microTiles->dirty (allDirtyRects);
		vector<Rect*> screenRects;
		microTiles->getConsolidatedRects (*getRect(), screenRects);

		// Finally, tell all the children to draw using our list of dirty rects as clip boxes.
		screenRects.clear();
		children.clear(); children = getChildren();
		// TODO: sort children properly.
	    for (vector<Surface*>::iterator iter = children.begin(); iter != children.end(); iter++){
 		    (*iter)->draw (imageProcessingSurfaceManager, this);
 		}
	    for (vector<Surface*>::iterator iter = children.begin(); iter != children.end(); iter++){
 		    (*iter)->postDraw (imageProcessingSurfaceManager, this);
 		}

		microTiles->getConsolidatedRects (*getRect(), screenRects);
		for (vector<Rect*>::const_iterator iter = screenRects.begin(); iter != screenRects.end(); iter++){
			SDL_UpdateRect ( getSurface(), int ((*iter)->getX()), int ((*iter)->getY()), int ((*iter)->getWidth()), int ((*iter)->getHeight()) );
		}
	}
	//cout << "Screen done drawing. Dirty? " << isDirty() << endl;
}

