#ifndef SOUND_INSTANCE_H
#define SOUND_INSTANCE_H

#include <string>
#include <vector>
#include "interface_lockable.h"


class Interface_SoundInstanceObserver;
class AudioEffect;
class SoundInstance : public Interface_Lockable{
	public:
		enum State { statePlaying, statePaused, stateStopped, stateRepeating };

		SoundInstance (const std::string& name, Uint32 length, double bytesPerMs);
		virtual ~SoundInstance();

		double getPan();
		void setPan (double);
		double getVolume();
		void setVolume (double);
		Uint32 getPlayPosition();
		void setPlayPosition (Uint32);
		Uint32 getBufferPosition();
		void setBufferPosition (Uint32);
		Uint32 getLength();
		Uint32 getTimeLength();
		void setLength (Uint32);
		State getState();
		void play (bool repeating = false);
		void pause();
		void stop();
		std::string getName();
		void addObserver (Interface_SoundInstanceObserver*);
		void removeObserver (Interface_SoundInstanceObserver*);
		bool getInUse();
		void setInUse (bool inUse);

		void update (unsigned int timePassed);
		unsigned int getTimePassed();
		void setTimePassed (unsigned int time);

		void addEffect (AudioEffect*);
		void removeEffect (AudioEffect*);
		bool hasEffect (AudioEffect*);
		std::vector<AudioEffect*> getEffects();

	private:
		void reset();
		std::vector<Interface_SoundInstanceObserver*>::iterator findObserver (Interface_SoundInstanceObserver*);
		Uint32 getLastPlayTimePosition();
		void setLastPlayTimePosition (Uint32);
		Uint32 getLastPlayTime();
		void setLastPlayTime (Uint32);
		void setState (State);
		std::vector<AudioEffect*>::iterator findEffect (AudioEffect*);

		// Don't lock more than one of these at a time.
		std::string name_; Interface_Lockable nameLock_;
		Uint32 bufferPosition_; Interface_Lockable bufferPositionLock_;
		Uint32 length_; Interface_Lockable lengthLock_;
		Uint32 lastPlayTime_; Interface_Lockable lastPlayTimeLock_;
		Uint32 lastPlayTimePosition_; Interface_Lockable lastPlayTimePositionLock_;
		State state_; Interface_Lockable stateLock_;
		double pan_; Interface_Lockable panLock_; // [-1 .. 1] 
		double volume_; Interface_Lockable volumeLock_; // [0 .. 1]
		std::vector<Interface_SoundInstanceObserver*> observers_; Interface_Lockable observersLock_;
		double bytesPerMs_; Interface_Lockable bytesPerMsLock_;
		bool inUse_; Interface_Lockable inUseLock_;
		std::vector<AudioEffect*> effects_; Interface_Lockable effectsLock_;
		unsigned int timePassed_; Interface_Lockable timePassedLock_;
};


#endif
