/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "sprite.h"
#include "geometry.h"

// pixels per second
#define START_SPEED 20
#define TOLERANCE .1


Sprite::Sprite (Surface* parent, string bmpImage, const Rect& movementPerimeter) : Surface (parent, bmpImage){
	this->movementPerimeter = new Rect(); this->movementPerimeter->copyFrom (movementPerimeter);
	//cout << "Sprite constructor 1 called " << endl;
	//cout << "Sprite constructed with movement perimeter " << this->movementPerimeter->toString() << endl;
	nextDest = NULL;
	speed = START_SPEED;
}
Sprite::Sprite (Surface* parent, string bmpImage) : Surface (parent, bmpImage){
	//cout << "Sprite constructor 2 called " << endl;
	movementPerimeter = NULL;
	nextDest = NULL;
	speed = START_SPEED;
}
Sprite::Sprite (Surface* parent) : Surface (parent){
	//cout << "Sprite constructor 3 called " << endl;
	movementPerimeter = NULL;
	nextDest = NULL;
	speed = START_SPEED;
}
Sprite::~Sprite(){
    //printf ("Sprite destroying.\n");
	if (nextDest) delete nextDest; 
	if (movementPerimeter) delete movementPerimeter;
	nextDest = NULL;
}

bool Sprite::update (unsigned int timePassed){
	//cout << "Sprite updating" << endl;
    //cout << "My loc: " << getX() << "," << getY() << endl;
	unsigned int oldTimePassed = timePassed;

    if (!nextDest && hasDestinations()){
    	headTo (popDestination());
    }
	/*
	if (nextDest){
		cout << "Time to next dest: " << getTimeToNextDestInMs() << endl;
	}
	else{
		cout << "No dest." << endl;
	}
	*/
    while (nextDest && getTimeToNextDestInMs() < timePassed){
    	//cout << "Arriving. . ." << endl;
        timePassed -= int (getTimeToNextDestInMs());
        setXY ( int (nextDest->getX()), int (nextDest->getY()) );
        delete nextDest; nextDest = NULL;
        if (hasDestinations()){
        	headTo (popDestination());
	        //cout << "My loc after headto: " << getX() << "," << getY() << endl;
        }
    }
    if (nextDest){
	    double moveX = getVectorToNextDest().getX() * (getSpeedInPixelsPerMs() * timePassed);
        double moveY = getVectorToNextDest().getY() * (getSpeedInPixelsPerMs() * timePassed);
        
		/*
        cout << "Time passed: " << timePassed << endl;
        cout << "Speed in px per ms: " << getSpeedInPixelsPerMs() << endl;
        cout << "Next dest: " << nextDest->getX() << "," << nextDest->getY() << endl;
        cout << "Vector to nextDest: " << getVectorToNextDest().getX() << "," << getVectorToNextDest().getY() << endl;
        cout << "Moving sprite " << moveX << "," << moveY << endl;
        cout << "My loc before moveXY: " << getX() << "," << getY() << endl;
		*/
        
        moveXY (moveX, moveY);
        //cout << "My loc after moveXY: " << getX() << "," << getY() << endl;
    }

    return Surface::update (oldTimePassed);
}

void Sprite::setSpeed (const double& newSpeed){
	speed = newSpeed;
}

void Sprite::moveXY (const double& x, const double& y){
	//cout << "Calling moveXY" << endl;
	if (movementPerimeter){
		//cout << "Movement perimeter is " << movementPerimeter->toString() << endl;
		if ( (x || y) && movementPerimeter->containsPoint (Point (getX() + x, getY() + y)) ) setXY (getX() + x, getY() + y);
	}
	else{
		if (x) setX (getX() + x);
		if (y) setY (getY() + y);
	}
}

const Rect& Sprite::getMovementPerimeter() const{
	return *movementPerimeter;
}
void Sprite::setMovementPerimeter (const Rect& perimeter){
	if (movementPerimeter) delete movementPerimeter;
	movementPerimeter = new Rect();
	movementPerimeter->copyFrom (perimeter);
}

bool Sprite::headTo (const Point& dest){
	return headTo (dest.getX(), dest.getY());
}
bool Sprite::headTo (const double& x, const double& y){
    //cout << "Heading to " << x << "," << y << endl;
    if (nextDest){
		delete nextDest; nextDest = NULL;
    }
   	nextDest = new Point (x, y);
    return true;
}
bool Sprite::isDoneMoving() const{ return nextDest ? false : true; }

double Sprite::getSpeedInPixelsPerSecond() const{
	return speed;
}
double Sprite::getSpeedInPixelsPerMs() const{
    return getSpeedInPixelsPerSecond() / 1000.0;
}

bool Sprite::isAtDestination(){
	if (isDoneMoving()){
    	string message = "isAtDestination(): not moving!";
        cout << message << endl; throw message;
    }
    return Utils::absFloat (getX() - nextDest->getX()) < TOLERANCE && Utils::absFloat (getY() - nextDest->getY()) < TOLERANCE;
}

Point Sprite::getVectorToNextDest(){
	if (isDoneMoving()){
    	string message = "getVectorToNextDest(): nextDest is NULL!";
        cout << message; throw message;
    }
    double xRemaining = nextDest->getX() - getX();
    double yRemaining = nextDest->getY() - getY();
    return Utils::getNormalized2dVector (Point (xRemaining, yRemaining));
}
double Sprite::getTimeToNextDestInMs(){
    if (isDoneMoving()){
    	string message = "getTimeToNextDestInMs(): no destination set!";
        cout << message; throw message;
    }
	//cout << "Distance to nextDest: " << Utils::get2dDistance (Point (getX(), getY()), *nextDest) << endl;
    return Utils::get2dDistance (Point (getX(), getY()), *nextDest) / getSpeedInPixelsPerMs();
}

void Sprite::addDestination (const Point& destination){
	destinations.push_back (destination);
}
bool Sprite::hasDestinations() const{
    return destinations.size() > 0;
}
Point Sprite::getCurrentDestination() const{
	if (isDoneMoving()){
    	string message = "getCurrentDestination(): no destination set!";
        cout << message; throw message;
    }
	return *nextDest;
}
Point Sprite::peekDestination() const{
    if (destinations.size() > 0){
	    return destinations [0];
    }
    string message = "No destinations in queue!";
    cout << message; throw message;
}
void Sprite::clearDestinations(){
	destinations.clear();
}
Point Sprite::popDestination(){
    if (destinations.size() > 0){
    	Point retPoint = destinations [0];
        destinations.erase (destinations.begin());
     	return retPoint;
    }
    string message = "No destinations in queue!";
    cout << message; throw message;
}

