/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SPRITE_H
#define SPRITE_H

#include <iostream>
#include "surface.h"
#include "utils.h"


using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/
class Sprite : public Surface{
	public:
		Sprite (Surface* parent, string bmpImage, const Rect& movementPerimeter);
		Sprite (Surface* parent, string bmpImage);
		Sprite (Surface* parent);
		virtual ~Sprite();

		virtual bool update (unsigned int timePassed);
		void moveXY (const double& x, const double& y);
		void moveX (const double& x){ moveXY (x, 0); }
		void moveY (const double& y){ moveXY (0, y); }
		double getWidth(){ return getRect()->getWidth(); }
		double getHeight(){ return getRect()->getHeight(); }
		double getCenterX(){ return (getWidth() / 2) + getX(); }
		double getCenterY(){ return (getHeight() / 2) + getY(); }
		double getCenterAbsoluteX(){ return (getWidth() / 2) + getAbsoluteX(); }
		double getCenterAbsoluteY(){ return (getHeight() / 2) + getAbsoluteY(); }
		void setCenterX (double x){ setX (x - (getWidth() / 2) ); }
		void setCenterY (double y){ setY (y - (getHeight() / 2) ); }

		bool isDoneMoving() const;
		virtual bool headTo (const Point& dest);
		virtual bool headTo (const double& x, const double& y);
		void addDestination (const Point& dest);
		void clearDestinations();
		bool hasDestinations() const;
		Point getCurrentDestination() const;
		void setSpeed (const double& newSpeed);
		double getSpeedInPixelsPerSecond() const;
		double getSpeedInPixelsPerMs() const;
		double getTimeToNextDestInMs();
		Point peekDestination() const;

		const Rect& getMovementPerimeter() const;
		void setMovementPerimeter (const Rect& perimeter);

	private:
		bool isAtDestination();
		Point popDestination();
		Point getVectorToNextDest();

		double speed;
		Point* nextDest;
		vector<Point> destinations;

		Rect* movementPerimeter;
};

#endif
