/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "stampSet.h"
#include <iostream>
#include <fstream>

using namespace std;


StampSet::StampSet (ResourcePool* resources, string definitionFile){
	image = NULL;
	this->resources = resources;
    if (definitionFile != ""){
    	this->load (definitionFile);
    }
    else{
        image = NULL;
    }
	//cout << "Image is " << this->image << endl;
}
StampSet::StampSet (StampSetSettings settings){
	this->stamps = settings.getStamps();
    this->image = settings.getImage();
    this->resources = settings.getResources();
}

StampSet::~StampSet(){
    cleanup();
}

Interface_Cloneable* StampSet::clone() const{
    map< int, Stamp* > newStamps;
    for (map< int, Stamp* >::const_iterator iter = stamps.begin(); iter != stamps.end(); iter++){
		newStamps [iter->first] = dynamic_cast< Stamp* > (iter->second->clone());
    }
    StampSetSettings settings (newStamps, image, resources);
    return new StampSet (settings);
}

void StampSet::cleanup(){
	for (map< int, Stamp* >::iterator next = stamps.begin(); 
			next != stamps.end(); next++){
    	delete (next->second); 
    }
    stamps.clear();
}

SDL_Surface* StampSet::getCurrentSurface(){
	return image;
}

SDL_Rect StampSet::getRect (int index){
	SDL_Rect retRect = stamps [index]->getRect();
	//cout << "StampSet::getRect() returning: " << retRect.x << ", " << retRect.y 
	//	<< ", " << retRect.w << ", " << retRect.h << "." << endl;
	return retRect;
}

void StampSet::load (string definitionFile){
	//cout << "Loading a stamp set's image!" << endl;
    ifstream defFile;
    defFile.open (definitionFile.c_str());
    unsigned int frameWidth, frameHeight;
	int frameOffsetX, frameOffsetY, frameIndex;
    string imageFile;
	int childCount = 0;

    defFile >> imageFile;
	if (resources){
		this->image = resources->loadBitmap (imageFile);
	}
	// Get the image file name.
	defFile >> frameIndex;
	defFile >> frameOffsetX;
	defFile >> frameOffsetY;
    defFile >> frameWidth;
    defFile >> frameHeight;
	while (! defFile.eof()){
		/*
		cout << "StampSet's definition file: '" << definitionFile << "'." << endl;
		cout << "StampSet's imageFile: " << imageFile << ", frameHeight: " << frameHeight 
			<< ", frameWidth: " << frameWidth << ", offsetX: " << frameOffsetX
			<< ", offsetY: " << frameOffsetY  
			<< endl;
		*/

		// Now set up this stamp set's stamps.
		stamps [frameIndex] = new Stamp (frameOffsetX, frameOffsetY, frameWidth, frameHeight);
		childCount++;

		defFile >> frameIndex;
		defFile >> frameOffsetX;
		defFile >> frameOffsetY;
	    defFile >> frameWidth;
	    defFile >> frameHeight;
	}
	//cout << "Loaded " << childCount << " children." << endl;
    defFile.close();
}
