#include "../audioManager.h"
#include "../testUtils.h"
#include "../soundInstanceReference.h"
#include "../audioEffectEcho.h"
#include "../audioEffectTone.h"
#include "../audioEffectNone.h"
#include <iostream>
#include <SDL/SDL.h>
#include "../testUtils.h"
#include "../audioProcessingManager.h"
#include "../utils.h"


namespace Audio{
	class AudioManagerTester{
		public: 
			void applyPanning16 (Uint32* data, unsigned int numSamples, double pan){
				Audio::audioManager_.applyPanning16 (data, numSamples, pan);
			}
	};
}

int main (int numArgs, char** args){
	/*
	{
		Audio::AudioData crasher1 (16, 44100);
		for (int x = 0; x < 100; x++){
			crasher1.setSample (x, Audio::AudioSample (123, 456, 16));
		}
		Audio::AudioData crasher2 (16, 44100);
		for (int x = 0; x < 100; x++){
			crasher2.setSample (x, Audio::AudioSample (987, 654, 16));
		}
		Audio::AudioData crasher3 (16, 44100);
		for (int x = 0; x < 100; x++){
			crasher3 = crasher1;
			crasher1 = crasher2;
			crasher2 = crasher3;
		}
	}
	*/
	{
		Audio::AudioData testData (16, 44100);
		testData.setSample (0, Audio::AudioSample (100, 69, 16));
		testData.setSample (1, Audio::AudioSample (911, 319, 16));
		cout << check ("audio data test 1", 100, testData.getSample (0).getLeftSample()) << endl;
		cout << check ("audio data test 2", 69, testData.getSample (0).getRightSample()) << endl;
		cout << check ("audio data test 2.1", 911, testData.getSample (1).getLeftSample()) << endl;
		cout << check ("audio data test 2.2", 319, testData.getSample (1).getRightSample()) << endl;
		Audio::AudioData testData2 (testData);
		cout << check ("audio data test 3", 100, testData2.getSample (0).getLeftSample()) << endl;
		cout << check ("audio data test 4", 69, testData2.getSample (0).getRightSample()) << endl;
		cout << check ("audio data test 4.1", 911, testData2.getSample (1).getLeftSample()) << endl;
		cout << check ("audio data test 4.2", 319, testData2.getSample (1).getRightSample()) << endl;
		testData2.setSample (0, Audio::AudioSample (123, 987, 16));
		cout << check ("audio data test 5", 123, testData2.getSample (0).getLeftSample()) << endl;
		cout << check ("audio data test 6", 987, testData2.getSample (0).getRightSample()) << endl;
		cout << check ("audio data test 6.1", 911, testData2.getSample (1).getLeftSample()) << endl;
		cout << check ("audio data test 6.2", 319, testData2.getSample (1).getRightSample()) << endl;
		Interface_AudioProcessingManager* processor = new Audio::AudioProcessingManager (16, 44100);
		processor->startEffectStack (testData, 0, 2);
		processor->setCompositeSample (0, Audio::AudioSample (135, 791, 16));
		processor->setCompositeSample (1, Audio::AudioSample (246, 975, 16));
		processor->endEffectStack();
		Audio::AudioData testData3 (processor->getCompositeAudioData());
		cout << check ("audio data test 7", 135, testData3.getSample (0).getLeftSample()) << endl;
		cout << check ("audio data test 8", 791, testData3.getSample (0).getRightSample()) << endl;
		cout << check ("audio data test 8.1", 246, testData3.getSample (1).getLeftSample()) << endl;
		cout << check ("audio data test 8.2", 975, testData3.getSample (1).getRightSample()) << endl;
		delete processor;
	}
	{
		{
			Audio::AudioSample test1 (0, 0, 16);
			Audio::AudioSample test2 (0, 0, 16);
			Audio::AudioSample blend1 (Utils::audioBlend (test1, test2));
			cout << check ("audio blend test 1", 0, blend1.getLeftSample()) << endl;
			cout << check ("audio blend test 2", 0, blend1.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test1 (-1000, -1000, 16);
			Audio::AudioSample test2 (-500, -500, 16);
			Audio::AudioSample blend (Utils::audioBlend (test1, test2));
			cout << check ("audio blend test 2.1", -1500, blend.getLeftSample()) << endl;
			cout << check ("audio blend test 2.2", -1500, blend.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test1 (-500, -500, 16);
			Audio::AudioSample test2 (1000, 1000, 16);
			Audio::AudioSample blend (Utils::audioBlend (test1, test2));
			cout << check ("audio blend test 2.3", 500, blend.getLeftSample()) << endl;
			cout << check ("audio blend test 2.4", 500, blend.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test1 (1000, 1000, 16);
			Audio::AudioSample test2 (-500, -500, 16);
			Audio::AudioSample blend (Utils::audioBlend (test1, test2));
			cout << check ("audio blend test 2.5", 500, blend.getLeftSample()) << endl;
			cout << check ("audio blend test 2.6", 500, blend.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test3 (32765, 32765, 16);
			Audio::AudioSample test4 (32765, 32765, 16);
			Audio::AudioSample blend2 (Utils::audioBlend (test3, test4));
			cout << check ("audio blend test 3", 32765, blend2.getLeftSample()) << endl;
			cout << check ("audio blend test 4", 32765, blend2.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test5 (-32765, -32765, 16);
			Audio::AudioSample test6 (-32765, -32765, 16);
			Audio::AudioSample blend3 (Utils::audioBlend (test5, test6));
			cout << check ("audio blend test 5", -32765, blend3.getLeftSample()) << endl;
			cout << check ("audio blend test 6", -32765, blend3.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test7 (0, 0, 16);
			Audio::AudioSample test8 (-100, -100, 16);
			Audio::AudioSample blend4 (Utils::audioBlend (test7, test8));
			cout << check ("audio blend test 7", -100, blend4.getLeftSample()) << endl;
			cout << check ("audio blend test 8", -100, blend4.getRightSample()) << endl;
		}
		{
			Audio::AudioSample test1 (200, 200, 16);
			Audio::AudioSample test2 (32765, 32765, 16);
			Audio::AudioSample blend (Utils::audioBlend (test1, test2));
			cout << check ("audio blend test 9", 32765, blend.getLeftSample()) << endl;
			cout << check ("audio blend test 10", 32765, blend.getRightSample()) << endl;
		}
		{
			Audio::AudioData test (16, 44100);
			test.setSample (0, Audio::AudioSample (32765, -32765, 16));
			short left = ((short*) &(test.getRaw() [0])) [0];
			short right = ((short*) &(test.getRaw() [0])) [1];
			cout << check ("get raw test 1", 32765, left) << endl;
			cout << check ("get raw test 2", -32765, right) << endl;
		}
		{
			Audio::AudioData test1 (16, 44100);
			test1.setSample (0, Audio::AudioSample (32765, -32765, 16));
			Audio::AudioData test2 (16, 44100);
			test2.copySamplesFrom (test1, 0, 1);
			cout << check ("copy samples from test 1", 32765, test2.getSample (0).getLeftSample()) << endl;
			cout << check ("copy samples from test 2", -32765, test2.getSample (0).getRightSample()) << endl;
		}
		{
			Audio::AudioData test1 (16, 44100);
			test1.setSample (0, Audio::AudioSample (32765, -32765, 16));
			Audio::AudioData test2 (16, 44100);
			test2.copySamplesFrom (test1, 0, 1, 1);
			cout << check ("copy samples from test 3", 32765, test2.getSample (1).getLeftSample()) << endl;
			cout << check ("copy samples from test 4", -32765, test2.getSample (1).getRightSample()) << endl;
			cout << check ("copy samples from test 5", 0, test2.getSample (0).getLeftSample()) << endl;
			cout << check ("copy samples from test 6", 0, test2.getSample (0).getRightSample()) << endl;
			test1.copySamplesFrom (test2, 1, 1, 10);
			cout << check ("copy samples from test 7", 32765, test1.getSample (10).getLeftSample()) << endl;
			cout << check ("copy samples from test 8", -32765, test1.getSample (10).getRightSample()) << endl;
		}
		{
			Audio::AudioData test1 (16, 44100);
			test1.setSample (0, Audio::AudioSample (0, 0, 16));
			Audio::AudioData test2 (16, 44100);
			test2.copySamplesFrom (test1, 100, 1);
			cout << check ("copy samples from test 9", 0, test2.getNumberOfSamples()) << endl;
		}
		{
			Audio::AudioData test1 (16, 44100);
			test1.setSample (0, Audio::AudioSample (1234, 5678, 16));
			Audio::AudioData test2 (16, 44100);
			test2.copySamplesFrom (test1, 0, 100);
			cout << check ("copy samples from test 10", 1, test2.getNumberOfSamples()) << endl;
			cout << check ("copy samples from test 11", 1234, test2.getSample (0).getLeftSample()) << endl;
			cout << check ("copy samples from test 12", 5678, test2.getSample (0).getRightSample()) << endl;
		}
		{
			unsigned int dataSize = 2 * sizeof (short) * 0;
			short* data = (short*) malloc (dataSize);
			Audio::AudioData test1 (16, 44100, (char*) data, dataSize);
			free (data); data = NULL;
			cout << check ("audio data test 1", 0, test1.getNumberOfSamples()) << endl;
		}
		{
			unsigned int dataSize = 2 * sizeof (short) * 1;
			short* data = (short*) malloc (dataSize);
			Audio::AudioData test1 (16, 44100, (char*) data, dataSize);
			free (data); data = NULL;
			cout << check ("audio data test 2", 1, test1.getNumberOfSamples()) << endl;
		}
		{
			unsigned int dataSize = 2 * sizeof (short) * 100;
			short* data = (short*) malloc (dataSize);
			Audio::AudioData test1 (16, 44100, (char*) data, dataSize);
			free (data); data = NULL;
			cout << check ("audio data test 3", 100, test1.getNumberOfSamples()) << endl;
		}
		{
			Audio::AudioData test1 (16, 44100);
			test1.setSample (0, Audio::AudioSample (1234, 5678, 16));
			short* sample = (short*) &(test1.getRaw() [0]);
			cout << check ("audio data test 4", test1.getSample (0).getLeftSample(), *sample) << endl;
			sample++;
			cout << check ("audio data test 5", test1.getSample (0).getRightSample(), *sample) << endl;
		}
		{
			Uint32* data = (Uint32*) malloc (sizeof (Uint32) * 2);
			short* channel = (short*) data;
			channel [0] = 100;
			channel [1] = -100;
			channel [2] = -100;
			channel [3] = 100;
			Audio::AudioManagerTester tester;
			tester.applyPanning16 (data, 2, 0.5);
			cout << check ("apply panning test 1", 100, channel [0]) << endl;
			cout << check ("apply panning test 2", -100, channel [1]) << endl;
			cout << check ("apply panning test 3", -100, channel [2]) << endl;
			cout << check ("apply panning test 4", 100, channel [3]) << endl;
			free (data); data = NULL;
		}
		{
			Uint32* data = (Uint32*) malloc (sizeof (Uint32) * 2);
			short* channel = (short*) data;
			channel [0] = 50;
			channel [1] = 50;
			channel [2] = -50;
			channel [3] = -50;
			Audio::AudioManagerTester tester;
			tester.applyPanning16 (data, 2, 0.0);
			cout << check ("apply panning test 5", 100, channel [0]) << endl;
			cout << check ("apply panning test 6", 0, channel [1]) << endl;
			cout << check ("apply panning test 7", -100, channel [2]) << endl;
			cout << check ("apply panning test 8", 0, channel [3]) << endl;
			free (data); data = NULL;
		}
		{
			Uint32* data = (Uint32*) malloc (sizeof (Uint32) * 2);
			short* channel = (short*) data;
			channel [0] = -50;
			channel [1] = 50;
			channel [2] = 50;
			channel [3] = -50;
			Audio::AudioManagerTester tester;
			tester.applyPanning16 (data, 2, 0.0);
			cout << check ("apply panning test 9", 0, channel [0]) << endl;
			cout << check ("apply panning test 10", 0, channel [1]) << endl;
			cout << check ("apply panning test 11", 0, channel [2]) << endl;
			cout << check ("apply panning test 12", 0, channel [3]) << endl;
			free (data); data = NULL;
		}
		{
			Uint32* data = (Uint32*) malloc (sizeof (Uint32) * 2);
			short* channel = (short*) data;
			channel [0] = 50;
			channel [1] = 50;
			channel [2] = 50;
			channel [3] = 50;
			Audio::AudioManagerTester tester;
			tester.applyPanning16 (data, 2, 1.0);
			cout << check ("apply panning test 13", 0, channel [0]) << endl;
			cout << check ("apply panning test 14", 100, channel [1]) << endl;
			cout << check ("apply panning test 15", 0, channel [2]) << endl;
			cout << check ("apply panning test 16", 100, channel [3]) << endl;
			free (data); data = NULL;
		}
		{
			Uint32* data = (Uint32*) malloc (sizeof (Uint32) * 2);
			short* channel = (short*) data;
			channel [0] = 0;
			channel [1] = 100;
			channel [2] = 0;
			channel [3] = 100;
			Audio::AudioManagerTester tester;
			tester.applyPanning16 (data, 2, 1.0);
			cout << check ("apply panning test 17", 0, channel [0]) << endl;
			cout << check ("apply panning test 18", 100, channel [1]) << endl;
			cout << check ("apply panning test 19", 0, channel [2]) << endl;
			cout << check ("apply panning test 20", 100, channel [3]) << endl;
			free (data); data = NULL;
		}
		/*
		{
			Audio::AudioData test1 (16, 44100);
			Audio::AudioData test2 (16, 44100);
			int samplesCount = 10000;
			for (int x = 0; x < samplesCount; x++){
				int val = x % 60000;
				val -= 30000;
				test1.setSample (x, Audio::AudioSample (-val, val, 16));
			}
			test2.copySamplesFrom (test1);
			bool passing = true;
			for (int x = 0; passing && x < samplesCount; x++){
				if (test1.getSample (x).getLeftSample() != test2.getSample (x).getLeftSample() 
						|| test1.getSample (x).getRightSample() 
							!= test2.getSample (x).getRightSample()){
					passing = false;
				}
			}
			cout << check ("copy samples from test 3", true, passing) << endl;
		}
		*/
	}
	/*
	{
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.playSound ("testSound.wav");
		std::tr1::shared_ptr<SoundInstanceReference> sound2 = sound;

		cout << "Waiting for sound to finish." << endl;

		Uint32 start = SDL_GetTicks();
		Uint32 last = 0;
		Uint32 timeLength = sound->getTimeLength();
		bool paused = false;
		bool restarted = false;
		while (sound->getState() != SoundInstanceReference::stateStopped){
			Uint32 now = SDL_GetTicks();
			Uint32 diff = now - start;
			Audio::audioManager_.update (diff);
			if (paused && diff < timeLength){ 
				continue;
			}
			if (paused && !restarted){
				restarted = true;
				sound->stop();
				sound2->play();
			}
			if (diff % 100 == 0 && now != last){ 
				cout << "<" << sound->getPlayPosition() << ">" << diff << endl;
				last = now;
			}
			if (!paused && timeLength / 2 < sound->getPlayPosition()){
				paused = true;
				sound2->pause();
			}
		}
	}
	{
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.playSound ("testSound.wav");
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
	}
	{
		cout << "Sound should stop after one second. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->play();
		Uint32 last = SDL_GetTicks();
		Uint32 now = SDL_GetTicks();
		while (SDL_GetTicks() - now < 1000){ 
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	*/
	{
		cout << "Sound should be unchanged. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectNone());
		sound->play();
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	{
		cout << "Sound should be panned left. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectNone());
		sound->setPan (0.0);
		sound->play();
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	{
		cout << "Sound should be panned right. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectNone());
		sound->setPan (1.0);
		sound->play();
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	/*
	{
		cout << "Sound should be a 440 hz tone. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectTone (44100.0 / 440));
		sound->setVolume (0.2);
		sound->play();
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	*/
	{
		cout << "Sound should be echoed. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectEcho (200, 0.5));
		sound->play();
		Uint32 last = SDL_GetTicks();
		while (sound->getState() == SoundInstanceReference::statePlaying){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	/*
	{
		cout << "Sound should be repeating. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->play (true);
		Uint32 last = SDL_GetTicks();
		Uint32 start = last;
		while (sound->getState() == SoundInstanceReference::stateRepeating && SDL_GetTicks() - start < 5000){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	{
		cout << "Sound should be repeating with echo. . . " << flush;
		std::tr1::shared_ptr<SoundInstanceReference> sound = Audio::audioManager_.loadSound ("testSound.wav");
		sound->addEffect (new AudioEffectEcho (200, 0.5));
		//sound->setVolume (0.2);
		sound->play (true);
		Uint32 last = SDL_GetTicks();
		Uint32 start = last;
		while (sound->getState() == SoundInstanceReference::stateRepeating && SDL_GetTicks() - start < 5000){
			Audio::audioManager_.update (SDL_GetTicks() - last);
			last = SDL_GetTicks();
		}
		cout << "done" << endl;
	}
	*/

	cout << "test done. use ctrl-c or ctrl-brk to exit" << endl;
	Uint32 start = SDL_GetTicks();
	while (SDL_GetTicks() - start < 1000);

	return 0;
}
