#include "../binList.h"
#include "../testUtils.h"
#include <iostream>
#include <sstream>
#include <algorithm>
#include <math.h>

using namespace std;


class Item{
	public:
		Item (int num) : num_ (num) { }
		~Item(){ }
		bool operator== (const Item& other) const{ return other.getValue() == num_; }

		int getValue() const{ return num_; }
	private:
		int num_;
};

class GenericIndexEvaluator : public Interface_IndexEvaluator<Item>{
	public:
		GenericIndexEvaluator(){ }
		virtual ~GenericIndexEvaluator(){ }
		virtual bool lessThan (const Item& item1, const Item& item2) const{ 
			return item1.getValue() < item2.getValue();
		}
		virtual bool equals (const Item& item1, const Item& item2) const{ 
			return item1.getValue() == item2.getValue();
		}
};

bool hasAll (const vector<Item>& binList, const vector<Item>& list){
	for (vector<Item>::const_iterator iter = list.begin(); iter != list.end(); iter++){
		if (find (binList.begin(), binList.end(), *iter) == binList.end()) return false;
	}
	return true;
}

void addItem (const Item& item, BinList<Item>* binList, vector<Item>& list){
	//cout << item.getValue() << endl;
	binList->append (item);
	list.push_back (item);
}

int main (int numArgs, char** args){
	GenericIndexEvaluator* evaluator = new GenericIndexEvaluator();
	BinList<Item>* binList;
	vector<Item> list;
	vector<Item> fromBinList;

	{
		binList = new BinList<Item> (evaluator);
		list.clear();
		fromBinList.clear();
		addItem (Item (5), binList, list);
		addItem (Item (7), binList, list);
		addItem (Item (3), binList, list);
		addItem (Item (1), binList, list);
		addItem (Item (2), binList, list);
		Item popped = binList->pop();
		cout << check ("Pop test 1", 7, popped.getValue()) << endl;
		cout << check ("Pop test 2", true, binList->hasItem (5)) << endl;
		cout << check ("Pop test 3", true, binList->hasItem (3)) << endl;
		cout << check ("Pop test 4", true, binList->hasItem (1)) << endl;
		cout << check ("Pop test 5", true, binList->hasItem (2)) << endl;
		delete binList;
	}

	{
		binList = new BinList<Item> (evaluator);
		list.clear();
		fromBinList.clear();
		addItem (Item (5), binList, list);
		addItem (Item (7), binList, list);
		addItem (Item (3), binList, list);
		addItem (Item (1), binList, list);
		addItem (Item (2), binList, list);
		Item popped = binList->popFront();
		cout << check ("Pop test 11", 1, popped.getValue()) << endl;
		cout << check ("Pop test 12", true, binList->hasItem (5)) << endl;
		cout << check ("Pop test 13", true, binList->hasItem (3)) << endl;
		cout << check ("Pop test 14", true, binList->hasItem (7)) << endl;
		cout << check ("Pop test 15", true, binList->hasItem (2)) << endl;
		delete binList;
	}

	{
		binList = new BinList<Item> (evaluator);
		list.clear();
		fromBinList.clear();
		addItem (Item (5), binList, list);
		addItem (Item (7), binList, list);
		addItem (Item (3), binList, list);
		addItem (Item (1), binList, list);
		addItem (Item (2), binList, list);
		binList->remove (Item (5));
		cout << check ("Remove test 1", true, binList->hasItem (3)) << endl;
		cout << check ("Remove test 2", true, binList->hasItem (1)) << endl;
		cout << check ("Remove test 3", true, binList->hasItem (7)) << endl;
		cout << check ("Remove test 4", true, binList->hasItem (2)) << endl;
		delete binList;
	}

	{
		binList = new BinList<Item> (evaluator);
		list.clear();
		fromBinList.clear();
		unsigned int randVal = time (NULL);
		cout << "using random seed " << randVal << endl;
		srand (randVal);
		for (int x = 0; x < 1000; x++){
			addItem (Item (rand() % 10000), binList, list);
		}
		binList->getAll (fromBinList);

		cout << check ("append test 1", true, hasAll (fromBinList, list)) << endl;
	
		cout << check ("size test 1", int (list.size()), int (binList->size())) << endl;

		try{
			int y = list.size();
			cout << "Running " << y << " randomized pops, appends, and removes." << endl;
			for (int x = 0; x < y; x++){
				if (x % 2 == 0){
					addItem (Item (rand() % 10000), binList, list);
				}
				if (list.size() > 0){
					Item popped (binList->popFront());
					vector<Item>::iterator loc = find (list.begin(), list.end(), popped);
					if (loc != list.end()) list.erase (loc);
					else{ 
						ostringstream msg;
						msg << x;
						throw msg.str();
					}	
					fromBinList.clear(); binList->getAll (fromBinList);
					if (!hasAll (fromBinList, list)){
						ostringstream msg;
						msg << "Pop failed after " << x << " iterations.";
						throw msg.str();
					}
				}
				
				if (list.size() > 0){
					int toRemove = random() % list.size();
					binList->remove (list [toRemove]);
					list.erase (list.begin() + toRemove);
					fromBinList.clear(); binList->getAll (fromBinList);
					if (!hasAll (fromBinList, list)){
						ostringstream msg;
						msg << "Remove failed after " << x << " iterations.";
						throw msg.str();
					}
				}

				if (list.size() != binList->size()){
					ostringstream msg;
					msg << "Size failed after " << x << " iterations.";
					throw msg.str();
				}
			}
			cout << "Passed random test." << endl;
		}
		catch (string s){
			cout << "Failed random test. " << s << endl;
		}
		delete binList;
	}


	delete evaluator;

	return 0;
}
