#include <SDL/SDL.h>
#include "../imageProcessingSurfaceManager.h"
#include "../testUtils.h"
#include "../utils.h"
#include <iostream>
#include <string>
using namespace std;


int main (int numArgs, char** args){

	// Create a 640x480 16-bit software SDL_Surface.
	SDL_Surface* target = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, 640, 480, 16, 0xf800, 0x7e0, 0x1f, 0x00);
	SDL_SetColorKey (target, SDL_SRCCOLORKEY, 0x00);
	for (int y = 0; y < 480; y++){
		for (int x = 0; x < 640; x++){
			Utils::setPixel (target, x, y, 0x5555);
		}
	}

	SDL_Rect blittingRect; blittingRect.x = 100; blittingRect.y = 100; blittingRect.w = 100; blittingRect.h = 100;
	SDL_Surface* blitting = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, blittingRect.w, blittingRect.h, 16, 0xf800, 0x7e0, 0x1f, 0x00);
	SDL_SetColorKey (blitting, SDL_SRCCOLORKEY, target->format->colorkey);
	SDL_FillRect (blitting, NULL, 0x00);
	for (int x = 10; x < 90; x++){
		Utils::setPixel (blitting, x, 10, 0x3333);
	}
	for (int y = 11; y < 90; y++){
		Utils::setPixel (blitting, 10, y, 0x3333);
		Utils::setPixel (blitting, 90, y, 0x3333);
	}
	for (int x = 10; x < 90; x++){
		Utils::setPixel (blitting, x, 90, 0x3333);
	}


	ImageProcessingSurfaceManager manager (target, Rect (0, 0, 640, 480));

	bool success;

	Rect identityRect (0, 0, 0, 0);
	manager.startEffectStack (blitting, blittingRect, identityRect.toSDL_Rect());

	cout << check ("get composite pixel test 0.1", 0x00, manager.getCompositePixel (0, 0)) << endl;
	cout << check ("get composite pixel test 0.2", 0x3333, manager.getCompositePixel (110, 110)) << endl;
	cout << check ("get composite pixel test 0.3", 0x00, manager.getCompositePixel (111, 111)) << endl;

	cout << check ("get composite rect test 0.5", Rect (0, 0, 0, 0).toString(), manager.getCompositeRect().toString()) << endl;
	manager.setCompositePixel (110, 110, 0x7777);
	cout << check ("get composite pixel test 0.4", 0x7777, manager.getCompositePixel (110, 110)) << endl;
	cout << check ("get target pixel test 0.5", 0x5555, manager.getTargetPixel (110, 110)) << endl;

	cout << check ("get composite rect test 1", Rect (110, 110, 1, 1).toString(), manager.getCompositeRect().toString()) << endl;

	cout << check ("get target surface test 1", target, manager.getTargetSurface()) << endl;

	success = true;
	for (int y = 0; y < 480 && success; y++){
		for (int x = 0; x < 640 && success; x++){
			if (Utils::getPixel (target, x, y) != manager.getTargetPixel (x, y)){
				cout << "get target pixel test 1 failed at " << x << "," << y << "." << endl;
				success = false;
			}
		}
	}
	cout << check ("get target pixel test 1", true, success) << endl;

	for (int y = 0; y < 480; y++){
		for (int x = 0; x < 640; x++){
			manager.setCompositePixel (x, y, (x / 10) * (y / 10));
		}
	}

	success = true;
	for (int y = 0; y < 480 && success; y++){
		for (int x = 0; x < 640 && success; x++){
			if (manager.getCompositePixel (x, y) != (x / 10) * (y / 10)){
				cout << "get composite pixel test 1 failed at " << x << "," << y << "." << endl;
				success = false;
			}
		}
	}
	cout << check ("get composite pixel test 1", true, success) << endl;

	success = true;
	for (int y = 0; y < 480 && success; y++){
		for (int x = 0; x < 640 && success; x++){
			if (Utils::getPixel (target, x, y) != manager.getTargetPixel (x, y)){
				cout << "get target pixel test 2 failed at " << x << "," << y << "." << endl;
				success = false;
			}
		}
	}
	cout << check ("get target pixel test 2", true, success) << endl;

	vector<Rect> bigViewport;
    bigViewport.push_back (Rect (0, 0, 640, 480));
	manager.endEffectStack (bigViewport);

	cout << check ("get dirty effect rects test 1", 1, manager.getDirtyEffectRects().size()) << endl;
	if (manager.getDirtyEffectRects().size() == 1)
		cout << check ("get dirty effect rects test 2", Rect (0, 0, 640, 480).toString(), manager.getDirtyEffectRects() [0].toString()) << endl;

	manager.startEffectStack (blitting, blittingRect, identityRect.toSDL_Rect());
	cout << check ("get composite rect test 2", Rect (0, 0, 0, 0).toString(), manager.getCompositeRect().toString()) << endl;
	manager.endEffectStack (bigViewport);

	cout << check ("get blitting surface rect test 1", Rect (100, 100, 100, 100).toString(), manager.getBlittingRect().toString()) << endl;

	cout << check ("get dirty effect rects test 3", 1, manager.getDirtyEffectRects().size()) << endl;

	manager.reset();

	cout << check ("get dirty effect rects test 4", 0, manager.getDirtyEffectRects().size()) << endl;
	
	cout << check ("get target surface test 2", target, manager.getTargetSurface()) << endl;



	// Reset the target surface.
	for (int y = 0; y < 480; y++){
		for (int x = 0; x < 640; x++){
			Utils::setPixel (target, x, y, 0x5555);
		}
	}

	manager.startEffectStack (blitting, blittingRect, identityRect.toSDL_Rect());

	manager.setCompositePixel (110, 110, 0x7777);
	cout << check ("set composite pixel test 1", 0x7777, manager.getCompositePixel (110, 110)) << endl;

	manager.endEffectStack (bigViewport);

	cout << check ("end effect stack test 1", 0x7777, Utils::getPixel (target, 110, 110)) << endl;
	cout << check ("end effect stack test 2", 0x5555, Utils::getPixel (target, 0, 0)) << endl;
	cout << check ("end effect stack test 3", 0x5555, Utils::getPixel (target, 10, 10)) << endl; // Manager does not blit the blitting surface on its own!


	SDL_FreeSurface (blitting);
	blitting = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, blittingRect.w, blittingRect.h, 16, 0xf800, 0x7e0, 0x1f, 0x00);
	SDL_SetColorKey (blitting, SDL_SRCCOLORKEY, EFFECT_SURFACE_COLORKEY);
	SDL_FillRect (blitting, NULL, EFFECT_SURFACE_COLORKEY);
	Utils::setPixel (blitting, 0, 0, 0x9999);


	ImageProcessingSurfaceManager manager2 (target, Rect (0, 0, 640, 480));
	blittingRect.x = 320; blittingRect.y = 400; blittingRect.w = 30; blittingRect.h = 30;
	manager2.startEffectStack (blitting, blittingRect, identityRect.toSDL_Rect());
	cout << check ("is transparent test 1", false, manager2.compositePixelIsTransparent (320, 400)) << endl;
	cout << check ("is transparent test 2", true, manager2.compositePixelIsTransparent (320, 399)) << endl;
	cout << check ("is transparent test 3", true, manager2.compositePixelIsTransparent (319, 400)) << endl;
	cout << check ("is transparent test 4", true, manager2.compositePixelIsTransparent (321, 400)) << endl;
	cout << check ("is transparent test 5", true, manager2.compositePixelIsTransparent (320, 401)) << endl;



	SDL_FreeSurface (blitting);
	SDL_FreeSurface (target);

	return 0;
}
