#include "../inputHandler.h"
#include "../testUtils.h"
#include <iostream>
#include <list>
#include <string>
#include "../interface_inputHandlerObserver.h"

using namespace std;


class Observer : public Interface_InputHandlerObserver{
	public:
		Observer() : triggered_ (false) { }
		virtual ~Observer(){}

		virtual void inputHandlerUpdate (const KeyStatesSet& set){
			//cout << "input handler " << this << " updating" << endl;
			//cout << "space down? " << set.keyDown (SDLK_SPACE) << endl;
			triggered_ = true;
			lastSet_ = set;
			//cout << "space down now? " << lastSet_.keyDown (SDLK_SPACE) << endl;
			//cout << "space down now? " << getLastSet().keyDown (SDLK_SPACE) << endl;
		}		
		KeyStatesSet getLastSet() const{
			return lastSet_;
		}
		bool getTriggered() const{
			return triggered_;
		}
	private:
		bool triggered_;
		KeyStatesSet lastSet_;
};

int main (int argc, char** args){
	{
		InputHandler handler;
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		KeyStates states (keys);
		cout << check ("KeyStates test 1", true, states.getKeyDown (SDLK_SPACE)) << endl;
		KeyStates states2 (states);
		cout << check ("KeyStates test 2", true, states2.getKeyDown (SDLK_SPACE)) << endl;
		KeyStatesSet set;
		set.push (states);
		cout << check ("KeyStatesSet test 1", true, set.keyDown (SDLK_SPACE)) << endl;
		cout << check ("KeyStatesSet test 2", true, set.keyJustPressed (SDLK_SPACE)) << endl;
		set.push (states);
		cout << check ("KeyStatesSet test 3", true, set.keyDown (SDLK_SPACE)) << endl;
		cout << check ("KeyStatesSet test 4", false, set.keyJustPressed (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		handler.handleInput (keys);
		cout << check ("handle input test 1", false, observer1.getTriggered()) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		handler.setFocus (&observer1);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		handler.handleInput (keys);
		cout << check ("handle input test 2", true, observer1.getTriggered()) << endl;
		handler.returnFocus (&observer1);
		observer1 = Observer();
		handler.handleInput (keys);
		cout << check ("handle input test 3", false, observer1.getTriggered()) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		//cout << "observer1 at " << &observer1 << endl;
		handler.setFocus (&observer1);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		KeyStatesSet set;
		set.push (KeyStates (keys));
		cout << check ("observer test 1", true, set.keyDown (SDLK_SPACE)) << endl;
		observer1.inputHandlerUpdate (set);
		cout << check ("observer test 2", true, observer1.getLastSet().keyDown (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		Observer observer2;
		handler.setFocus (&observer2);
		handler.setFocus (&observer1);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		//cout << "observer1 at " << &observer1 << endl;
		//cout << "observer2 at " << &observer2 << endl;
		handler.handleInput (keys);
		//cout << "space down even now? " << observer1.getLastSet().keyDown (SDLK_SPACE) << endl;
		cout << check ("handle input test 5", true, observer1.getTriggered()) << endl;
		cout << check ("handle input test 6", true, observer1.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 7", true, observer2.getTriggered()) << endl;
		cout << check ("handle input test 8", false, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		Observer observer2;
		handler.setFocus (&observer2);
		handler.setFocus (&observer1);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		//cout << "observer1 at " << &observer1 << endl;
		//cout << "observer2 at " << &observer2 << endl;
		handler.handleInput (keys);
		//cout << "space down even now? " << observer1.getLastSet().keyDown (SDLK_SPACE) << endl;
		cout << check ("handle input test 9", false, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		handler.returnFocus (&observer1);
		handler.handleInput (keys);
		cout << check ("handle input test 10", true, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 11", false, observer2.getLastSet().keyJustPressed (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		Observer observer1;
		InputHandlerObserverState state1 (&observer1);
		state1.ignoreKey (SDLK_SPACE);
		cout << check ("observer state test 1", true, state1.isIgnoredKey (SDLK_SPACE)) << endl;
		state1.unignoreKey (SDLK_SPACE);
		cout << check ("observer state test 2", false, state1.isIgnoredKey (SDLK_SPACE)) << endl;
	}
	{
		InputHandler handler;
		Observer observer1;
		Observer observer2;
		handler.setFocus (&observer2);
		handler.setFocus (&observer1);
		handler.ignoreKey (&observer1, SDLK_SPACE);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		handler.handleInput (keys);
		cout << check ("handle input test 12", false, observer1.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 13", true, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		handler.unignoreKey (&observer1, SDLK_SPACE);
		handler.handleInput (keys);
		cout << check ("handle input test 14", true, observer1.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 15", false, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		Observer observer2;
		Observer observer3;
		handler.setFocus (&observer3);
		handler.setFocus (&observer2);
		handler.setFocus (&observer1);
		handler.ignoreKey (&observer2, SDLK_SPACE);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		handler.handleInput (keys);
		handler.returnFocus (&observer1);
		handler.handleInput (keys);
		cout << check ("handle input test 16", true, observer3.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 17", false, observer3.getLastSet().keyJustPressed (SDLK_SPACE)) << endl;
		cout << check ("handle input test 18", false, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	{
		InputHandler handler;
		Observer observer1;
		Observer observer2;
		handler.setFocus (&observer1);
		Uint8* keys = (Uint8*) malloc (sizeof (Uint8) * (SDLK_LAST + 1)); memset (keys, 0, sizeof (Uint8) * (SDLK_LAST + 1));
		keys [SDLK_SPACE] = SDL_PRESSED;
		handler.handleInput (keys);
		handler.setFocus (&observer2);
		handler.handleInput (keys);
		cout << check ("handle input test 19", true, observer2.getLastSet().keyDown (SDLK_SPACE)) << endl;
		cout << check ("handle input test 20", false, observer2.getLastSet().keyJustPressed (SDLK_SPACE)) << endl;
		free (keys); keys = NULL;
	}
	// Trying to uncover memory error discovered by valgrind
	{
		InputHandler handler;
		handler.handleInput();
	}

	return 0;
}
