#include "../microTileField.h"
#include "../rect.h"
#include <iostream>
#include <list>
#include <string>

using namespace std;


string check (string testName, string expected, string got){
	if (expected == got) return "Passed " + testName + ".";
	else return "Failed " + testName + ". Expected '" + expected + "', got '" + got + "'";
}
string check (string testName, int expected, int got){
	char expect [100];
	char get [100];
	sprintf (expect, "%d", expected);
	sprintf (get, "%d", got);
	return check (testName, expect, get);
}
string check (string testName, bool expected, bool got){
	string expect = expected ? "true" : "false";
	string get = got ? "true" : "false";
	return check (testName, expect, get);
}

string rectVectorToString (vector<Rect*> what){
	string retString = "[";

	list<Rect> toSort;
	for (vector<Rect*>::iterator iter = what.begin(); iter != what.end(); iter++){
		toSort.push_back (**iter);
	}
	toSort.sort();
	for (list<Rect>::iterator iter = toSort.begin(); iter != toSort.end(); iter++){
		if (iter != toSort.begin()) retString += ", ";
		retString += iter->toString();
	}
	retString += "]";

	return retString;
}
string rectVectorToString (vector<Rect> what){
	string retString = "[";

	list<Rect> toSort;
	for (vector<Rect>::iterator iter = what.begin(); iter != what.end(); iter++){
		toSort.push_back (*iter);
	}
	toSort.sort();
	for (list<Rect>::iterator iter = toSort.begin(); iter != toSort.end(); iter++){
		if (iter != toSort.begin()) retString += ", ";
		retString += iter->toString();
	}
	retString += "]";

	return retString;
}

int main (int argc, char** args){
	MicroTileField tileField (640, 480, 20, 15);
	int unitWidth = 640 / 20;
	int unitHeight = 480 / 15;

	vector<Rect> makeDirty;
	vector<Rect*> rects;
	vector<Rect> expected;

	cout << tileField.test();

	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("clear test 1", 0, rects.size()) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, 0, 1));
	expected.clear(); 
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 0.25", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, 1, 0));
	expected.clear(); 
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 0.5", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, 0, 0));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 1", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (-1, 0, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 2", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, -1, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 3", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (640, 0, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 4", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 480, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 5", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (-1, -1, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 6", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (640, 480, 1, 1));
	expected.clear();
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 7", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (0, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 8", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (unitWidth, unitHeight, 1, 1));
	expected.clear(); expected.push_back (Rect (unitWidth, unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 8.5", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (unitWidth - 1, unitHeight - 1, unitWidth, unitHeight));
	expected.clear();
		expected.push_back (Rect (0, 0, unitWidth * 2, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 9", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (1, 0, unitWidth, unitHeight));
	expected.clear(); 
		expected.push_back (Rect (0, 0, unitWidth * 2, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 10", rectVectorToString (expected), rectVectorToString (rects)) << endl;	

	makeDirty.clear(); makeDirty.push_back (Rect (0, 1, unitWidth, unitHeight));
	expected.clear(); 
		expected.push_back (Rect (0, 0, unitWidth, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 11", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, 2 * unitWidth, 2 * unitHeight));
	expected.clear(); 
		expected.push_back (Rect (0, 0, unitWidth * 2, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 12", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, 0, 3 * unitWidth, 3 * unitHeight));
	expected.clear(); 
		expected.push_back (Rect (0, 0, unitWidth * 3, unitHeight * 3));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 13", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); 
		makeDirty.push_back (Rect (0, 0, 1, 1));
		makeDirty.push_back (Rect (2 * unitWidth, 0, 1, 1));
		makeDirty.push_back (Rect (unitWidth, unitHeight, 1, 1));
		makeDirty.push_back (Rect (0, 2 * unitHeight, 1, 1));
		makeDirty.push_back (Rect (2 * unitWidth, 2 * unitHeight, 1, 1));
	expected.clear();
		expected.push_back (Rect (0, 0, unitWidth, unitHeight));
		expected.push_back (Rect (2 * unitWidth, 0, unitWidth, unitHeight));
		expected.push_back (Rect (unitWidth, unitHeight, unitWidth, unitHeight));
		expected.push_back (Rect (0, 2 * unitHeight, unitWidth, unitHeight));
		expected.push_back (Rect (2 * unitWidth, 2 * unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 14", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear();
		// 1 0 1
		// 0 1 0
		// 1 0 1 
		makeDirty.push_back (Rect (0, 0, 1, 1));
		makeDirty.push_back (Rect (2 * unitWidth, 0, 1, 1));
		makeDirty.push_back (Rect (unitWidth, unitHeight, 1, 1));
		makeDirty.push_back (Rect (0, 2 * unitHeight, 1, 1));
		makeDirty.push_back (Rect (2 * unitWidth, 2 * unitHeight, 1, 1));
	expected.clear();
		expected.push_back (Rect (0, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 1, 1), rects);
	cout << check ("mark test 15", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	expected.clear();
		expected.push_back (Rect (2 * unitWidth, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (2 * unitWidth, 0, 1, 1), rects);
	cout << check ("mark test 16", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	expected.clear();
		expected.push_back (Rect (unitWidth, unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (unitWidth, unitHeight, 1, 1), rects);
	cout << check ("mark test 17", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	expected.clear();
		expected.push_back (Rect (0, 2 * unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 2 * unitHeight, 1, 1), rects);
	cout << check ("mark test 18", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	expected.clear();
		expected.push_back (Rect (2 * unitWidth, 2 * unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (2 * unitWidth, 2 * unitHeight, 2, 2), rects);
	cout << check ("mark test 18.5", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	expected.clear();
		expected.push_back (Rect (2 * unitWidth, 2 * unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (2 * unitWidth, 2 * unitHeight, 1, 1), rects);
	cout << check ("mark test 19", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (-1, 0, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (0, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 20", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (0, -1, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (0, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 21", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (-1, -1, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (0, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 22", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (639, 0, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (640 - unitWidth, 0, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 23", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (-1, 479, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (0, 480 - unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 24", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (639, 479, unitWidth, unitHeight));
	expected.clear(); expected.push_back (Rect (640 - unitWidth, 480 - unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 25", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (unitWidth, unitHeight, 2 * unitWidth, unitHeight));
	expected.clear(); 
		expected.push_back (Rect (unitWidth, unitHeight, unitWidth * 2, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (2 * unitWidth - 1, unitHeight, 2, 2), rects);
	cout << check ("mark test 26", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); makeDirty.push_back (Rect (unitWidth, unitHeight, unitWidth, 2 * unitHeight));
	expected.clear(); 
		expected.push_back (Rect (unitWidth, unitHeight, unitWidth, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (unitWidth, 2 * unitHeight - 1, 2, 2), rects);
	cout << check ("mark test 27", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); 
		makeDirty.push_back (Rect (243, 50, 30, 30));
		makeDirty.push_back (Rect (245, 50, 30, 30));
		makeDirty.push_back (Rect (273, 50, 2, 30));
		makeDirty.push_back (Rect (273, 50, 2, 30));
		makeDirty.push_back (Rect (243, 50, 2, 30));
	expected.clear(); 
		expected.push_back (Rect (7 * unitWidth, unitHeight, unitWidth * 2, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 28", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear(); 
		makeDirty.push_back (Rect (273, 50, 2, 30));
		makeDirty.push_back (Rect (243, 50, 2, 30));
	expected.clear(); 
		expected.push_back (Rect (7 * unitWidth, unitHeight, unitWidth * 2, unitHeight * 2));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 29", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear();
		makeDirty.push_back (Rect (unitWidth * 6, unitHeight, 30, 30));
		makeDirty.push_back (Rect (unitWidth * 8, unitHeight, 30, 30));
	expected.clear(); 
		expected.push_back (Rect (6 * unitWidth, unitHeight, unitWidth, unitHeight));
		expected.push_back (Rect (8 * unitWidth, unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 30", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	makeDirty.clear();
		makeDirty.push_back (Rect (unitWidth * 6, unitHeight, 30, 30));
		makeDirty.push_back (Rect (unitWidth * 8, unitHeight, 30, 30));
	expected.clear(); 
		expected.push_back (Rect (6 * unitWidth, unitHeight, unitWidth, unitHeight));
		expected.push_back (Rect (8 * unitWidth, unitHeight, unitWidth, unitHeight));
	tileField.reset(); tileField.dirty (makeDirty);
	rects.clear(); tileField.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 31", rectVectorToString (expected), rectVectorToString (rects)) << endl;
	tileField.dirty (makeDirty);
	cout << check ("mark test 31.5", rectVectorToString (expected), rectVectorToString (rects)) << endl;

	MicroTileField tileField2 (640, 480, 32, 24);
	unitWidth = 640 / 32;
	unitHeight = 480 / 24;

	makeDirty.clear(); 
		makeDirty.push_back (Rect (640 - unitWidth, 480 - unitHeight, unitWidth, unitHeight));
	expected.clear(); 
		expected.push_back (Rect (640 - unitWidth, 480 - unitHeight, unitWidth, unitHeight));
	tileField2.reset(); tileField2.dirty (makeDirty);
	rects.clear(); tileField2.getConsolidatedRects (Rect (0, 0, 640, 480), rects);
	cout << check ("mark test 50", rectVectorToString (expected), rectVectorToString (rects)) << endl;


	return 0;
}
