#include "../rectQuadTree.h"
#include <iostream>
#include <string>
#include <list>
#include "../rect.h"
using namespace std;


vector<Rect*> allocatedRects;
void cleanup(){
	for (vector<Rect*>::iterator iter = allocatedRects.begin(); iter != allocatedRects.end(); iter++){
		delete *iter;
	}
	allocatedRects.clear();
}

string showSortedList (vector<Rect*> given){
	//cout << "showSortedList: given has " << given.size() << endl;
	list<Rect*> show;
	for (vector<Rect*>::iterator iter = given.begin(); iter != given.end(); iter++){
		show.push_back (*iter);
	}
	show.sort();
	string retString = "[";
	for (list<Rect*>::iterator iter = show.begin(); iter != show.end(); iter++){
		if (iter != show.begin()) retString += ", ";
		retString += (*iter)->toString();
	}
	retString += "]";
	return retString;
}

void checkList (const string& testName, const vector<Rect*>& expected, const vector<Rect*>& got){
	//cout << "checkList: expecting has " << expected.size() << ", got has " << got.size() << endl;
	if (showSortedList (expected) == showSortedList (got)) cout << "Passed " << testName << endl;
	else cout << "Failed " << testName << ". Expected: " << showSortedList (expected) << ", got: " << showSortedList (got) << endl;
}

void ins (Rect* item, RectQuadTree& tree, vector<Rect*>& runningList){
	runningList.push_back (item);
	tree.insert (item);
}

Rect* newRect (float x, float y, float width, float height){
	Rect* newest = new Rect (x, y, width, height);
	allocatedRects.push_back (newest);
	return newest;
}

int main (int numArgs, char** args){
	vector<Rect*> inserted;
	vector<Rect*> junk;
	vector<Rect*> getList;

	RectQuadTree memUseTester;
	ins (newRect (0, 0, 10, 10), memUseTester, inserted);
	ins (newRect (0, 0, 10, 10), memUseTester, inserted);
	memUseTester.clear();
	
	inserted.clear();

	RectQuadTree tester;

	getList.clear();
	ins (newRect (0, 0, 1, 1), tester, inserted);
	tester.getItems (&Rect (0, 0, 1, 1), getList);
	checkList ("insert test 1", inserted, getList);

	ins (newRect (10, 10, 1, 1), tester, inserted);
	getList.clear();
	tester.getItems (&Rect (0, 0, 11, 11), getList);
	checkList ("insert test 2", inserted, getList);
	junk.clear();
	getList.clear();
	tester.getItems (&Rect (20, 20, 10, 10), getList);
	checkList ("insert test 3", junk, getList);
	junk.push_back (inserted [0]);
	getList.clear();
	tester.getItems (&Rect (0, 0, 5, 5), getList);
	checkList ("insert test 4", junk, getList);
	junk.clear(); junk.push_back (inserted [1]);
	getList.clear();
	tester.getItems (&Rect (8, 8, 5, 5), getList);
	checkList ("insert test 5", junk, getList);

	ins (newRect (20, 20, 1, 1), tester, inserted);
	getList.clear();
	tester.getItems (&Rect (0, 0, 21, 21), getList);
	checkList ("insert test 6", inserted, getList);
	junk.clear(); junk.push_back (inserted [0]);
	getList.clear();
	tester.getItems (&Rect (0, 0, 1, 1), getList);
	checkList ("insert test 7", junk, getList);
	junk.clear(); junk.push_back (inserted [1]);
	getList.clear();
	tester.getItems (&Rect (10, 10, 1, 1), getList);
	checkList ("insert test 8", junk, getList);
	junk.clear(); junk.push_back (inserted [2]);
	getList.clear();
	tester.getItems (&Rect (20, 20, 1, 1), getList);
	checkList ("insert test 9", junk, getList);

	inserted.clear();
	tester.clear();
	ins (newRect (0, 0, 100, 100), tester, inserted);
	getList.clear();
	tester.getItems (&Rect (45, 45, 1, 1), getList);
	checkList ("insert test 10", inserted, getList);
	tester.insert (newRect (0, 0, 1, 1));
	getList.clear();
	tester.getItems (&Rect (0, 0, 100, 100), getList);
	checkList ("insert test 11", inserted, getList);
	tester.insert (newRect (99, 99, 1, 1));
	getList.clear();
	tester.getItems (&Rect (0, 0, 100, 100), getList);
	checkList ("insert test 12", inserted, getList);
	tester.insert (newRect (99, 0, 1, 1));
	getList.clear();
	tester.getItems (&Rect (0, 0, 100, 100), getList);
	checkList ("insert test 13", inserted, getList);
	tester.insert (newRect (0, 99, 1, 1));
	getList.clear();
	tester.getItems (&Rect (0, 0, 100, 100), getList);
	checkList ("insert test 14", inserted, getList);

	getList.clear();
	tester.clear();
	ins (newRect (0, 0, 32, 32), tester, inserted);
	ins (newRect (64, 0, 32, 32), tester, inserted);
	ins (newRect (32, 32, 32, 32), tester, inserted);
	ins (newRect (0, 64, 32, 32), tester, inserted);
	inserted.clear();	
	ins (newRect (64, 64, 32, 32), tester, inserted);
	tester.getItems (&Rect (64, 64, 1, 1), getList);
	checkList ("insert test 15", inserted, getList);

	cout << "Testing insert of 1000 items. . . " << endl;
	for (int j = 0; j < 1000; j++){
		int x = int (rand() % 1000) - 500;
		int y = int (rand() % 1000) - 500;
		int width = int (rand() % 500);
		int height = int (rand() % 500);
		Rect* newOne = newRect (x, y, width, height);
		cout << "New rect is " << newOne->toString() << endl;
		tester.insert (newOne);
	}
	cout << "Finished insert of 1000 items. . . " << endl;

	cleanup();

}
