#include "../segmentContainer.h"
#include <iostream>
#include <vector>
#include "../testUtils.h"
#include <math.h>
#include <algorithm>
using namespace std;


int randint (const unsigned int& low, const unsigned int& high){
	return (rand() % ((high - low) + 1)) + low;
}

int main (int numArgs, char** args){

	{
		SegmentContainer container (Rect (0, 0, 100, 100));
		Line line1 (Point (4, 5), Point (5, 5));
		container.insert (line1);
		std::vector<Line> target; target.push_back (line1);
		std::vector<Line> got; 
		container.getSegmentsInRadius (got, Point (4, 4), 1);
		cout << check ("get segments in radius test 1.0", int (target.size()), int (got.size())) << endl;
		got.clear(); container.getSegmentsInRadius (got, Point (4, 4), 1);
		cout << check ("get segments in radius test 1.1", int (target.size()), int (got.size())) << endl;
	}
	{
		SegmentContainer container (Rect (0, 0, 100, 100));
		std::vector<Line> target; 
		Line line1 (Point (4, 5), Point (5, 5));
		container.insert (line1);
		target.push_back (line1);
		Line line2 (Point (10, 5), Point (5, 5));
		container.insert (line2);
		target.push_back (line2);
		Line line3 (Point (20, 5), Point (5, 5));
		container.insert (line3);
		target.push_back (line3);
		std::vector<Line> got;
		container.getAllSegments (got);
		bool found = false;
		for (std::vector<Line>::iterator iter = got.begin(); iter != got.end(); iter++){
			found = false;
			for (std::vector<Line>::iterator targ = target.begin(); targ != target.end() && !found; targ++){
				if (iter->equals (*targ)) found = true;
			}
			if (!found) break;
		}
		cout << check ("get all segments test 1", true, found) << endl;
	}

	int seed = time (NULL);
	if (numArgs > 1){
		seed = atoi (args [1]);
	}
	cout << "Random seed: " << seed << endl;
	srand (seed);
	
	SegmentContainer container (Rect (0, 0, 1000, 1000), 5);
	std::vector<Line> inserted;
	int segCount = 500;
	for (int x = 0; x < segCount; x++){
		Line line (Point (randint (0, 1000), randint (0, 1000)), Point (randint (0, 1000), randint (0, 1000)));
    //for line in [makeLine (421,461,273,231), makeLine (896,845,666,931), makeLine (36,352,81,195), makeLine (531,551,602,649), makeLine (381,599,889,357)]:
		container.insert (line);
		inserted.push_back (line);
        //if ((x - 9) % 10 == 0):
        //   print str (x) + ' inserted so far'
	}

	cout << check ("insert test 1", (int) inserted.size(), (int) container.size()) << endl;
	//cout << check ("insert test 2", (int) inserted.size(), (int) container.absoluteCount()) << endl;

	vector<SegmentIntersection> containerIntersects;

	int randX = randint (0, 900);
	int randY = randint (0, 900);
	Line line (Point (randX, randY), Point (min (999, randX + 1000), min (999, randY + 1000)));
	vector<Line> linearFound;
	for (unsigned int x = 0; x < inserted.size(); x++){
		if (line.intersectsLine (inserted [x])){
			linearFound.push_back (inserted [x]);
		}
	}
	container.getIntersects (containerIntersects, line);
	vector<Line> containerFound;
	for (unsigned int x = 0; x < containerIntersects.size(); x++){
		if (line.intersectsLine (containerIntersects [x].getLine())){
			containerFound.push_back (containerIntersects [x].getLine());
		}
	}

	cout << check ("get intersects test 1", (int) linearFound.size(), (int) containerFound.size()) << endl;
	//cout << "container found " << containerFound.size() << " intersections." << endl;

	bool matched (true);
	for (vector<Line>::const_iterator iter = linearFound.begin(); iter != linearFound.end(); iter++){
		matched = false;
		for (vector<Line>::const_iterator contIter = containerFound.begin(); contIter != containerFound.end(); contIter++){
			if (iter->equals (*contIter)){
				matched = true;
				break;
			}
		}
		if (!matched) break;
	}
	cout << check ("get intersects test 2", true, matched) << endl;

	if (!matched){
		cout << "INSERTED" << endl;
		for (vector<Line>::const_iterator iter = inserted.begin(); iter != inserted.end(); iter++){
			cout << iter->toString() << endl;
		}
		cout << "LINEAR FOUND" << endl;
		for (vector<Line>::const_iterator iter = linearFound.begin(); iter != linearFound.end(); iter++){
			cout << iter->toString() << endl;			
		}
		cout << "CONTAINER FOUND" << endl;
		for (vector<Line>::const_iterator iter = containerFound.begin(); iter != containerFound.end(); iter++){
			cout << iter->toString() << endl;
		}
		cout << "INTERSECTOR" << endl;
		cout << line.toString() << endl;
	}

	matched = true;
	for (int radius = 0; radius <= 1000 && matched; radius += 10){
		vector<Line> nearbyCollection;
		int x = randint (0, 1000);
		int y = randint (0, 1000);
		Point nearPoint (x, y);
		container.getSegmentsInRadius (nearbyCollection, nearPoint, radius);
		vector<Line> nearbyLinear;
		for (vector<Line>::const_iterator iter = inserted.begin(); iter != inserted.end(); iter++){
			if (iter->distanceToPoint (nearPoint) <= radius) nearbyLinear.push_back (*iter);
		}
		//cout << "nearby liner has " << nearbyLinear.size() << endl;
		//cout << "nearby collection has " << nearbyCollection.size() << endl;
		//cout << check ("segments in radius test 1", int (nearbyLinear.size()), int (nearbyCollection.size())) << endl;
		for (vector<Line>::const_iterator iter = nearbyLinear.begin(); iter != nearbyLinear.end(); iter++){
			matched = false;
			for (vector<Line>::const_iterator collIter = nearbyCollection.begin(); collIter != nearbyCollection.end(); collIter++){
				if (iter->equals (*collIter)){
					matched = true;
					break;
				}
			}
			if (!matched){ 
				cout << "point now " << nearPoint.toString() << ", radius " << radius << endl;
				cout << "Couldn't find " << iter->toString() << endl;
			}
		}
	}
	cout << check ("segments in radius test 1", true, matched) << endl;

	return 0;
}
