#include "../surface.h"
#include "../testUtils.h"
#include "../utils.h"
#include <SDL/SDL.h>
#include <math.h>
#include <iostream>

#include "../screen.h"
#include "../resourcePool.h"
using namespace std;


bool checkBoxDraw (Screen* screen, Surface* box, unsigned int updateValue = 0){
	bool success = true;
	screen->update (100);
	screen->draw();
	double width = box->getRect()->getWidth();
	double height = box->getRect()->getHeight();
	double locX = box->getX();
	double locY = box->getY();
	for (int y = 0; y < int (height); y++){
		for (int x = 0; x < int (width); x++){
			if (success && (Utils::getPixel (screen->getSurface(), int (x + locX), int (y + locY)) != Utils::getPixel (box->getSurface(), x, y))){
				cout << "Draw test first failed at " << x << "," << y << endl;
				success = false;
			}
		}
	}
	return success;
}

void showScreen (Screen* screen){
	bool ignoreKeys = true;
	int last = SDL_GetTicks();
	bool wait = false;
	while (!(SDL_GetKeyState (NULL) [SDLK_SPACE]) || ignoreKeys){
		screen->update (SDL_GetTicks() - last);
		screen->draw();
		SDL_PumpEvents();
		last = SDL_GetTicks();
		if (!SDL_GetKeyState (NULL) [SDLK_SPACE]) ignoreKeys = false;
		if (!wait) break;
	}
}

int main (int numArgs, char** args){

	if (SDL_Init (SDL_INIT_VIDEO)< 0){
		cout <<"Could not initialize SDL:" << SDL_GetError() << endl;
		SDL_Quit();
	}

	{	
		SDL_Surface* testerSurface1 = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, 10, 10, 16, 0xf800, 0x7e0, 0x1f, 0x00);
		SDL_Surface* testerSurface2 = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, 10, 10, 16, 0xf800, 0x7e0, 0x1f, 0x00);

		SDL_SetColorKey (testerSurface1, SDL_SRCCOLORKEY, 0x00);
		SDL_SetColorKey (testerSurface2, SDL_SRCCOLORKEY, 0x00);
		SDL_FillRect (testerSurface1, NULL, 0x00);
		SDL_FillRect (testerSurface2, NULL, 0x00);

		Surface* tester1 = new Surface (NULL, testerSurface1);
		tester1->setXY (0, 0); tester1->getRect()->setWidth (10); tester1->getRect()->setHeight (10);
		Surface* tester2 = new Surface (NULL, testerSurface2);
		tester2->setXY (10, 0); tester2->getRect()->setWidth (10); tester2->getRect()->setHeight (10);

		Utils::setPixel (testerSurface1, 0, 0, 0xffff);
		Utils::setPixel (testerSurface2, 0, 0, 0xffff);
		cout << check ("creation test 1", false, tester1 == NULL) << endl;
		cout << check ("collision test 1", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 2", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1)) << endl;
		cout << check ("collision test 3", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth())) << endl;
		cout << check ("collision test 4", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1)) << endl;
		tester1->setX (0.25);
		cout << check ("collision test 5", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 6", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1)) << endl;
		cout << check ("collision test 7", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth())) << endl;
		cout << check ("collision test 8", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1)) << endl;
		tester1->setX (0.75);
		cout << check ("collision test 9", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 10", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1)) << endl;
		cout << check ("collision test 11", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth())) << endl;
		cout << check ("collision test 12", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1)) << endl;
		tester1->setX (0.25);
		cout << check ("collision test 13", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 14", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 0.9)) << endl;
		cout << check ("collision test 15", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth())) << endl;
		cout << check ("collision test 16", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 0.9)) << endl;
		tester1->setX (0);
		cout << check ("collision test 17", false, tester1->collidesWith (tester2, .9)) << endl;
		cout << check ("collision test 18", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1 + 0.9)) << endl;
		cout << check ("collision test 19", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 0.9)) << endl;
		cout << check ("collision test 20", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1 + 0.9)) << endl;
		tester1->setX (1);
		cout << check ("collision test 21", false, tester1->collidesWith (tester2, -.9)) << endl;
		cout << check ("collision test 22", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1 - 0.9)) << endl;
		cout << check ("collision test 23", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 0.9)) << endl;
		cout << check ("collision test 24", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1 - 0.9)) << endl;
		tester2->setX (tester2->getX() + 0.1);
		cout << check ("collision test 25", false, tester1->collidesWith (tester2, -.9)) << endl;
		cout << check ("collision test 26", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 1 - 0.9)) << endl;
		cout << check ("collision test 27", true, tester1->collidesWith (tester2, tester1->getRect()->getWidth() - 0.9)) << endl;
		cout << check ("collision test 28", false, tester1->collidesWith (tester2, tester1->getRect()->getWidth() + 1 - 0.9)) << endl;
		tester1->setXY (0, 0);
		tester2->setXY (0, 10);

		for (int y = 0; y < tester1->getRect()->getHeight(); y++){
			for (int x = 0; x < tester1->getRect()->getWidth(); x++){
				Utils::setPixel (testerSurface1, x, y, 0xffff);
			}
		}
		for (int y = 0; y < tester2->getRect()->getHeight(); y++){
			for (int x = 0; x < tester2->getRect()->getWidth(); x++){
				Utils::setPixel (testerSurface2, x, y, 0xffff);
			}
		}

		cout << check ("collision test 29", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 30", true, tester1->collidesWith (tester2, 0, 1)) << endl;
		cout << check ("collision test 31", false, tester2->collidesWith (tester1)) << endl;
		cout << check ("collision test 32", true, tester2->collidesWith (tester1, 0, -1)) << endl;
		tester1->setXY (0, 10);
		tester2->setXY (0, 0);
		cout << check ("collision test 33", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 34", true, tester1->collidesWith (tester2, 0, -1)) << endl;
		cout << check ("collision test 35", false, tester2->collidesWith (tester1)) << endl;
		cout << check ("collision test 36", true, tester2->collidesWith (tester1, 0, 1)) << endl;
		tester1->setXY (0, 0);
		tester2->setXY (10, 0);
		cout << check ("collision test 37", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 38", true, tester1->collidesWith (tester2, 1, 0)) << endl;
		cout << check ("collision test 39", false, tester2->collidesWith (tester1)) << endl;
		cout << check ("collision test 40", true, tester2->collidesWith (tester1, -1, 0)) << endl;
		tester1->setXY (10, 0);
		tester2->setXY (0, 0);
		cout << check ("collision test 41", false, tester2->collidesWith (tester1)) << endl;
		cout << check ("collision test 42", true, tester2->collidesWith (tester1, 1, 0)) << endl;
		cout << check ("collision test 43", false, tester1->collidesWith (tester2)) << endl;
		cout << check ("collision test 44", true, tester1->collidesWith (tester2, -1, 0)) << endl;

		delete tester1;
		delete tester2;

		SDL_FreeSurface (testerSurface1);
		SDL_FreeSurface (testerSurface2);
	}
	
	{
		SDL_Surface* testerSurface1 = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, 10, 10, 16, 0xf800, 0x7e0, 0x1f, 0x00);
		SDL_Surface* testerSurface2 = SDL_CreateRGBSurface (SDL_SWSURFACE | SDL_SRCCOLORKEY, 10, 10, 16, 0xf800, 0x7e0, 0x1f, 0x00);

		SDL_SetColorKey (testerSurface1, SDL_SRCCOLORKEY, 0x00);
		SDL_SetColorKey (testerSurface2, SDL_SRCCOLORKEY, 0x00);
		SDL_FillRect (testerSurface1, NULL, 0x00);
		SDL_FillRect (testerSurface2, NULL, 0x00);

		Surface* tester1 = new Surface (NULL, testerSurface1);
		tester1->setXY (0, 0); tester1->getRect()->setWidth (10); tester1->getRect()->setHeight (10);
		Surface* tester2 = new Surface (NULL, testerSurface2);
		tester2->setXY (10, 0); tester2->getRect()->setWidth (10); tester2->getRect()->setHeight (10);

		Utils::setPixel (testerSurface1, 0, 0, 0xffff);
		Utils::setPixel (testerSurface2, 0, 0, 0xffff);

		tester1->setXY (123.45, 67.890);
		cout << check ("abs visual rect test 1", 123.45, tester1->getAbsoluteVisualRect().getX()) << endl;
		cout << check ("abs visual rect test 2", 67.890, tester1->getAbsoluteVisualRect().getY()) << endl;
		cout << check ("visual rect test 1", 123.45, tester1->getVisualRect().getX()) << endl;
		cout << check ("visual rect test 2", 67.890, tester1->getVisualRect().getY()) << endl;
		cout << check ("rect test 1", 123.45, tester1->getRect()->getX()) << endl;
		cout << check ("rect test 2", 67.890, tester1->getRect()->getY()) << endl;
		tester1->setXY (-123.45, -67.890);
		cout << check ("abs visual rect test 3", -123.45, tester1->getAbsoluteVisualRect().getX()) << endl;
		cout << check ("abs visual rect test 4", -67.890, tester1->getAbsoluteVisualRect().getY()) << endl;
		cout << check ("visual rect test 3", -123.45, tester1->getVisualRect().getX()) << endl;
		cout << check ("visual rect test 4", -67.890, tester1->getVisualRect().getY()) << endl;
		cout << check ("rect test 3", -123.45, tester1->getRect()->getX()) << endl;
		cout << check ("rect test 4", -67.890, tester1->getRect()->getY()) << endl;
		tester1->getRect()->setWidth (0); tester1->getRect()->setHeight (0);
		cout << check ("abs visual rect test 5", 0.0, tester1->getAbsoluteVisualRect().getWidth()) << endl;
		cout << check ("abs visual rect test 6", 0.0, tester1->getAbsoluteVisualRect().getHeight()) << endl;
		cout << check ("visual rect test 5", 0.0, tester1->getVisualRect().getWidth()) << endl;
		cout << check ("visual rect test 6", 0.0, tester1->getVisualRect().getHeight()) << endl;
		cout << check ("rect test 5", 0.0, tester1->getRect()->getWidth()) << endl;
		cout << check ("rect test 6", 0.0, tester1->getRect()->getHeight()) << endl;
		tester1->getRect()->setWidth (10.5); tester1->getRect()->setHeight (20.5);
		cout << check ("abs visual rect test 7", 10.5, tester1->getAbsoluteVisualRect().getWidth()) << endl;
		cout << check ("abs visual rect test 8", 20.5, tester1->getAbsoluteVisualRect().getHeight()) << endl;
		cout << check ("visual rect test 7", 10.5, tester1->getVisualRect().getWidth()) << endl;
		cout << check ("visual rect test 8", 20.5, tester1->getVisualRect().getHeight()) << endl;
		cout << check ("rect test 7", 10.5, tester1->getRect()->getWidth()) << endl;
		cout << check ("rect test 8", 20.5, tester1->getRect()->getHeight()) << endl;

		delete tester1;
		delete tester2;

		SDL_FreeSurface (testerSurface1);
		SDL_FreeSurface (testerSurface2);
	}

	{
		// Next, we'll test surface drawing.

		Screen* screen = new Screen();
		ResourcePool* resources = new ResourcePool();
		screen->setResourcePool (resources);

		Surface* background = new Surface (screen, "blueScreen.bmp", false);
		background->setZ (0);

		Surface* tester1 = new Surface (background, "testWhiteSquare.bmp", false);
		tester1->setZ (1);

		tester1->setXY (0, 0);
		cout << check ("draw test 1", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (0.25, 0.25);
		cout << check ("draw test 2", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (0.1, 0.1);
		cout << check ("draw test 3", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (0.75, 0.75);
		cout << check ("draw test 4", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (1.0, 1.0);
		cout << check ("draw test 5", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (1.25, 1.25);
		cout << check ("draw test 6", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);

		delete screen; screen = NULL;
		delete resources; resources = NULL;
	}

	{
		// Next, we'll test surface drawing.

		Screen* screen = new Screen();
		ResourcePool* resources = new ResourcePool();
		screen->setResourcePool (resources);

		Surface* background = new Surface (screen, "blueScreen.bmp", false);
		background->setZ (0);

		Surface* tester1 = new Surface (background, "testWhiteSquare.bmp", false);
		tester1->setZ (1);

		tester1->setXY (630, 0);
		cout << check ("draw test 7", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (630.25, 0);
		cout << check ("draw test 8", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (630.5, 0);
		cout << check ("draw test 9", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);
		tester1->setXY (630.75, 0);
		cout << check ("draw test 10", true, checkBoxDraw (screen, tester1)) << endl;
		showScreen (screen);

		delete screen; screen = NULL;
		delete resources; resources = NULL;
	}


	SDL_Quit();

	return 0;
}
