#include "../synchChallenger.h"
#include <iostream>
#include <string>

using namespace std;


string check (string testName, string expected, string got){
	if (expected == got) return "Passed " + testName + ".";
	else return "Failed " + testName + ". Expected '" + expected + "', got '" + got + "'";
}
string check (string testName, int expected, int got){
	char expect [100];
	char get [100];
	sprintf (expect, "%d", expected);
	sprintf (get, "%d", got);
	return check (testName, expect, get);
}
string check (string testName, bool expected, bool got){
	string expect = expected ? "true" : "false";
	string get = got ? "true" : "false";
	return check (testName, expect, get);
}

int main (int argc, char** args){
	SynchChallenger* challenger = new SynchChallenger ("test1.mus");

	
	cout << check ("select test 1", true, challenger->selectChallengeCollection (0)) << endl;
	cout << check ("select test 2", true, challenger->selectChallengeCollection (1)) << endl;
	cout << check ("select test 3", false, challenger->selectChallengeCollection (1000)) << endl;
	cout << check ("select test 4", true, challenger->selectChallengeCollection (0)) << endl;

	cout << check ("start test 1", true, challenger->start()) << endl;

	cout << check ("seek test 1", true, challenger->seek (0)) << endl;
	challenger->seek (1000);
	cout << check ("seek test 2", 1000, challenger->getCurrentLocation()) << endl;
	challenger->seek (0);
	cout << check ("seek test 3", 0, challenger->getCurrentLocation()) << endl;
	cout << check ("seek test 4", false, challenger->seek (1000000)) << endl;
	cout << check ("seek test 5", 0, challenger->getCurrentLocation()) << endl;

	cout << check ("reset test 1", true, challenger->reset()) << endl;
	challenger->seek (1000);
	challenger->reset();
	cout << check ("reset test 2", 0, challenger->getCurrentLocation()) << endl;

	cout << check ("buttons pressed test 1", false, challenger->buttonsPressed (1)) << endl;

	challenger->seek (1000);
	cout << check ("buttons pressed test 2", true, challenger->buttonsPressed (1)) << endl;
	challenger->seek (500);
	cout << check ("buttons pressed test 3", false, challenger->buttonsPressed (2)) << endl;

	challenger->selectChallengeCollection (1);
	cout << check ("select test 5", 0, challenger->getCurrentLocation()) << endl;
	challenger->seek (20000);
	challenger->selectChallengeCollection (0);
	cout << check ("select test 6", 500, challenger->getCurrentLocation()) << endl;
	challenger->selectChallengeCollection (1);
	cout << check ("select test 7", 20000, challenger->getCurrentLocation()) << endl;
	challenger->selectChallengeCollection (0);
	
	cout << check ("challenges test 1", 7, challenger->getChallenges().size()) << endl;

	challenger->seek (1099);
	cout << check ("buttons pressed test 4", true, challenger->buttonsPressed (1)) << endl;
	cout << check ("buttons pressed test 4.5", true, challenger->buttonsPressed (1)) << endl;
	cout << check ("buttons pressed test 5", false, challenger->buttonsPressed (0)) << endl;
	cout << check ("buttons pressed test 6", false, challenger->buttonsPressed (3)) << endl;

	cout << check ("start test 1", true, challenger->start()) << endl;

	cout << check ("update test 1", true, challenger->update (1)) << endl;
	cout << check ("update test 2", 1100, challenger->getCurrentLocation()) << endl;
	
	cout << check ("buttons pressed test 7", false, challenger->buttonsPressed (1)) << endl;
	cout << check ("buttons pressed test 8", false, challenger->buttonsPressed (2)) << endl;

	cout << check ("update test 3", true, challenger->update (1000)) << endl;
	challenger->pause();
	cout << check ("update test 3.5", false, challenger->update (1000)) << endl;
	cout << check ("update test 3.75", 2100, challenger->getCurrentLocation()) << endl;
	challenger->start();
	
	cout << check ("buttons pressed test 9", true, challenger->buttonsPressed (2)) << endl;

	cout << check ("seek test 6", true, challenger->seek (7000)) << endl;
	cout << check ("seek test 7", 7000, challenger->getCurrentLocation()) << endl;

	cout << check ("buttons pressed test 10", true, challenger->buttonsPressed (7)) << endl;

	challenger->buttonsPressed (0);
	cout << check ("streak test 1", 0, challenger->getCurrentStreak()) << endl;
	challenger->buttonsPressed (7);
	cout << check ("streak test 2", 1, challenger->getCurrentStreak()) << endl;
	challenger->seek (5000);
	challenger->buttonsPressed (5);
	cout << check ("streak test 3", 2, challenger->getCurrentStreak()) << endl;
	challenger->buttonsPressed (5);
	cout << check ("streak test 3.5", 2, challenger->getCurrentStreak()) << endl;
	challenger->buttonsPressed (0);
	cout << check ("streak test 4", 0, challenger->getCurrentStreak()) << endl;

	challenger->seek (7000);
	challenger->buttonsPressed (7);
	challenger->seek (5000);
	challenger->buttonsPressed (5);
	challenger->seek (7000);
	challenger->buttonsPressed (7);
	challenger->selectChallengeCollection (1);
	challenger->buttonsPressed (0);
	cout << check ("select test 5", 0, challenger->getCurrentStreak()) << endl;
	challenger->selectChallengeCollection (0);
	cout << check ("select test 6", 3, challenger->getCurrentStreak()) << endl;

	cout << check ("update test 4", false, challenger->update (10000)) << endl;
	cout << check ("update test 5", 7000, challenger->getCurrentLocation()) << endl;
	
	challenger->seek (3000);
	cout << check ("buttons pressed test 11", true, challenger->buttonsPressed (3)) << endl;
	challenger->seek (5000);
	cout << check ("buttons pressed test 12", true, challenger->buttonsPressed (5)) << endl;

	challenger->seek (3000);
	challenger->update (2000);
	cout << check ("update test 6", true, challenger->buttonsPressed (5)) << endl;

	SynchChallenger* challenger2 = dynamic_cast<SynchChallenger*> (challenger->clone());

	cout << check ("clone test 1", 5000, challenger2->getCurrentLocation()) << endl;
	cout << check ("clone test 2", true, challenger->getChallenges().size() == challenger2->getChallenges().size()) << endl;
	cout << check ("clone test 3", 5, challenger2->getCurrentStreak()) << endl;
	cout << check ("clone test 4", false, challenger == challenger2) << endl;
	challenger->buttonsPressed (0);
	cout << check ("clone test 5", true, challenger2->getCurrentStreak() == 5 && challenger->getCurrentStreak() == 0) << endl;
	

	delete challenger2;
	delete challenger;

	return 0;
}
