#include "../utils.h"
#include "../testUtils.h"
#include <iostream>
#include <list>
#include <string>

using namespace std;


int main (int argc, char** args){

	double lowest = 1000;
	double highest = -1000;
	for (int x = 0; x < 100000; x++){
		double result = Utils::fastPseudoRand (x);
		if (result < lowest) lowest = result;
		if (result > highest) highest = result;
	}

	cout << "Highest pseudo random: " << highest << endl;
	cout << "Lowest pseudo random: " << lowest << endl;

	cout << check ("fast pseudo rand test 1", true, highest > 0.9) << endl;
	cout << check ("fast pseudo rand test 2", true, highest <= 1.0) << endl;
	cout << check ("fast pseudo rand test 3", true, lowest < -0.9) << endl;
	cout << check ("fast pseudo rand test 4", true, lowest >= -1.0) << endl;

	cout << check ("space coord to screen coord test 1", 0.0, Utils::spaceCoordToScreenCoord (0, 0)) << endl;
	cout << check ("space coord to screen coord test 2", 1.0, Utils::spaceCoordToScreenCoord (1, 0)) << endl;
	cout << check ("space coord to screen coord test 3", 1.5, Utils::spaceCoordToScreenCoord (1.5, 0)) << endl;
	cout << check ("space coord to screen coord test 4", 1.0 / pow (2, 1), Utils::spaceCoordToScreenCoord (1, -1)) << endl;
	cout << check ("space coord to screen coord test 5", 1.0 / pow (2, 2), Utils::spaceCoordToScreenCoord (1, -2)) << endl;
	cout << check ("space coord to screen coord test 6", 1.5 / pow (2, 3), Utils::spaceCoordToScreenCoord (1.5, -3)) << endl;

	cout << check ("screen coord to space coord test 1", 0.0, Utils::screenCoordToSpaceCoord (0, 0)) << endl;
	cout << check ("screen coord to space coord test 2", 1.0, Utils::screenCoordToSpaceCoord (1, 0)) << endl;
	cout << check ("screen coord to space coord test 3", 1.5, Utils::screenCoordToSpaceCoord (1.5, 0)) << endl;
	cout << check ("screen coord to space coord test 4", 1.0, Utils::screenCoordToSpaceCoord (1.0 / pow (2, 1), -1)) << endl;
	cout << check ("screen coord to space coord test 5", 1.0, Utils::screenCoordToSpaceCoord (1.0 / pow (2, 2), -2)) << endl;
	cout << check ("screen coord to space coord test 6", 1.5, Utils::screenCoordToSpaceCoord (1.5 / pow (2, 3), -3)) << endl;

	Optional<int> opt1 (1);
	Optional<int> opt2;
	Optional<int> opt3 = opt2;
	cout << check ("optional test 1", false, opt2.isSet()) << endl;
	cout << check ("optional test 2", 1, opt1.get()) << endl;
	cout << check ("optional test 3", true, opt1.isSet()) << endl;
	opt2 = opt1;
	cout << check ("optional test 4", true, opt2.isSet()) << endl;
	cout << check ("optional test 5", 1, opt2.get()) << endl;
	cout << check ("optional test 6", false, opt3.isSet()) << endl;
	opt3 = opt2;
	cout << check ("optional test 7", true, opt3.isSet()) << endl;
	cout << check ("optional test 8", 1, opt3.get()) << endl;

	{
		Uint16 pixel1 = 0x0000;
		Uint16 pixel2 = 0x001f;

		Uint16 blended = Utils::linearPixelBlendRGB (pixel1, pixel2, 0.5);
		
		cout << check ("Uint16 linear pixel blend rgb test 1", 15, blended) << endl;
	}

	cout << check ("compareDoubles test 1", 0, Utils::compareDoubles (0, 0, 0.001)) << endl;
	cout << check ("compareDoubles test 2", 0, Utils::compareDoubles (0, 0.0001, 0.001)) << endl;
	cout << check ("compareDoubles test 2.5", 0, Utils::compareDoubles (0.0001, 0, 0.001)) << endl;
	cout << check ("compareDoubles test 3", 0, Utils::compareDoubles (0, -0.0001, 0.001)) << endl;
	cout << check ("compareDoubles test 3.5", 0, Utils::compareDoubles (-0.0001, 0, 0.001)) << endl;
	cout << check ("compareDoubles test 4", -1, Utils::compareDoubles (1, -1, 0.001)) << endl;
	cout << check ("compareDoubles test 5", 1, Utils::compareDoubles (-1, 1, 0.001)) << endl;


	return 0;
}
