#ifndef WAYPOINT_PLAN_TEST
#define WAYPOINT_PLAN_TEST


#include "../waypointPlan.h"
#include "../testUtils.h"
#include "../geometry.h"
#include <iostream>
#include <vector>
using namespace std;

class WaypointPlanTester{
	public:
		void runTests(){
			{
				std::vector<Point> plan;
				WaypointPlan tester (plan);
				cout << checkP ("get last waypoint test 1", Optional<Point>(), tester.getLastWaypoint()) << endl;
				cout << check ("get distance to next waypoint test 1", 0.0, tester.getDistanceToNextWaypoint()) << endl;
				cout << check ("finished path test 1", true, tester.finishedPath()) << endl;
				cout << checkP ("get current point test 1", Optional<Point>(), tester.getCurrentPoint()) << endl;
			}
			{
				std::vector<Point> plan;
				Optional<Point> testPoint (Point (0,0));
				plan.push_back (testPoint.get());
				WaypointPlan tester (plan);
				cout << checkP ("get last waypoint test 2", testPoint, tester.getLastWaypoint()) << endl;
				cout << check ("get distance to next waypoint test 2", 0.0, tester.getDistanceToNextWaypoint()) << endl;
				cout << check ("finished path test 2", true, tester.finishedPath()) << endl;
				cout << checkP ("get current point test 2", testPoint, tester.getCurrentPoint()) << endl;
				tester.update (1000);
				cout << checkP ("get last waypoint test 3", testPoint, tester.getLastWaypoint()) << endl;
				cout << checkP ("get current point test 3", testPoint, tester.getCurrentPoint()) << endl;
			}
			{
				std::vector<Point> plan;
				plan.push_back (Point (0,0));
				plan.push_back (Point (10,0));
				WaypointPlan tester (plan);
				cout << checkP ("get last waypoint test 4", Optional<Point> (Point (0,0)), tester.getLastWaypoint()) << endl;
				cout << check ("get distance to next waypoint test 3", 10.0, tester.getDistanceToNextWaypoint()) << endl;
				cout << check ("finished path test 3", false, tester.finishedPath()) << endl;
				tester.update (10);
				cout << checkP ("get last waypoint test 5", Optional<Point> (Point (10,0)), tester.getLastWaypoint()) << endl;
				cout << check ("get distance to next waypoint test 4", 0.0, tester.getDistanceToNextWaypoint()) << endl;
				cout << check ("finished path test 4", true, tester.finishedPath()) << endl;
				cout << checkP ("get current point test 4", Optional<Point> (Point (10,0)), tester.getCurrentPoint()) << endl;
				tester.update (10);
				cout << checkP ("get current point test 5", Optional<Point> (Point (10,0)), tester.getCurrentPoint()) << endl;
				cout << check ("get distance to next waypoint test 5", 0.0, tester.getDistanceToNextWaypoint()) << endl;
			}
			{
				std::vector<Point> plan;
				plan.push_back (Point (0,0));
				plan.push_back (Point (10,0));
				plan.push_back (Point (20,0));
				WaypointPlan tester (plan);
				cout << check ("get distance to next waypoint test 6", 10.0, tester.getDistanceToNextWaypoint()) << endl;
				tester.update (10);
				cout << checkP ("get current point test 6", Optional<Point> (Point (10,0)), tester.getCurrentPoint()) << endl;
				cout << check ("get distance to next waypoint test 7", 10.0, tester.getDistanceToNextWaypoint()) << endl;
				tester.update (5);
				cout << checkP ("get current point test 7", Optional<Point> (Point (15,0)), tester.getCurrentPoint()) << endl;
				cout << check ("get distance to next waypoint test 8", 5.0, tester.getDistanceToNextWaypoint()) << endl;
				tester.update (2);
				cout << checkP ("get current point test 8", Optional<Point> (Point (17,0)), tester.getCurrentPoint()) << endl;
				cout << check ("get distance to next waypoint test 9", 3.0, tester.getDistanceToNextWaypoint()) << endl;
			}
		}
	private:
		std::string checkP (const std::string& title, Optional<Point> expected, Optional<Point> got){
			if (expected == got){
				return "Passed " + title;
			}
			return "Failed " + title + ". Expected: " + (expected.isSet() ? expected.get().toString() : "NOTSET") + " Got: " + (got.isSet() ? got.get().toString() : "NOTSET");
		}
};

int main (int numArgs, char** args){
	WaypointPlanTester tester;
	tester.runTests();

	return 0;
}


#endif
