#include "text.h"
#include "letter.h"
#include "animationSet.h"
#include "surface.h"

using namespace std;


Text::Text (Surface* parent, const string& fontFile, const string& text) : Surface (parent){
	this->fontFile = fontFile;
	setText (text);
}
Text::~Text(){}

void Text::addVisualEffect (Effect* newEffect){
	vector<Surface*> childs = getChildren();
	for (vector<Surface*>::iterator kid = childs.begin(); kid != childs.end(); kid++){
		(*kid)->addVisualEffect (newEffect->clone());
	}
	Surface::addVisualEffect (newEffect);
}

void Text::setText (const string& text){
	if (this->text == text) return;
	this->text = text;
	const char* next = text.c_str();
	freeChildren();
	double currentX = 0;
	double currentY = 0;
	double tallest = 0;
	double maxX = 0;
	double maxY = 0;
	Letter* newLetter = NULL;
	
	while (*next){
		if ( *next == '\n' || (*next == '\\' && *(next + 1) == 'n') ){
			currentX = 0;
			currentY += tallest;
			tallest = 0;
			if (*next == '\\' && *(next + 1) == 'n') next++;
		}
		else{
			try{
				newLetter = new Letter (this, fontFile, *next);
				newLetter->setX (currentX);
				newLetter->setY (currentY);
				double height = newLetter->getHeight();
				double width = newLetter->getWidth();
				if (height > tallest){
					tallest = height;
				}
				if (currentX + width > maxX){
					maxX = currentX + width;
				}
				if (currentY + height > maxY){
					maxY = currentY + height;
				}
				currentX += newLetter->getWidth();
			}
			catch (AnimationSetPlayException){
				cout << "Couldn't make new letter '" << *next << "'. Ignoring." << endl;
				if (newLetter){
					removeChildSurface (newLetter);
					delete newLetter; newLetter = NULL;
				}
			}
		}

		next++;
	}
	getRect()->setWidth (getX() + maxX);
	getRect()->setHeight (getY() + maxY);
}
