#include <algorithm>
#include "transition.h"
#include "interface_transitionObserver.h"
#include <vector>
#include <iostream>

using namespace std;


Transition::Transition (const int& pTimeLengthMs, const double& pStartX, const double& pEndX)
		  : timeLengthMs (pTimeLengthMs), startX (pStartX), endX (pEndX), timePassedMs (0){
	//cout << "transition setting end x to " << endX << endl;
}

bool Transition::update (unsigned int timePassed){
	if (!getLive()) return false;

	if ((timeLengthMs != -1) || (timeLengthMs < (int) timePassedMs)) timePassedMs += timePassed;
	if (timeLengthMs > -1 && int (timePassedMs) >= timeLengthMs) notifyObserversOfCompletion();

	return true;
}

void Transition::notifyObserversOfCompletion(){
	for (vector<Interface_TransitionObserver*>::iterator iter = observers.begin(); iter != observers.end(); iter++){
		(*iter)->transitionComplete (this);
	}
}

bool Transition::addObserver (Interface_TransitionObserver* observer){
	vector<Interface_TransitionObserver*>::iterator iter = find (observers.begin(), observers.end(), observer);
	if (iter != observers.end()) return false;

	observers.push_back (observer);
	return true;
}

bool Transition::removeObserver (Interface_TransitionObserver* observer){
	vector<Interface_TransitionObserver*>::iterator iter = find (observers.begin(), observers.end(), observer);
	if (iter == observers.end()) return false;

	observers.erase (iter);
	return true;
}
