#include "transitionJump.h"
#include <iostream>
#include <math.h>
#include "utils.h"

using namespace std;


TransitionJump::TransitionJump (const double& pixelToMeterFactor, const double& pStartX, const double& pEndX, 
	const double& jumpMetersPerSecond, const double& gravityMetersPerSecond)
	: Transition (-1, pStartX, pEndX){
	//cout << "Creating Jump transition with args " << pStartX << "," << pEndX << endl;
	//cout << "TransitionJump::TransitionJump() TransitionJump created" << endl;
	pixelToMeterFactor_ = pixelToMeterFactor;
	gravityMetersPerSecond_ = gravityMetersPerSecond;
	jumpMetersPerSecond_ = jumpMetersPerSecond;
	velocityMetersPerSecond_ = -jumpMetersPerSecond;
	location_ = pStartX;
}

// Not a proper clone; would need to update() with time passed for that. As is, just makes a new Jump starting at origin.
TransitionJump* TransitionJump::clone() const{
	return new TransitionJump (pixelToMeterFactor_, getStartX(), getEndX(), jumpMetersPerSecond_, gravityMetersPerSecond_);
}

bool TransitionJump::update (unsigned int timePassed){
	//cout << "TransitionJump::update() TransitionJump updated" << endl;
	//cout << "pixelToMeterFactor is " << pixelToMeterFactor_ << endl;
	location_ += ((velocityMetersPerSecond_ * pixelToMeterFactor_) * (timePassed / 1000.0));
	//cout << "location now " << location_ << endl;
	velocityMetersPerSecond_ += (gravityMetersPerSecond_ * (timePassed / 1000.0));
	return Transition::update (timePassed);
}

double TransitionJump::getX() const{
	return (location_ < getEndX() ? location_ : getEndX());
}
