/***************************************************************************
 *   Copyright (C) 2008 by Nick Easler,,,   *
 *   easlern@easlern-laptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef UTILS_H
#define UTILS_H

#include <math.h>
#include <SDL/SDL.h>
#include "rect.h"
#include <iostream>
#include "geometry.h"
#include "audioData.h"
//#include <xmmintrin.h>
#include <list>
using namespace std;

/**
	@author Nick Easler,,, <easlern@easlern-laptop>
*/

#define INTERRUPT_NOBODY        0
#define INTERRUPT_SELF          1<<0
#define INTERRUPT_FRIENDS       1<<1
#define INTERRUPT_ENEMIES       1<<2
#define INTERRUPT_OTHERS        INTERRUPT_FRIENDS | INTERRUPT_ENEMIES
#define INTERRUPT_ANYONE        INTERRUPT_SELF | INTERRUPT_OTHERS
#define INTERRUPT_ENVIRONMENT   1<<3

#define DISPLAY_BUFFERS			1

#define EFFECT_NONE 							0
#define EFFECT_IMAGE_PROCESSOR 					1
#define EFFECT_LOCATION_PROCESSOR 				2
#define EFFECT_VIEWPORT_PROCESSOR 				4
#define EFFECT_OCCLUSIONPORT_PROCESSOR 			8
#define EFFECT_TRANSFORMATION_PROCESSOR			16

// TODO: ideally, the effect surface color key will depend on what colors are not being used in the surface. For now, it's static.
#define EFFECT_SURFACE_COLORKEY	0xbadd

#define CPU_ABILITY_SSE2		1


typedef int InterruptFlag;
typedef int AnimationType;

#define AnimationSetMap IntMapToPointerHashed<Animation*>


//typedef float v4sf __attribute__ ((vector_size(16))); // float vector for SSE

class Utils{
	public:
    	Utils();

	    ~Utils();

		// Taken from http://softpixel.com/~cwright/programming/simd/cpuid.php .
		static unsigned int getCpuAbilities(){
			return 0; // Removed inline assembly, no need to query CPU anymore.
			/*
			unsigned int func = 0x01;
			unsigned int a; 
			unsigned int b; 
			unsigned int c; 
			unsigned int d;
			__asm__ ("cpuid" : "=a" (a), "=b" (b), "=c" (c), "=d" (d) : "a" (func));

			unsigned int abilities = 0;
			if (d & (0x01 << 6)) abilities |= CPU_ABILITY_SSE2;

			return abilities;
			*/
		}

		inline static const double PI(){ return 3.141592; }
		inline static const double E(){ return 2.718281; }
		inline static const double SQRT_2(){ return 1.4142135623730951; }

		// Taken from a web page on Perlin noise. TODO: put link to the page here!
		static double fastPseudoRand (int x){
			x = (x<<13) ^ x;
			return ( 1.0 - ( (x * (x * x * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);
		}

    	// Domain (0..1) range (0..1)
		static double gaussianDistribution (const double& x){
		    // (e^(-x^2/2))/(sqrt(2pi))
    		double actual = x * 8 - 4;
		    return (pow (E(), (-1 * pow (actual, 2) / 2))) / (sqrt (2 * PI())) * 2.506;
		}
	    // Default: domain (0..1) range (0..1)
		static double interpolatedSin (const double& value, const unsigned int& period = 2){
			//return sin (PI() * 2 * (value % period) / (period * 1.0));
			return sin (PI() * 2 * value / period);
		}
		// domain (0..1) range (0..1)
		static double logisticFunction (const double& x){
			double actual = x * 10 - 5;
			// 1 / ( 1 + e^(-t) )
			return 1.0 / (1 + pow (E(), -actual));
		}
		/*
		static float fastFloatSqrt (float num){
		    v4sf floatVec;
		    //float nums [4];
		    float __attribute__ ((aligned(128))) nums [4];
		    //for (int x = 0; x < 16; x++) nums [x] = 0;
		    nums [0] = num;
		    // WTF where are the aligned versions (loadaps/storeaps)?
		    //floatVec = __builtin_ia32_loadups (nums);
			floatVec = _mm_load_ss (nums);
		    //floatVec = __builtin_ia32_rsqrtss (floatVec);
		    floatVec = __builtin_ia32_sqrtss (floatVec);
		    //__builtin_ia32_storeups (nums, floatVec);
			_mm_store_ss (nums, floatVec);
		    //return nums [0] * num;
			return nums [0];
		}
		*/
		static SDL_Surface* createSDL_SurfaceLike (const SDL_Surface* from){
			SDL_Surface* like = SDL_CreateRGBSurface (from->flags, from->w, from->h, from->format->BitsPerPixel,  
										 			  from->format->Rmask, from->format->Gmask,
            	        				 			  from->format->Bmask, from->format->Amask);
			SDL_SetColorKey (like, SDL_SRCCOLORKEY, from->format->colorkey);
			SDL_FillRect (like, NULL, like->format->colorkey);
			return like;
		}
    	static int round (const double& num){
        	// Cheap way to do it.  :P
	        int retVal = 0;
    	    if (num - int (num) > .44444444444444444444){
        	    retVal = int (num) + 1;
	        }
    	    else{
        	    retVal = int (num);
	        }
    	    return retVal;
	    }
		static double screenCoordToSpaceCoord (const double& coord, const double& z){
			if (z >= 0) return coord;
			return coord * (pow (2, fabs (z)));
		}
		static double spaceCoordToScreenCoord (const double& coord, const double& z){
			if (z >= 0) return coord;
			return coord / (pow (2, fabs (z)));
		}
		static Audio::AudioSample audioBlend (const Audio::AudioSample& sample1, const Audio::AudioSample& sample2){
			int left1 = sample1.getLeftSample();
			int left2 = sample2.getLeftSample();
			int right1 = sample1.getRightSample();
			int right2 = sample2.getRightSample();
			int combinedLeft = left1 + left2;
			int combinedRight = right1 + right2;
			int maxVal = 32765;
			if (combinedLeft < -maxVal) combinedLeft = -maxVal;
			if (combinedLeft > maxVal) combinedLeft = maxVal;
			if (combinedRight < -maxVal) combinedRight = -maxVal;
			if (combinedRight > maxVal) combinedRight = maxVal;
			
			return Audio::AudioSample (combinedLeft, combinedRight, sample1.getBitSize());
		}
		// Weight is on scale 0 to 1. 0 blends to a solid pixel1 and 1 blends to a solid pixel2. Values in between blend linearly.
		// Note: this just returns alpha channel of pixel1!
		static Uint32 linearPixelBlendRGBA (const Uint32& pixel1, const Uint32& pixel2, const double& weight){
			double recip = 1 - weight;
			unsigned int r = max (0, min (
					( int (((pixel1 & 0xff000000) >> 24) * recip) + int (((pixel2 & 0xff000000) >> 24) * weight) )
				, 255 ) );
			unsigned int g = max (0, min ( 
					( int (((pixel1 & 0x00ff0000) >> 16) * recip) + int (((pixel2 & 0x00ff0000) >> 16) * weight) )
				, 255 ) );
			unsigned int b = max (0, min (
					( int (((pixel1 & 0x0000ff00) >> 8) * recip) + int (((pixel2 & 0x0000ff00) >> 8) * weight) )
				, 255 ) );
		
			return Uint32 ((r << 24) | (g << 16) | (b << 8) | (pixel1 & 0x000000ff));
		}
		static Uint32 pixelAverageRGBA (const int& numPixels, const Uint32* pixels){
			unsigned int rMask = 0xff000000;
			unsigned int gMask = 0x00ff0000;
			unsigned int bMask = 0x0000ff00;
			unsigned int aMask = 0x000000ff;
	
			unsigned int rSum = 0; unsigned int gSum = 0; unsigned int bSum = 0; unsigned int aSum = 0;
			int index;
			for (index = 0; index < numPixels; index++){
				rSum += (pixels [index] & rMask) >> 24;
				gSum += (pixels [index] & gMask) >> 16;
				bSum += (pixels [index] & bMask) >> 8;
				aSum += (pixels [index] & aMask);
			}

			rSum = min ((unsigned int) 0xff, (rSum / numPixels) & 0xff);
			gSum = min ((unsigned int) 0xff, (gSum / numPixels) & 0xff);
			bSum = min ((unsigned int) 0xff, (bSum / numPixels) & 0xff);
			aSum = min ((unsigned int) 0xff, (aSum / numPixels) & 0xff);
			
			return (rSum << 24) | (gSum << 16) | (bSum << 8) | aSum;
		}
		static Uint32 linearPixelBlendRGB (const Uint32& pixel1, const Uint32& pixel2, const double& weight){
			double recip = 1 - weight;
			unsigned int r = max (0, min (
					( int (((pixel1 & 0xff0000) >> 16) * recip) + int (((pixel2 & 0xff0000) >> 16) * weight) )
				, 255 ) );
			unsigned int g = max (0, min (
					( int (((pixel1 & 0x00ff00) >> 8) * recip) + int (((pixel2 & 0x00ff00) >> 8) * weight) )
				, 255 ) );
			unsigned int b = max (0, min (
					( int (((pixel1 & 0x0000ff)) * recip) + int (((pixel2 & 0x0000ff)) * weight) )
				, 255 ) );
			
			return Uint32 ((r << 16) | (g << 8) | (b));
		}
		static Uint32 pixelAverageRGB (const int& numPixels, const Uint32* pixels){
			unsigned int rMask = 0xff0000;
			unsigned int gMask = 0x00ff00;
			unsigned int bMask = 0x0000ff;
	
			unsigned int rSum = 0; unsigned int gSum = 0; unsigned int bSum = 0;
			int index;
			for (index = 0; index < numPixels; index++){
				rSum += (pixels [index] & rMask) >> 16;
				gSum += (pixels [index] & gMask) >> 8;
				bSum += (pixels [index] & bMask);
			}

			rSum = min ((unsigned int) 0xff, (rSum / numPixels) & 0xff);
			gSum = min ((unsigned int) 0xff, (gSum / numPixels) & 0xff);
			bSum = min ((unsigned int) 0xff, (bSum / numPixels) & 0xff);
			
			return (rSum << 16) | (gSum << 8) | (bSum);
		}
		static Uint16 linearPixelBlendRGB (const Uint16& pixel1, const Uint16& pixel2, const double& weight){
			double recip = 1 - weight;
			unsigned int r = max (0, min (
					( int (((pixel1 & 0xf800) >> 11) * recip) + int (((pixel2 & 0xf800) >> 11) * weight) )
				, 0x1f ) );
			unsigned int g = max (0, min (
					( int (((pixel1 & 0x7e0) >> 5) * recip) + int (((pixel2 & 0x7e0) >> 5) * weight) )
				, 0x3f ) );
			unsigned int b = max (0, min (
					( int (((pixel1 & 0x1f)) * recip) + int (((pixel2 & 0x1f)) * weight) )
				, 0x1f ) );
			
			return Uint16 ((r << 11) | (g << 5) | (b));
		}
		static Uint16 pixelAverageRGB (const int& numPixels, const Uint16* pixels){
			unsigned int rMask = 0xf800;
			unsigned int gMask = 0x7e0;
			unsigned int bMask = 0x1f;

			unsigned int rSum = 0; unsigned int gSum = 0; unsigned int bSum = 0;
			int index;
			for (index = 0; index < numPixels; index++){
				rSum += (pixels [index] & rMask) >> 11;
				gSum += (pixels [index] & gMask) >> 5;
				bSum += (pixels [index] & bMask);
			}

			rSum = min ((unsigned int) 0xff, (rSum / numPixels) & 0xff);
			gSum = min ((unsigned int) 0xff, (gSum / numPixels) & 0xff);
			bSum = min ((unsigned int) 0xff, (bSum / numPixels) & 0xff);
		
			return (rSum << 11) | (gSum << 5) | (bSum);
		}
		// TODO: uh, what is the pixel format for 8-bit color? some kind of palette or something? can u blend 8-bit colors? (is it a palette?)
		/*
		static Uint8 linearPixelBlend (const Uint8& pixel1, const Uint8& pixel2, const double& weight){
			double recip = 1 - weight;
			unsigned int r = ( int (((pixel1 & 0xff0000) >> 16) * recip) + int (((pixel2 & 0xff0000) >> 16) * weight) ) / 2;
			unsigned int g = ( int (((pixel1 & 0x00ff00) >> 8) * recip) + int (((pixel2 & 0x00ff00) >> 8) * weight) ) / 2;
			unsigned int b = ( int (((pixel1 & 0x0000ff)) * recip) + int (((pixel2 & 0x0000ff)) * weight) ) / 2;
		
			return Uint16((r << 16) | (g << 8) | (b));
		}
		*/
		static Uint32 getColorKey (const SDL_Surface* surface){
			return surface->format->colorkey;
		}
		// Taken from SDLlib.org .
		static void setPixel (SDL_Surface* surface, const int& x, const int& y, const Uint32& value){
			if (SDL_MUSTLOCK (surface)) SDL_LockSurface (surface);
	    	int bpp = surface->format->BytesPerPixel;
	    	/* Here p is the address to the pixel we want to retrieve */
		    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    		switch(bpp) {
		    	case 1:
		        	*p = value; 
					if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);
					return;

			    case 2:
		    	    *(Uint16 *)p = value; 
					if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);
					return;

			    case 3:
        			if(SDL_BYTEORDER == SDL_BIG_ENDIAN){
		    	        p [0] = (value >> 16) & 0xff; p [1] = (value >> 8) & 0xff; p [2] = value & 0xff;
					}
			        else{
						p [0] = value & 0xff; p [1] = (value >> 8) & 0xff; p [2] = (value >> 16) & 0xff;
					}

					if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);
					return;

			    case 4:
    	    		*(Uint32 *)p = value; 
					if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);
					return;

			    default:
			        printf ("Unexpected bpp in setPixel function!\n");
					if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);
		    	    return;       /* shouldn't happen, but avoids warnings */
		    }
		}
		
		// These ASSUME 16-bit surfaces! !!!
		static Uint32 getPixel (const SDL_Surface* surface, const int& x, const int& y){
			return *((Uint16 *) ((Uint8 *) surface->pixels + y * surface->pitch + x * 2));
		}
		
		static Uint16* getPixelRow (const SDL_Surface* surface, const int& y){
			return ((Uint16*) surface->pixels) + y * surface->pitch;
		}

		/*
		// Taken from SDLlib.org .
		static Uint32 getPixel (SDL_Surface* surface, const int& x, const int& y)
		{
			//if (SDL_MUSTLOCK (surface)) SDL_LockSurface (surface);

		    int bpp = surface->format->BytesPerPixel;
    		// Here p is the address to the pixel we want to retrieve 
		    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
			Uint32 pixel;

    		switch(bpp) {
		    	case 1:
					pixel = *p;
					break;

			    case 2:
					//cout << "Getting pixel at " << (Uint16 *)p << endl;
					pixel = *(Uint16 *)p;
					break;

			    case 3:
    	    		if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			            pixel = p[0] << 16 | p[1] << 8 | p[2];
		    	    else
        			    pixel = p[0] | p[1] << 8 | p[2] << 16;
					break;

			    case 4:
        			pixel = *(Uint32 *)p;
					break;

			    default:
			        printf ("Unexpected bpp in getPixel function!\n");
					break;
		    }

			//if (SDL_MUSTLOCK (surface)) SDL_UnlockSurface (surface);

			return pixel;
		}
		*/
	
		static Point getNormalized2dVector (const Point& toNormalize){
			if (toNormalize.getX() == 0 && toNormalize.getY() == 0) return toNormalize;
			double x = toNormalize.getX(); double y = toNormalize.getY();
			double length = sqrt (x * x + y * y);
			return Point (x / length, y / length);
		}
		static double get2dDistance (const Point& from, const Point& to){
			double x = to.getX() - from.getX();
			double y = to.getY() - from.getY();
			return sqrt (x * x + y * y);
		}
		static double absFloat (double value){
			if (value < 0) return value * -1;
			return value;
		}
		static double maxFloat (double x, double y){
			return x > y ? x : y;
		}
		static int compareDoubles (double a, double b, double tolerance = 0.000001){
			double diff = a - b;
			if (fabs (diff) < tolerance) return 0;
			if (diff > 0) return -1;
			return 1;
		}
		static void rasterizeLine (std::list <Point>& runningList, const Line& line);
};


#endif
